/***********************************************************
* Project:	CS298                                          *
* File:		Segment1.cpp                                   *
* Purpose:	This file contains member functions that are   *
*			used in the painter's algorithm.               *
************************************************************/

#define QSORT
#ifdef QSORT
#include <stdlib.h>
#include <search.h>
#endif //QSORT
#include "segment1.hpp"
#include <assert.h>

Segment::Segment(int maxcount)
{
	_polygonlist = new Poly3*[maxcount];
	assert(_polygonlist);
	for (int i=0; i<maxcount; i++)
		_polygonlist[i] = NULL;
	_maxcount = maxcount;
	_freeindex = 0;
}

Segment::~Segment()
{
	Empty();
	delete [] _polygonlist;
}

void Segment::Empty()
{
	for (int i=0; i< _freeindex; i++)
		if (_polygonlist[i])
		{
			delete _polygonlist[i];
			_polygonlist[i] = NULL;
		}
	_freeindex = 0;
}

void Segment::Post(const Poly3 &p)
{
//	assert(_freeindex < _maxcount);
/* I could just do this:
	_polygonlist[_freeindex] = new Poly3(p);
But that would be a waste of time, copying over the
_vector3vert and _vector2vert arrrays.  All I really
need to copy is the sorting and drawing info.  So I
make a special Poly3::SegmentCopy method to do just that*/
	_polygonlist[_freeindex] = new Poly3();
     _polygonlist[_freeindex]->SegmentCopy(p);
	_freeindex++;
}

void Segment::Post(const Polyhedron3 &p)
{
	for (int faceindex =0; faceindex<p._facecount; faceindex++)
		if (p._facevisible[faceindex])
		{
     		_polygonlist[_freeindex] = new Poly3();
     		_polygonlist[_freeindex]->SegmentCopy(p, faceindex);
			_freeindex++;
		}
}

#define SMALLEST_Z -100.0
#define LARGEST_Z 100.0

void Segment::RoomPost(const Polyhedron3 &p)
{
	for (int faceindex =0; faceindex<p._facecount; faceindex++)
	{
		if (p._facedistance[faceindex] < 0.0) // it's behind you
			continue; //don't show it.
         _polygonlist[_freeindex] = new Poly3();
         _polygonlist[_freeindex]->SegmentCopy(p, faceindex);
		if (p._facevisible[faceindex]) //Draw it first
			_polygonlist[_freeindex]->_centroid.Set(0.0, 0.0, SMALLEST_Z);
		else //Draw it last and draw it transparent.
		{
			_polygonlist[_freeindex]->_centroid.Set(0.0, 0.0, LARGEST_Z);
			_polygonlist[_freeindex]->SetTransparent(TRUE);
		}
	    _freeindex++;
	}
}

#ifndef QSORT
void Segment::Sort()
{ /*List the furthest away guys first, and the nearest guys last.  This
is the painter's algorithm preparation for Segment::SRGP_Draw.  Things
are a little changed now because I have fixed it so with polyhedra,
the _centroid.z() is equal to the negative distance between _eye and
the REAL centroid.  So there I still want to do the smallest z first
and the biggest z last.*/
	Poly3 *temp;
	for (int i=0; i<_freeindex; i++)
		for (int j=i+1; j<_freeindex; j++)
			if (_polygonlist[j]->_centroid.z() <
				_polygonlist[i]->_centroid.z())
			{
				temp = _polygonlist[i];
				_polygonlist[i] = _polygonlist[j];
				_polygonlist[j] = temp;
			}
}
#else  //QSORT
/*  This worked once in a Fall 94 CS 151 DOS program with code like this:

int Imagepair_sort(const void *r1, const void *r2)
{
	if (((Imagepair *)r1)->vpixel.hitcount <
		((Imagepair *)r2)->vpixel.hitcount)
		return -1;
	if (((Imagepair *)r1)->vpixel.hitcount >
		((Imagepair *)r2)->vpixel.hitcount)
		return 1;
	return 0; //otherwise
}
void Julia::Sort()
{
	qsort(flockimage, degree*flockcount, sizeof(Imagepair), Imagepair_sort);
}
Here of course this situation is different as I have an array of pointers
rather than an array of objects. So I have to do this differently...
*/
	
/*extern "C" int fcmp(const void *a, const void *b) //NO GOOD
{//First try, compiler doesn't get it, doesn't believe pointer is to a Poly3.
 	if ( *(Poly3**)a->Centroid().z() <  *(Poly3**)b->Centroid().z() )
		return -1;
	if ( *(Poly3**)a->Centroid().z() == ( *(Poly3**)b->Centroid().z() )
		return 0;
	return 1;
}
*/

extern "C" int fcmp(const void *a, const void *b) //Second try, THIS WORKS
{	// Explicity cast them to what they really are, one step at a time.
	Poly3** pa = (Poly3**)a;
	Poly3** pb = (Poly3**)b;
	Poly3* ppa = *pa;
	Poly3* ppb = *pb;
	double ca = ppa->Centroid().z();
	double cb = ppb->Centroid().z();
	double diff = ca - cb;
	return (diff > 0) - (diff < 0);
}

void Segment::Sort()
{
	qsort((void *)_polygonlist, _freeindex, sizeof(Poly3 *), fcmp);
}

#endif //QSORT

void Segment::Draw(GDIToolbox &gdibox)
{ //Draw the furthest first and then the nearest: painter's algorithm.
	for (int i=0; i<_freeindex; i++)
		_polygonlist[i]->Draw(gdibox);
}


