/***********************************************************
* Project:	CS298                                          *
* File:		shapes3.cpp                                    *
* Purpose:	This file holds information about  3D and 4D   *
*			objects.                                       *
************************************************************/

#include "polyhedron4.hpp"
#include "random.h" //To randomize colors
#include <math.h> //For trig functions.


//================POLYHEDRA in 3D
Polyhedron3::Polyhedron3():_vector3vert(NULL),_vector2vert(NULL),
	_pointvert(NULL),_centroid(0.0,0.0,0.0),
	_radius(0.0), _facecount(0),
     _facevertcount(NULL),_face(NULL),_fillcolor(NULL), _facenormal(NULL),
	_facevisible(NULL), _facecentroid(NULL), _facedistance(NULL),
	_edgecount(0),_vertedgecount(NULL),_vertedge(NULL), _internalvert(NULL), 
	BarycentricCoord(NULL), _internalface(NULL)

{}


Polyhedron3::Polyhedron3(const Polyhedron3 &poly):_vector3vert(NULL),
	_vector2vert(NULL),
	_pointvert(NULL),_centroid(0.0,0.0,0.0),
	_radius(0.0), _facecount(0),
     _facevertcount(NULL),_face(NULL),_fillcolor(NULL), _facenormal(NULL),
	_facevisible(NULL), _facecentroid(NULL), _facedistance(NULL),
	_edgecount(0),_vertedgecount(NULL),_vertedge(NULL), _internalvert(NULL),
	BarycentricCoord(NULL),_internalface(NULL)

{
	Size(poly._count, poly._facecount, poly._facevertcount,poly._edgecount,
		poly._vertedgecount);

	for (int i=0; i<_count; i++)
	{
		_vector3vert[i] = poly._vector3vert[i];
		_vector2vert[i] = poly._vector2vert[i];
		_pointvert[i] = poly._pointvert[i];
	}
	for ( i=0; i<_edgecount; i++)
		_internalvert[i] = poly._internalvert[i];


	_centroid = poly._centroid;
	_radius = poly._radius;
	for (i=0; i<_facecount; i++)
	{
		for (int j=0; j<_facevertcount[i]; j++)
			_face[i][j] = poly._face[i][j];
		_fillcolor[i] = poly._fillcolor[i];
		_facenormal[i] = poly._facenormal[i];
		_facecentroid[i] = poly._facecentroid[i];
		_facevisible[i] = poly._facevisible[i];
		_facedistance[i] = poly._facedistance[i];
		_internalface[i] = poly._internalface[i];
	}

	for( i=0;i<_edgecount;i++)
	{
		for( int j=0;j<_vertedgecount[i]; j++)
			_vertedge[i][j]=poly._vertedge[i][j];
	}
	for(i=0;i<_count;i++)
	{
		for(int j=0;j<4;j++)//4 or number of edges coming out of V
			BarycentricCoord[i][j]=poly.BarycentricCoord[i][j];
	}

}

Polyhedron3::~Polyhedron3()
{
	_freememory();
}

void Polyhedron3::_setcentroid()
{
	Real distance;

	_centroid = Vector3(0.0, 0.0, 0.0);
	_radius = 0.0;
     for (int i=0; i<Count(); i++)
     	_centroid += _vector3vert[i];
     _centroid /= Count();
	for (i=0; i<Count(); i++)
	{
		distance = (_vector3vert[i] - _centroid).Magnitude();
		if (distance > _radius)
			_radius = distance;
	}
}


void Polyhedron3::_freememory()
{
	if (_vector3vert)
		{delete [] _vector3vert; _vector3vert = NULL;}
	if (_vector2vert)
		{delete [] _vector2vert; _vector2vert = NULL;}
	if (_pointvert)
		{delete [] _pointvert; _pointvert = NULL;}
	if (_facevertcount)
		{delete [] _facevertcount; _facevertcount = NULL;}
	if (_face)
		for (int i=0; i< _facecount; i++)
			if (_face[i])
				{delete [] _face[i]; _face[i] = NULL;}
	if (_face)
		{delete [] _face; _face = NULL;}
	if (_fillcolor)
		{delete [] _fillcolor; _fillcolor = NULL;}
	if (_facenormal)
		{delete [] _facenormal; _facenormal = NULL;}
	if (_facecentroid)
		{delete [] _facecentroid; _facecentroid = NULL;}
	if (_facevisible)
		{delete [] _facevisible; _facevisible = NULL;}
	if (_facedistance)
		{delete [] _facedistance; _facedistance = NULL;}

	if (_internalvert)
		{delete [] _internalvert; _internalvert = NULL;}
	if (_internalface)
		{delete [] _internalface; _internalface = NULL;}

	if (_vertedgecount)
		{delete [] _vertedgecount; _vertedgecount = NULL;}
	if (_vertedge)
		for (int i=0; i< _edgecount; i++)
			if (_vertedge[i])
				{delete [] _vertedge[i]; _vertedge[i] = NULL;}
	if (_vertedge)
		{delete [] _vertedge; _vertedge = NULL;}
	if (BarycentricCoord)
		for (int i=0; i< _count; i++)
			if (BarycentricCoord[i])
				{delete [] BarycentricCoord[i]; BarycentricCoord[i] = NULL;}
	if (BarycentricCoord)
		{delete [] BarycentricCoord; BarycentricCoord = NULL;}

}

void Polyhedron3::SetBarycentricCoords()
{
	Real det;
	
	det=Determinant(_vector3vert[1],_vector3vert[3],_vector3vert[8],_vector3vert[5]);
	if(det == 0)
		for(int i=0;i<4;i++)
			BarycentricCoord[0][i]=-1;
	else
	{
		BarycentricCoord[0][0]=Determinant(_vector3vert[0],_vector3vert[3],_vector3vert[8],_vector3vert[5])/det;
		BarycentricCoord[0][1]=Determinant(_vector3vert[1],_vector3vert[0],_vector3vert[8],_vector3vert[5])/det;
		BarycentricCoord[0][2]=Determinant(_vector3vert[1],_vector3vert[3],_vector3vert[0],_vector3vert[5])/det;
		BarycentricCoord[0][3]=Determinant(_vector3vert[1],_vector3vert[3],_vector3vert[8],_vector3vert[0])/det;
	}

	det=Determinant(_vector3vert[2],_vector3vert[0],_vector3vert[6],_vector3vert[9]);
	if(det == 0)
		for(int i=0;i<4;i++)
			BarycentricCoord[1][i]=-1;
	else
	{
		BarycentricCoord[1][0]=Determinant(_vector3vert[1],_vector3vert[0],_vector3vert[6],_vector3vert[9])/det;
		BarycentricCoord[1][1]=Determinant(_vector3vert[2],_vector3vert[1],_vector3vert[6],_vector3vert[9])/det;
		BarycentricCoord[1][2]=Determinant(_vector3vert[2],_vector3vert[0],_vector3vert[1],_vector3vert[9])/det;
		BarycentricCoord[1][3]=Determinant(_vector3vert[2],_vector3vert[0],_vector3vert[6],_vector3vert[1])/det;
	}

	det=Determinant(_vector3vert[3],_vector3vert[7],_vector3vert[1],_vector3vert[10]);
	if(det == 0)
		for(int i=0;i<4;i++)
			BarycentricCoord[2][i]=-1;
	else
	{
		BarycentricCoord[2][0]=Determinant(_vector3vert[2],_vector3vert[7],_vector3vert[1],_vector3vert[10])/det;
		BarycentricCoord[2][1]=Determinant(_vector3vert[3],_vector3vert[2],_vector3vert[1],_vector3vert[10])/det;
		BarycentricCoord[2][2]=Determinant(_vector3vert[3],_vector3vert[7],_vector3vert[2],_vector3vert[10])/det;
		BarycentricCoord[2][3]=Determinant(_vector3vert[3],_vector3vert[7],_vector3vert[1],_vector3vert[2])/det;
	}

	det=Determinant(_vector3vert[4],_vector3vert[0],_vector3vert[2],_vector3vert[11]);
	if(det == 0)
		for(int i=0;i<4;i++)
			BarycentricCoord[3][i]=-1;
	else
	{
		BarycentricCoord[3][0]=Determinant(_vector3vert[3],_vector3vert[0],_vector3vert[2],_vector3vert[11])/det;
		BarycentricCoord[3][1]=Determinant(_vector3vert[4],_vector3vert[3],_vector3vert[2],_vector3vert[11])/det;
		BarycentricCoord[3][2]=Determinant(_vector3vert[4],_vector3vert[0],_vector3vert[3],_vector3vert[11])/det;
		BarycentricCoord[3][3]=Determinant(_vector3vert[4],_vector3vert[0],_vector3vert[2],_vector3vert[3])/det;
	}

	det=Determinant(_vector3vert[5],_vector3vert[7],_vector3vert[3],_vector3vert[12]);
	if(det == 0)
		for(int i=0;i<4;i++)
			BarycentricCoord[4][i]=-1;
	else
	{
		BarycentricCoord[4][0]=Determinant(_vector3vert[4],_vector3vert[7],_vector3vert[3],_vector3vert[12])/det;
		BarycentricCoord[4][1]=Determinant(_vector3vert[5],_vector3vert[4],_vector3vert[3],_vector3vert[12])/det;
		BarycentricCoord[4][2]=Determinant(_vector3vert[5],_vector3vert[7],_vector3vert[4],_vector3vert[12])/det;
		BarycentricCoord[4][3]=Determinant(_vector3vert[5],_vector3vert[7],_vector3vert[3],_vector3vert[4])/det;
	}
	
	det=Determinant(_vector3vert[6],_vector3vert[0],_vector3vert[4],_vector3vert[13]);
	if(det == 0)
		for(int i=0;i<4;i++)
			BarycentricCoord[5][i]=-1;
	else
	{
		BarycentricCoord[5][0]=Determinant(_vector3vert[5],_vector3vert[0],_vector3vert[4],_vector3vert[13])/det;
		BarycentricCoord[5][1]=Determinant(_vector3vert[6],_vector3vert[5],_vector3vert[4],_vector3vert[13])/det;
		BarycentricCoord[5][2]=Determinant(_vector3vert[6],_vector3vert[0],_vector3vert[5],_vector3vert[13])/det;
		BarycentricCoord[5][3]=Determinant(_vector3vert[6],_vector3vert[0],_vector3vert[4],_vector3vert[5])/det;
	}
	
	det=Determinant(_vector3vert[7],_vector3vert[5],_vector3vert[14],_vector3vert[1]);
	if(det == 0)
		for(int i=0;i<4;i++)
			BarycentricCoord[6][i]=-1;
	else
	{
		BarycentricCoord[6][0]=Determinant(_vector3vert[6],_vector3vert[5],_vector3vert[14],_vector3vert[1])/det;
		BarycentricCoord[6][1]=Determinant(_vector3vert[7],_vector3vert[6],_vector3vert[14],_vector3vert[1])/det;
		BarycentricCoord[6][2]=Determinant(_vector3vert[7],_vector3vert[5],_vector3vert[6],_vector3vert[1])/det;
		BarycentricCoord[6][3]=Determinant(_vector3vert[7],_vector3vert[5],_vector3vert[14],_vector3vert[6])/det;
	}

	
	det=Determinant(_vector3vert[2],_vector3vert[4],_vector3vert[6],_vector3vert[15]);
	if(det == 0)
		for(int i=0;i<4;i++)
			BarycentricCoord[7][i]=-1;
	else
	{
		BarycentricCoord[7][0]=Determinant(_vector3vert[7],_vector3vert[4],_vector3vert[6],_vector3vert[15])/det;
		BarycentricCoord[7][1]=Determinant(_vector3vert[2],_vector3vert[7],_vector3vert[6],_vector3vert[15])/det;
		BarycentricCoord[7][2]=Determinant(_vector3vert[2],_vector3vert[4],_vector3vert[7],_vector3vert[15])/det;
		BarycentricCoord[7][3]=Determinant(_vector3vert[2],_vector3vert[4],_vector3vert[6],_vector3vert[7])/det;
	}


	det=Determinant(_vector3vert[9],_vector3vert[0],_vector3vert[11],_vector3vert[13]);
	if(det == 0)
		for(int i=0;i<4;i++)
			BarycentricCoord[8][i]=-1;
	else
	{
		BarycentricCoord[8][0]=Determinant(_vector3vert[8],_vector3vert[0],_vector3vert[11],_vector3vert[13])/det;
		BarycentricCoord[8][1]=Determinant(_vector3vert[9],_vector3vert[8],_vector3vert[11],_vector3vert[13])/det;
		BarycentricCoord[8][2]=Determinant(_vector3vert[9],_vector3vert[0],_vector3vert[8],_vector3vert[13])/det;
		BarycentricCoord[8][3]=Determinant(_vector3vert[9],_vector3vert[0],_vector3vert[11],_vector3vert[8])/det;
	}


	det=Determinant(_vector3vert[10],_vector3vert[1],_vector3vert[8],_vector3vert[14]);
	if(det == 0)
		for(int i=0;i<4;i++)
			BarycentricCoord[9][i]=-1;
	else
	{
		BarycentricCoord[9][0]=Determinant(_vector3vert[9],_vector3vert[1],_vector3vert[8],_vector3vert[14])/det;
		BarycentricCoord[9][1]=Determinant(_vector3vert[10],_vector3vert[9],_vector3vert[8],_vector3vert[14])/det;
		BarycentricCoord[9][2]=Determinant(_vector3vert[10],_vector3vert[1],_vector3vert[9],_vector3vert[14])/det;
		BarycentricCoord[9][3]=Determinant(_vector3vert[10],_vector3vert[1],_vector3vert[8],_vector3vert[9])/det;
	}


	det=Determinant(_vector3vert[11],_vector3vert[2],_vector3vert[15],_vector3vert[9]);
	if(det == 0)
		for(int i=0;i<4;i++)
			BarycentricCoord[10][i]=-1;
	else
	{
		BarycentricCoord[10][0]=Determinant(_vector3vert[10],_vector3vert[2],_vector3vert[15],_vector3vert[9])/det;
		BarycentricCoord[10][1]=Determinant(_vector3vert[11],_vector3vert[10],_vector3vert[15],_vector3vert[9])/det;
		BarycentricCoord[10][2]=Determinant(_vector3vert[11],_vector3vert[2],_vector3vert[10],_vector3vert[9])/det;
		BarycentricCoord[10][3]=Determinant(_vector3vert[11],_vector3vert[2],_vector3vert[15],_vector3vert[10])/det;
	}


	det=Determinant(_vector3vert[12],_vector3vert[8],_vector3vert[3],_vector3vert[10]);
	if(det == 0)
		for(int i=0;i<4;i++)
			BarycentricCoord[11][i]=-1;
	else
	{
		BarycentricCoord[11][0]=Determinant(_vector3vert[11],_vector3vert[8],_vector3vert[3],_vector3vert[10])/det;
		BarycentricCoord[11][1]=Determinant(_vector3vert[12],_vector3vert[11],_vector3vert[3],_vector3vert[10])/det;
		BarycentricCoord[11][2]=Determinant(_vector3vert[12],_vector3vert[8],_vector3vert[11],_vector3vert[10])/det;
		BarycentricCoord[11][3]=Determinant(_vector3vert[12],_vector3vert[8],_vector3vert[3],_vector3vert[11])/det;
	}

	det=Determinant(_vector3vert[13],_vector3vert[4],_vector3vert[15],_vector3vert[11]);
	if(det == 0)
		for(int i=0;i<4;i++)
			BarycentricCoord[12][i]=-1;
	else
	{
		BarycentricCoord[12][0]=Determinant(_vector3vert[12],_vector3vert[4],_vector3vert[15],_vector3vert[11])/det;
		BarycentricCoord[12][1]=Determinant(_vector3vert[13],_vector3vert[12],_vector3vert[15],_vector3vert[11])/det;
		BarycentricCoord[12][2]=Determinant(_vector3vert[13],_vector3vert[4],_vector3vert[12],_vector3vert[11])/det;
		BarycentricCoord[12][3]=Determinant(_vector3vert[13],_vector3vert[4],_vector3vert[15],_vector3vert[12])/det;
	}

	det=Determinant(_vector3vert[14],_vector3vert[5],_vector3vert[8],_vector3vert[12]);
	if(det == 0)
		for(int i=0;i<4;i++)
			BarycentricCoord[13][i]=-1;
	else
	{
		BarycentricCoord[13][0]=Determinant(_vector3vert[13],_vector3vert[5],_vector3vert[8],_vector3vert[12])/det;
		BarycentricCoord[13][1]=Determinant(_vector3vert[14],_vector3vert[13],_vector3vert[8],_vector3vert[12])/det;
		BarycentricCoord[13][2]=Determinant(_vector3vert[14],_vector3vert[5],_vector3vert[13],_vector3vert[12])/det;
		BarycentricCoord[13][3]=Determinant(_vector3vert[14],_vector3vert[5],_vector3vert[8],_vector3vert[13])/det;
	}

	det=Determinant(_vector3vert[15],_vector3vert[6],_vector3vert[13],_vector3vert[9]);
	if(det == 0)
		for(int i=0;i<4;i++)
			BarycentricCoord[14][i]=-1;
	else
	{
		BarycentricCoord[14][0]=Determinant(_vector3vert[14],_vector3vert[6],_vector3vert[13],_vector3vert[9])/det;
		BarycentricCoord[14][1]=Determinant(_vector3vert[15],_vector3vert[14],_vector3vert[13],_vector3vert[9])/det;
		BarycentricCoord[14][2]=Determinant(_vector3vert[15],_vector3vert[6],_vector3vert[14],_vector3vert[9])/det;
		BarycentricCoord[14][3]=Determinant(_vector3vert[15],_vector3vert[6],_vector3vert[13],_vector3vert[14])/det;
	}

	det=Determinant(_vector3vert[12],_vector3vert[7],_vector3vert[14],_vector3vert[10]);
	if(det == 0)
		for(int i=0;i<4;i++)
			BarycentricCoord[15][i]=-1;
	else
	{
		BarycentricCoord[15][0]=Determinant(_vector3vert[15],_vector3vert[7],_vector3vert[14],_vector3vert[10])/det;
		BarycentricCoord[15][1]=Determinant(_vector3vert[12],_vector3vert[15],_vector3vert[14],_vector3vert[10])/det;
		BarycentricCoord[15][2]=Determinant(_vector3vert[12],_vector3vert[7],_vector3vert[15],_vector3vert[10])/det;
		BarycentricCoord[15][3]=Determinant(_vector3vert[12],_vector3vert[7],_vector3vert[14],_vector3vert[15])/det;
	}

}

// checking whether BarycentricCoords are in [0,1]
void Polyhedron3::SetVertInHull()
{	
	SetBarycentricCoords();

	BOOL internal;
	for(int j=0; j<_count; j++)
	{
		internal=TRUE;
		for(int i=0; i<4;i++)
		{//if at least one Barycentric coordinate is less than zero of greater than one
		 //the point is outside, and not in the hull.
	/* If we make the inequalities >= and <= then we take the borderline case as 
	non internal or visible. */
			if(BarycentricCoord[j][i] <= 0 || BarycentricCoord[j][i]>=1)
			{												
				internal=FALSE;
				break;
			}
		}
		_internalvert[j]=internal;
	}
	return;
}


void Polyhedron3::SetInternalFace()
{
	for(int i=0; i<_facecount; i++)
	{
		BOOL internal=FALSE;
		for(int j=0;j<_facevertcount[i];j++)
		{
			if(_internalvert[_face[i][j]])
			{
				internal=TRUE;
				break;
			}
		}
	_internalface[i]=internal;
	}
	return;
}

Polyhedron3 Polyhedron3::operator=(const Polyhedron3 &poly)
{
 	if (this == &poly)
		return *this;
	Size(poly.Count(), poly._facecount, poly._facevertcount, poly._edgecount,
		poly._vertedgecount);
	for (int i=0; i<_count; i++)
	{
		_vector3vert[i] = poly._vector3vert[i];
		_vector2vert[i] = poly._vector2vert[i];
		_pointvert[i] = poly._pointvert[i];
	}

	for ( i=0; i<_edgecount; i++)
		_internalvert[i] = poly._internalvert[i];


	_centroid = poly._centroid;
	_radius = poly._radius;
	for (i=0; i<_facecount; i++)
	{
		for (int j=0; j<_facevertcount[i]; j++)
			_face[i][j] = poly._face[i][j];
		_fillcolor[i] = poly._fillcolor[i];
		_facenormal[i] = poly._facenormal[i];
		_facecentroid[i] = poly._facecentroid[i];
		_facevisible[i] = poly._facevisible[i];
		_facedistance[i] = poly._facedistance[i];
		_internalface[i] = poly._internalface[i];	
	}

	for( i=0;i<_edgecount;i++)
	{
		for(int j=0;j<_vertedgecount[i]; j++)
			_vertedge[i][j]=poly._vertedge[i][j];
	}
	for( i=0;i<_count;i++)
	{
		for(int j=0;j<4; j++)
			BarycentricCoord[i][j]=poly.BarycentricCoord[i][j];
	}

	return *this;
}

void Polyhedron3::Size(int newcount, int newfacecount,
	int *newfacevertcount, int newedgecount, int *newvertedgecount)
{
 	_freememory();//Empty the polyhedron out first.
	_count = newcount;
	_vector3vert = new Vector3[_count];
	assert(_vector3vert);
	_vector2vert = new Vector2[_count];
	assert(_vector2vert);
	_pointvert = new Point[_count];
	assert(_pointvert);
	_facecount = newfacecount;
	_facevertcount = new int[_facecount];
	assert(_facevertcount);
	_face = new int*[_facecount];
	assert(_face);
	_fillcolor = new COLORREF[_facecount];
	assert(_fillcolor);
	_facenormal = new Vector3[_facecount];
	assert(_facenormal);
	_facecentroid = new Vector3[_facecount];
	assert(_facecentroid);
	_facevisible = new BOOL[_facecount];
	assert(_facevisible);
	_facedistance = new Real[_facecount];
	assert(_facedistance);
	for (int i=0; i<_facecount; i++)
	{
		_facevertcount[i] = newfacevertcount[i];
		int temp = _facevertcount[i];
		_face[i] = new int[temp];
		assert(_face[i]);
	}
	
	_edgecount=newedgecount;
	_vertedgecount= new int[_edgecount];
	assert(_vertedgecount);
	_vertedge=new Vector3*[_edgecount];
	assert(_vertedge);
	BarycentricCoord=new Real*[_count];
	assert(BarycentricCoord);

	for( i=0;i<_edgecount;i++)
	{
		_vertedgecount[i]=newvertedgecount[i];
		int temp = _vertedgecount[i];
		_vertedge[i]=new Vector3[temp];
		assert(_vertedge[i]);
	}
	for( i=0;i<_count;i++)
	{
		int temp = 4;
		BarycentricCoord[i]=new Real[temp];
		assert(BarycentricCoord[i]);
	}

	_internalvert = new BOOL[_edgecount];
	assert(_internalvert);
	_internalface = new BOOL[_facecount];
	assert(_internalface);

}


void Polyhedron3::_setfacecentroid()
{
     for (int i=0; i<_facecount; i++)
	{
		_facecentroid[i] = Vector3(0.0, 0.0, 0.0);
	     for (int j=0; j<_facevertcount[i]; j++)
     		_facecentroid[i] += _vector3vert[_face[i][j]];
	     _facecentroid[i] /= _facevertcount[i];
	}
}

void Polyhedron3::_setfacenormal()
{
     for (int i=0; i<_facecount; i++)
 		_facenormal[i] =
    			(  (_vector3vert[_face[i][1]] -  _vector3vert[_face[i][0]]) *
    			   (_vector3vert[_face[i][2]] - _vector3vert[_face[i][1]])
			).Normalize();
}

// Note that it is important to label the vertices of each face in
// counterclockwise order (looking down at the face from the
// outside) so that the cross product (V1 - V0) x (V2 - V1)
// gives a normal direction pointing out.
// The coordinates for the dodecahedron and icosahedron are lifted from
// "Jim Blinn's Corner," IEEE Computer Graphics & Applications, November,
// 1987, pp. 62 - 66.
// Adjust tope so that distance from center to a vertex ( with
// Platonic and Archimedean topes all vertices are at same distance
// from center, this is not true with the duals ) is radius.

void Polyhedron3::Cube(Real radius)
{
	int newfacevertcount[6] = {4,4,4,4,4,4};
	Size(8, 6, newfacevertcount, 0, NULL);
	Real halfedge = radius / sqrt(3.0); //radius = sqrt(3 * edge^2).

	_vector3vert[0] = Vector3(halfedge, halfedge, halfedge);
	_vector3vert[1] = Vector3(halfedge, halfedge, -halfedge);
	_vector3vert[2] = Vector3(-halfedge, halfedge, -halfedge);
	_vector3vert[3] = Vector3(-halfedge, halfedge, halfedge);
	_vector3vert[4] = Vector3(halfedge, -halfedge, halfedge);
	_vector3vert[5] = Vector3(halfedge, -halfedge, -halfedge);
	_vector3vert[6] = Vector3(-halfedge, -halfedge, -halfedge);
	_vector3vert[7] = Vector3(-halfedge, -halfedge, halfedge);

	_face[0][0] = 0;
	_face[0][1] = 1;
	_face[0][2] = 2;
	_face[0][3] = 3;

	_face[1][0] = 0;
	_face[1][1] = 4;
	_face[1][2] = 5;
	_face[1][3] = 1;

	_face[2][0] = 0;
	_face[2][1] = 3;
	_face[2][2] = 7;
	_face[2][3] = 4;

	_face[3][0] = 6;
	_face[3][1] = 5;
	_face[3][2] = 4;
	_face[3][3] = 7;

	_face[4][0] = 6;
	_face[4][1] = 2;
	_face[4][2] = 1;
	_face[4][3] = 5;

	_face[5][0] = 6;
	_face[5][1] = 7;
	_face[5][2] = 3;
	_face[5][3] = 2;

	for (int i=0; i< 6; i++)
	{
		_facevisible[i] = TRUE;
		_fillcolor[i] = RGB(128+Random(128),
					Random(2)?256:Random(256),
					Random(2)?256:Random(256));
		_facedistance[i] = 0.0;
	}
	_setfacecentroid();
	_setfacenormal();
	_centroid = Vector3(0.0, 0.0, 0.0);
	_radius = radius;
}

void Polyhedron3::Dodecahedron(Real radius)
{
	static newfacecount[12] = {5,5,5,5,5,5,5,5,5,5,5,5,};
	static Real phi = ( 1.0 + sqrt(5.0) ) / 2.0;	// The golden ratio.
	static Real phi1 = 2.0 / ( 1.0 + sqrt(5.0) ); // Reciprocal of phi.
	Real rad;

	Size(20,12,newfacecount, 0, NULL);
	rad = ( 1.0 / sqrt(3.0) ) * radius;

	_vector3vert[0] =  Vector3(0.,-rad*phi1,-rad*phi);
	_vector3vert[1] =  Vector3(rad,rad,rad);
	_vector3vert[2] =  Vector3(rad,rad,-rad);
	_vector3vert[3] =  Vector3(rad,-rad,rad);
	_vector3vert[4] =  Vector3(rad,-rad,-rad);
	_vector3vert[5] =  Vector3(-rad,rad,rad);
	_vector3vert[6] =  Vector3(-rad,rad,-rad);
	_vector3vert[7] =  Vector3(-rad,-rad,rad);
	_vector3vert[8] =  Vector3(-rad,-rad,-rad);
	_vector3vert[9] =  Vector3(rad*phi1,rad*phi,0.);
	_vector3vert[10] = Vector3(-rad*phi1,rad*phi,0.);
	_vector3vert[11] = Vector3(rad*phi1,-rad*phi,0.);
	_vector3vert[12] = Vector3(-rad*phi1,-rad*phi,0.);
	_vector3vert[13] = Vector3(rad*phi,0.,rad*phi1);
	_vector3vert[14] = Vector3(rad*phi,0.,-rad*phi1);
	_vector3vert[15] = Vector3(-rad*phi,0.,rad*phi1);
	_vector3vert[16] = Vector3(-rad*phi,0.,-rad*phi1);
	_vector3vert[17] = Vector3(0.,rad*phi1,rad*phi);
	_vector3vert[18] = Vector3(0.,-rad*phi1,rad*phi);
	_vector3vert[19] = Vector3(0.,rad*phi1,-rad*phi);

	_face[0][0] =  2;
	_face[0][1] =  9;
	_face[0][2] =  1;
	_face[0][3] = 13;
	_face[0][4] = 14;

	_face[1][0] =  5;
	_face[1][1] = 10;
	_face[1][2] =  6;
	_face[1][3] = 16;
	_face[1][4] = 15;

	_face[2][0] =  3;
	_face[2][1] = 11;
	_face[2][2] =  4;
	_face[2][3] = 14;
	_face[2][4] = 13;

	_face[3][0] =  8;
	_face[3][1] = 12;
	_face[3][2] =  7;
	_face[3][3] = 15;
	_face[3][4] = 16;

	_face[4][0] =  3;
	_face[4][1] = 13;
	_face[4][2] =  1;
	_face[4][3] = 17;
	_face[4][4] = 18;

	_face[5][0] =  2;
	_face[5][1] = 14;
	_face[5][2] =  4;
	_face[5][3] =  0;
	_face[5][4] = 19;

	_face[6][0] =  5;
	_face[6][1] = 15;
	_face[6][2] =  7;
	_face[6][3] = 18;
	_face[6][4] = 17;

	_face[7][0] =  8;
	_face[7][1] = 16;
	_face[7][2] =  6;
	_face[7][3] = 19;
	_face[7][4] =  0;

	_face[8][0] =  5;
	_face[8][1] = 17;
	_face[8][2] =  1;
	_face[8][3] =  9;
	_face[8][4] = 10;

	_face[9][0] =  3;
	_face[9][1] = 18;
	_face[9][2] =  7;
	_face[9][3] = 12;
	_face[9][4] = 11;

	_face[10][0] =  2;
	_face[10][1] = 19;
	_face[10][2] =  6;
	_face[10][3] = 10;
	_face[10][4] =  9;

	_face[11][0] =  8;
	_face[11][1] =  0;
	_face[11][2] =  4;
	_face[11][3] = 11;
	_face[11][4] = 12;


	for (int i=0; i< _facecount; i++)
	{
		_facevisible[i] = TRUE;
		_fillcolor[i] = RGB(128+Random(128),
					Random(2)?256:Random(256),
					Random(2)?256:Random(256));
		_facedistance[i] = 0.0;
	}
	_setfacecentroid();
	_setfacenormal();
	_centroid = Vector3(0.0, 0.0, 0.0);
	_radius = radius;
}

void Polyhedron3::Room(const Box3 &worldbox)
{
// Use this as a room for things to be inside.  Arrange so the
// normals point inward; if you are outside the room, a wall is
// transparent.
	int newfacecount[6] = {4,4,4,4,4,4};
	Size(8,6,newfacecount, 0, NULL);

	_vector3vert[0] = Vector3(worldbox.hix(), worldbox.loy(), worldbox.hiz());
	_vector3vert[1] = Vector3(worldbox.hix(), worldbox.hiy(), worldbox.hiz());
	_vector3vert[2] = Vector3(worldbox.lox(), worldbox.hiy(), worldbox.hiz());
	_vector3vert[3] = Vector3(worldbox.lox(), worldbox.loy(), worldbox.hiz());
	_vector3vert[4] = Vector3(worldbox.hix(), worldbox.loy(), worldbox.loz());
	_vector3vert[5] = Vector3(worldbox.hix(), worldbox.hiy(), worldbox.loz());
	_vector3vert[6] = Vector3(worldbox.lox(), worldbox.hiy(), worldbox.loz());
	_vector3vert[7] = Vector3(worldbox.lox(), worldbox.loy(), worldbox.loz());

	_face[0][0] = 3;
	_face[0][1] = 2;
	_face[0][2] = 1;
	_face[0][3] = 0;
	_fillcolor[0] = RGB(128, 0, 0); //dark red

	_face[1][0] = 4;
	_face[1][1] = 7;
	_face[1][2] = 3;
	_face[1][3] = 0;
	_fillcolor[1] = RGB(128,128 ,128); //dark gray;

	_face[2][0] = 1;
	_face[2][1] = 5;
	_face[2][2] = 4;
	_face[2][3] = 0;
	_fillcolor[2] = RGB(0,0,128); //BLUE;

	_face[3][0] = 2;
	_face[3][1] = 6;
	_face[3][2] = 5;
	_face[3][3] = 1;
	_fillcolor[3] = _fillcolor[1];

	_face[4][0] = 3;
	_face[4][1] = 7;
	_face[4][2] = 6;
	_face[4][3] = 2;
	_fillcolor[4] = _fillcolor[2];

	_face[5][0] = 5;
	_face[5][1] = 6;
	_face[5][2] = 7;
	_face[5][3] = 4;
	_fillcolor[5] = _fillcolor[0];

	for (int i=0; i< _facecount; i++)
	{
		_facevisible[i] = TRUE;
		_facedistance[i] = 0.0;
	}
	_setfacecentroid();
	_setfacenormal();
	_centroid = worldbox.Center();
	_radius = (Vector3(worldbox.XRadius(), worldbox.YRadius(),
		worldbox.ZRadius())).Magnitude();
}

void Polyhedron3::ProjectHouse(Real radius)
{
        int newfacevertcount[32] = {4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4};
		int newvertedgecount[17]={4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,8};
        Size(17,32,newfacevertcount,17,newvertedgecount);
        Real halfedge = radius / 2.0; //radius = sqrt(4 * edge^2).

		for (int i=0; i<17; i++)
			_vector3vert[i] = Vector3(1.0, 0.0, 0.0);
		/* We plan to use this object only to project onto, so we don't
		bother to put anything meaningful in the _vector3vert */

        _face[0][0] = 0;
        _face[0][1] = 3;
		_face[0][2] = 2;
        _face[0][3] = 1;

		_face[1][0] = 0;
        _face[1][1] = 5;
        _face[1][2] = 4;
        _face[1][3] = 3;

        _face[2][0] = 4;
        _face[2][1] = 5;
        _face[2][2] = 6;
        _face[2][3] = 7;

        _face[3][0] = 1;
        _face[3][1] = 2;
        _face[3][2] = 7;
        _face[3][3] = 6;

        _face[4][0] = 2;
        _face[4][1] = 3;
        _face[4][2] = 4;
        _face[4][3] = 7;

        _face[5][0] = 0;
        _face[5][1] = 1;
        _face[5][2] = 6;
        _face[5][3] = 5;

        _face[6][0] = 8;
        _face[6][1] = 11;
        _face[6][2] = 10;
        _face[6][3] = 9;

        _face[7][0] = 13;
        _face[7][1] = 12;
        _face[7][2] = 11;
        _face[7][3] = 8;

        _face[8][0] = 12;
        _face[8][1] = 13;
        _face[8][2] = 14;
        _face[8][3] = 15;

        _face[9][0] = 9;
        _face[9][1] = 10;
        _face[9][2] = 15;
        _face[9][3] = 14;

        _face[10][0] = 8;
        _face[10][1] = 9;
        _face[10][2] = 14;
        _face[10][3] = 13;

        _face[11][0] = 11;
        _face[11][1] = 12;
        _face[11][2] = 15;
        _face[11][3] = 10;

        _face[12][0] = 6;
        _face[12][1] = 1;
        _face[12][2] = 9;
        _face[12][3] = 14;

        _face[13][0] = 0;
        _face[13][1] = 3;
        _face[13][2] = 11;
        _face[13][3] = 8;

        _face[14][0] = 0;
        _face[14][1] = 8;
        _face[14][2] = 9;
        _face[14][3] = 1;

        _face[15][0] = 6;
        _face[15][1] = 14;
        _face[15][2] = 15;
        _face[15][3] = 7;

        _face[16][0] = 3;
        _face[16][1] = 4;
        _face[16][2] = 12;
        _face[16][3] = 11;

        _face[17][0] = 4;
        _face[17][1] = 7;
        _face[17][2] = 15;
        _face[17][3] = 12;

        _face[18][0] = 1;
        _face[18][1] = 9;
        _face[18][2] = 10;
        _face[18][3] = 2;

        _face[19][0] = 2;
        _face[19][1] = 7;
        _face[19][2] = 15;
        _face[19][3] = 10;

        _face[20][0] = 5;
        _face[20][1] = 13;
        _face[20][2] = 14;
        _face[20][3] = 6;

        _face[21][0] = 5;
        _face[21][1] = 0;
        _face[21][2] = 8;
        _face[21][3] = 13;

        _face[22][0] = 5;
        _face[22][1] = 4;
        _face[22][2] = 12;
        _face[22][3] = 13;

        _face[23][0] = 2;
        _face[23][1] = 10;
        _face[23][2] = 11;
        _face[23][3] = 3;

        _face[24][0] = 16;
        _face[24][1] = 8;
        _face[24][2] = 9;
		_face[24][3] = 16;

        _face[25][0] = 16;
        _face[25][1] = 13;
        _face[25][2] = 8;
		_face[25][3] = 16;

        _face[26][0] = 16;
        _face[26][1] = 14;
        _face[26][2] = 13;
		_face[26][3] = 16;

        _face[27][0] = 16;
        _face[27][1] = 9;
        _face[27][2] = 14;
		_face[27][3] = 16;

        _face[28][0] = 16;
        _face[28][1] = 12;
        _face[28][2] = 1;
		_face[28][3] = 16;

        _face[29][0] = 16;
        _face[29][1] = 11;
        _face[29][2] = 10;
		_face[29][3] = 16;
	
        _face[30][0] = 16;
        _face[30][1] = 10;
        _face[30][2] = 15;
		_face[30][3] = 16;
		
        _face[31][0] = 16;
        _face[31][1] = 15;
        _face[31][2] = 12;
		_face[31][3] = 16;

    
	for (i=0; i< 32; i++)
	{
		_facevisible[i] = TRUE;
		_fillcolor[i] = RGB(128+Random(128),
					Random(2)?256:Random(256),
					Random(2)?256:Random(256));
		_facedistance[i] = 0.0;
	}

	_setfacecentroid();
	_setfacenormal();
    _centroid = Vector3(0.0, 0.0, 0.0);
	_radius = radius;

}

void Polyhedron3::ProjectHouse1(Real radius)
{
        int newfacevertcount[32] = {4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4};
		int newvertedgecount[17]={4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,8};
        Size(17,32,newfacevertcount,17,newvertedgecount);
        Real halfedge = radius / 2.0; //radius = sqrt(4 * edge^2).

		for (int i=0; i<17; i++)
			_vector3vert[i] = Vector3(1.0, 0.0, 0.0);
		/* We plan to use this object only to project onto, so we don't
		bother to put anything meaningful in the _vector3vert */

        _face[0][0] = 0;
        _face[0][1] = 3;
		_face[0][2] = 2;
        _face[0][3] = 1;

		_face[1][0] = 0;
        _face[1][1] = 5;
        _face[1][2] = 4;
        _face[1][3] = 3;

        _face[2][0] = 4;
        _face[2][1] = 5;
        _face[2][2] = 6;
        _face[2][3] = 7;

        _face[3][0] = 1;
        _face[3][1] = 2;
        _face[3][2] = 7;
        _face[3][3] = 6;

        _face[4][0] = 2;
        _face[4][1] = 3;
        _face[4][2] = 4;
        _face[4][3] = 7;

        _face[5][0] = 0;
        _face[5][1] = 1;
        _face[5][2] = 6;
        _face[5][3] = 5;

        _face[6][0] = 8;
        _face[6][1] = 11;
        _face[6][2] = 10;
        _face[6][3] = 9;

        _face[7][0] = 13;
        _face[7][1] = 12;
        _face[7][2] = 11;
        _face[7][3] = 8;

        _face[8][0] = 12;
        _face[8][1] = 13;
        _face[8][2] = 14;
        _face[8][3] = 15;

        _face[9][0] = 9;
        _face[9][1] = 10;
        _face[9][2] = 15;
        _face[9][3] = 14;

        _face[10][0] = 8;
        _face[10][1] = 9;
        _face[10][2] = 14;
        _face[10][3] = 13;

        _face[11][0] = 11;
        _face[11][1] = 12;
        _face[11][2] = 15;
        _face[11][3] = 10;

        _face[12][0] = 6;
        _face[12][1] = 1;
        _face[12][2] = 9;
        _face[12][3] = 14;

        _face[13][0] = 0;
        _face[13][1] = 3;
        _face[13][2] = 11;
        _face[13][3] = 8;

        _face[14][0] = 0;
        _face[14][1] = 8;
        _face[14][2] = 9;
        _face[14][3] = 1;

        _face[15][0] = 6;
        _face[15][1] = 14;
        _face[15][2] = 15;
        _face[15][3] = 7;

        _face[16][0] = 3;
        _face[16][1] = 4;
        _face[16][2] = 12;
        _face[16][3] = 11;

        _face[17][0] = 4;
        _face[17][1] = 7;
        _face[17][2] = 15;
        _face[17][3] = 12;

        _face[18][0] = 1;
        _face[18][1] = 9;
        _face[18][2] = 10;
        _face[18][3] = 2;

        _face[19][0] = 2;
        _face[19][1] = 7;
        _face[19][2] = 15;
        _face[19][3] = 10;

        _face[20][0] = 5;
        _face[20][1] = 13;
        _face[20][2] = 14;
        _face[20][3] = 6;

        _face[21][0] = 5;
        _face[21][1] = 0;
        _face[21][2] = 8;
        _face[21][3] = 13;

        _face[22][0] = 5;
        _face[22][1] = 4;
        _face[22][2] = 12;
        _face[22][3] = 13;

        _face[23][0] = 2;
        _face[23][1] = 10;
        _face[23][2] = 11;
        _face[23][3] = 3;

        _face[24][0] = 16;
        _face[24][1] = 8;
        _face[24][2] = 9;
		_face[24][3] = 16;

        _face[25][0] = 16;
        _face[25][1] = 13;
        _face[25][2] = 8;
		_face[25][3] = 16;

        _face[26][0] = 16;
        _face[26][1] = 14;
        _face[26][2] = 13;
		_face[26][3] = 16;

        _face[27][0] = 16;
        _face[27][1] = 9;
        _face[27][2] = 14;
		_face[27][3] = 16;

        _face[28][0] = 16;
        _face[28][1] = 12;
        _face[28][2] = 1;
		_face[28][3] = 16;

        _face[29][0] = 16;
        _face[29][1] = 11;
        _face[29][2] = 10;
		_face[29][3] = 16;
	
        _face[30][0] = 16;
        _face[30][1] = 10;
        _face[30][2] = 15;
		_face[30][3] = 16;
		
        _face[31][0] = 16;
        _face[31][1] = 15;
        _face[31][2] = 12;
		_face[31][3] = 16;

          
        for (i=0; i< 32; i++)
	{
		_facevisible[i] = TRUE;
        _fillcolor[i] = TRANSPARENT;
		_facedistance[i] = 0.0;
	}

	_setfacecentroid();
	_setfacenormal();
    _centroid = Vector3(0.0, 0.0, 0.0);
	_radius = radius;

}


void Polyhedron3::ProjectHyperCube(Real radius)
{
		int newvertedgecount[16]={4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4};
        int newfacevertcount[24] = {4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4};
        Size(16, 24, newfacevertcount,16,newvertedgecount);
        Real halfedge = radius / 2.0; //radius = sqrt(4 * edge^2).

		for (int i=0; i<16; i++)
			_vector3vert[i] = Vector3(1.0, 0.0, 0.0);
		/* We plan to use this object only to project onto, so we don't
		bother to put anything meaningful in the _vector3vert */

        _face[0][0] = 0;
        _face[0][1] = 3;
		_face[0][2] = 2;
        _face[0][3] = 1;

		_face[1][0] = 0;
        _face[1][1] = 5;
        _face[1][2] = 4;
        _face[1][3] = 3;

        _face[2][0] = 4;
        _face[2][1] = 5;
        _face[2][2] = 6;
        _face[2][3] = 7;

        _face[3][0] = 1;
        _face[3][1] = 2;
        _face[3][2] = 7;
        _face[3][3] = 6;

        _face[4][0] = 2;
        _face[4][1] = 3;
        _face[4][2] = 4;
        _face[4][3] = 7;

        _face[5][0] = 0;
        _face[5][1] = 1;
        _face[5][2] = 6;
        _face[5][3] = 5;

        _face[6][0] = 8;
        _face[6][1] = 11;
        _face[6][2] = 10;
        _face[6][3] = 9;

        _face[7][0] = 13;
        _face[7][1] = 12;
        _face[7][2] = 11;
        _face[7][3] = 8;

        _face[8][0] = 12;
        _face[8][1] = 13;
        _face[8][2] = 14;
        _face[8][3] = 15;

        _face[9][0] = 9;
        _face[9][1] = 10;
        _face[9][2] = 15;
        _face[9][3] = 14;

        _face[10][0] = 8;
        _face[10][1] = 9;
        _face[10][2] = 14;
        _face[10][3] = 13;

        _face[11][0] = 11;
        _face[11][1] = 12;
        _face[11][2] = 15;
        _face[11][3] = 10;

        _face[12][0] = 6;
        _face[12][1] = 1;
        _face[12][2] = 9;
        _face[12][3] = 14;

        _face[13][0] = 0;
        _face[13][1] = 3;
        _face[13][2] = 11;
        _face[13][3] = 8;

        _face[14][0] = 0;
        _face[14][1] = 8;
        _face[14][2] = 9;
        _face[14][3] = 1;

        _face[15][0] = 6;
        _face[15][1] = 14;
        _face[15][2] = 15;
        _face[15][3] = 7;

        _face[16][0] = 3;
        _face[16][1] = 4;
        _face[16][2] = 12;
        _face[16][3] = 11;

        _face[17][0] = 4;
        _face[17][1] = 7;
        _face[17][2] = 15;
        _face[17][3] = 12;

        _face[18][0] = 1;
        _face[18][1] = 9;
        _face[18][2] = 10;
        _face[18][3] = 2;

        _face[19][0] = 2;
        _face[19][1] = 7;
        _face[19][2] = 15;
        _face[19][3] = 10;

        _face[20][0] = 5;
        _face[20][1] = 13;
        _face[20][2] = 14;
        _face[20][3] = 6;

        _face[21][0] = 5;
        _face[21][1] = 0;
        _face[21][2] = 8;
        _face[21][3] = 13;

        _face[22][0] = 5;
        _face[22][1] = 4;
        _face[22][2] = 12;
        _face[22][3] = 13;

        _face[23][0] = 2;
        _face[23][1] = 10;
        _face[23][2] = 11;
        _face[23][3] = 3;

    
	for (i=0; i< 24; i++)
	{
		_facevisible[i] = TRUE;
		_fillcolor[i] = RGB(128+Random(128),
					Random(2)?256:Random(256),
					Random(2)?256:Random(256));
		_facedistance[i] = 0.0;
	}

	_setfacecentroid();
	_setfacenormal();
    _centroid = Vector3(0.0, 0.0, 0.0);
	_radius = radius;



}

void Polyhedron3::ProjectHyperCube1(Real radius)
{
		int newvertedgecount[16]={4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4};
        int newfacevertcount[24] = {4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4};
        Size(16, 24, newfacevertcount,16,newvertedgecount);
        Real halfedge = radius / 2.0; //radius = sqrt(4 * edge^2).

		for (int i=0; i<16; i++)
			_vector3vert[i] = Vector3(1.0, 0.0, 0.0);
		/* We plan to use this object only to project onto, so we don't
		bother to put anything meaningful in the _vector3vert */

        _face[0][0] = 0;
        _face[0][1] = 3;
		_face[0][2] = 2;
        _face[0][3] = 1;

		_face[1][0] = 0;
        _face[1][1] = 5;
        _face[1][2] = 4;
        _face[1][3] = 3;

        _face[2][0] = 4;
        _face[2][1] = 5;
        _face[2][2] = 6;
        _face[2][3] = 7;

        _face[3][0] = 1;
        _face[3][1] = 2;
        _face[3][2] = 7;
        _face[3][3] = 6;

        _face[4][0] = 2;
        _face[4][1] = 3;
        _face[4][2] = 4;
        _face[4][3] = 7;

        _face[5][0] = 0;
        _face[5][1] = 1;
        _face[5][2] = 6;
        _face[5][3] = 5;

        _face[6][0] = 8;
        _face[6][1] = 11;
        _face[6][2] = 10;
        _face[6][3] = 9;

        _face[7][0] = 13;
        _face[7][1] = 12;
        _face[7][2] = 11;
        _face[7][3] = 8;

        _face[8][0] = 12;
        _face[8][1] = 13;
        _face[8][2] = 14;
        _face[8][3] = 15;

        _face[9][0] = 9;
        _face[9][1] = 10;
        _face[9][2] = 15;
        _face[9][3] = 14;

        _face[10][0] = 8;
        _face[10][1] = 9;
        _face[10][2] = 14;
        _face[10][3] = 13;

        _face[11][0] = 11;
        _face[11][1] = 12;
        _face[11][2] = 15;
        _face[11][3] = 10;

        _face[12][0] = 6;
        _face[12][1] = 1;
        _face[12][2] = 9;
        _face[12][3] = 14;

        _face[13][0] = 0;
        _face[13][1] = 3;
        _face[13][2] = 11;
        _face[13][3] = 8;

        _face[14][0] = 0;
        _face[14][1] = 8;
        _face[14][2] = 9;
        _face[14][3] = 1;

        _face[15][0] = 6;
        _face[15][1] = 14;
        _face[15][2] = 15;
        _face[15][3] = 7;

        _face[16][0] = 3;
        _face[16][1] = 4;
        _face[16][2] = 12;
        _face[16][3] = 11;

        _face[17][0] = 4;
        _face[17][1] = 7;
        _face[17][2] = 15;
        _face[17][3] = 12;

        _face[18][0] = 1;
        _face[18][1] = 9;
        _face[18][2] = 10;
        _face[18][3] = 2;

        _face[19][0] = 2;
        _face[19][1] = 7;
        _face[19][2] = 15;
        _face[19][3] = 10;

        _face[20][0] = 5;
        _face[20][1] = 13;
        _face[20][2] = 14;
        _face[20][3] = 6;

        _face[21][0] = 5;
        _face[21][1] = 0;
        _face[21][2] = 8;
        _face[21][3] = 13;

        _face[22][0] = 5;
        _face[22][1] = 4;
        _face[22][2] = 12;
        _face[22][3] = 13;

        _face[23][0] = 2;
        _face[23][1] = 10;
        _face[23][2] = 11;
        _face[23][3] = 3;

          
        for (i=0; i< 24; i++)
	{
		_facevisible[i] = TRUE;
        _fillcolor[i] = TRANSPARENT;
		_facedistance[i] = 0.0;
	}

	_setfacecentroid();
	_setfacenormal();
    _centroid = Vector3(0.0, 0.0, 0.0);
	_radius = radius;
}


//-----fourth dimension


Polytope4::Polytope4():_vector4vert(NULL), _vector3vert(NULL),_vector2vert(NULL),
    _pointvert(NULL),_centroid(0.0,0.0,0.0, 0.0),
	_radius(0.0), _facecount(0),
     _facevertcount(NULL),_face(NULL),_fillcolor(NULL), _solidnormal(NULL),
	_facevisible(NULL), _facecentroid(NULL), _facedistance(NULL), _edgecount(0),
	_vertedgecount(NULL),_vertedge(NULL)
{}

Polytope4::Polytope4(const Polytope4 &poly):_vector4vert(NULL), _vector3vert(NULL),
	_vector2vert(NULL),
    _pointvert(NULL),_centroid(0.0,0.0,0.0, 0.0),
	_radius(0.0), _facecount(0),
    _facevertcount(NULL),_face(NULL),_fillcolor(NULL), _solidnormal(NULL),
	_facevisible(NULL), _facecentroid(NULL), _facedistance(NULL), _edgecount(0),_vertedgecount(NULL),
	_vertedge(NULL)
{
	Size(poly._count, poly._facecount, poly._facevertcount,poly._edgecount, poly._vertedgecount);
	for (int i=0; i<_count; i++)
	{
        _vector4vert[i] = poly._vector4vert[i];
		_vector3vert[i] = poly._vector3vert[i];
		_vector2vert[i] = poly._vector2vert[i];
		_pointvert[i] = poly._pointvert[i];
	}
	_centroid = poly._centroid;
	_radius = poly._radius;
	for (i=0; i<_facecount; i++)
	{
		for (int j=0; j<_facevertcount[i]; j++)
			_face[i][j] = poly._face[i][j];
		_fillcolor[i] = poly._fillcolor[i];
		_solidnormal[i] = poly._solidnormal[i];
		_facecentroid[i] = poly._facecentroid[i];
		_facevisible[i] = poly._facevisible[i];
		_facedistance[i] = poly._facedistance[i];
	}
	for(i=0;i<_edgecount;i++)
	{
		for( int j=0;j<_vertedgecount[i]; j++)
			_vertedge[i][j]=poly._vertedge[i][j];
	}


}

Polytope4::~Polytope4()
{
	_freememory();
}

void Polytope4::_freememory()
{
    if (_vector4vert)
        {delete [] _vector4vert; _vector4vert = NULL;}   
	if (_vector3vert)
		{delete [] _vector3vert; _vector3vert = NULL;}
	if (_vector2vert)
		{delete [] _vector2vert; _vector2vert = NULL;}
	if (_pointvert)
		{delete [] _pointvert; _pointvert = NULL;}
	if (_facevertcount)
		{delete [] _facevertcount; _facevertcount = NULL;}
	if (_face)
		for (int i=0; i< _facecount; i++)
			if (_face[i])
				{delete [] _face[i]; _face[i] = NULL;}
	if (_face)
		{delete [] _face; _face = NULL;}
	if (_fillcolor)
		{delete [] _fillcolor; _fillcolor = NULL;}
	if (_solidnormal)
		{delete [] _solidnormal; _solidnormal = NULL;}
	if (_facecentroid)
		{delete [] _facecentroid; _facecentroid = NULL;}
	if (_facevisible)
		{delete [] _facevisible; _facevisible = NULL;}
	if (_facedistance)
		{delete [] _facedistance; _facedistance = NULL;}

	if (_vertedgecount)
		{delete [] _vertedgecount; _vertedgecount = NULL;}
	if (_vertedge)
		for (int i=0; i< _edgecount; i++)
			if (_vertedge[i])
				{delete [] _vertedge[i]; _vertedge[i] = NULL;}
	if (_vertedge)
		{delete [] _vertedge; _vertedge = NULL;}

}


Polytope4 Polytope4::operator=(const Polytope4 &poly)
{
 	if (this == &poly)
		return *this;
	Size(poly.Count(), poly._facecount, poly._facevertcount, poly._edgecount, poly._vertedgecount);
	for (int i=0; i<_count; i++)
    {
        _vector4vert[i] = poly._vector4vert[i];
        _vector3vert[i] = poly._vector3vert[i];
		_vector2vert[i] = poly._vector2vert[i];
		_pointvert[i] = poly._pointvert[i];
	}
	_centroid = poly._centroid;
	_radius = poly._radius;
	for (i=0; i<_facecount; i++)
	{
		for (int j=0; j<_facevertcount[i]; j++)
			_face[i][j] = poly._face[i][j];
		_fillcolor[i] = poly._fillcolor[i];
		_solidnormal[i] = poly._solidnormal[i];
		_facecentroid[i] = poly._facecentroid[i];
		_facevisible[i] = poly._facevisible[i];
		_facedistance[i] = poly._facedistance[i];
	}
	for(i=0;i<_edgecount;i++)
	{
		for(int j=0;j<_vertedgecount[i]; j++)
			_vertedge[i][j]=poly._vertedge[i][j];
	}

	return *this;
}

void Polytope4::Size(int newcount, int newfacecount,
	int *newfacevertcount, int newedgecount, int *newvertedgecount)
{
 	_freememory();//Empty the Polytope out first.
	_count = newcount;
    _vector4vert = new Vector4[_count];
    assert(_vector4vert);
    _vector3vert = new Vector3[_count];
    assert(_vector3vert);
    _vector2vert = new Vector2[_count];
	assert(_vector2vert);
	_pointvert = new Point[_count];
	assert(_pointvert);
	_facecount = newfacecount;
	_facevertcount = new int[_facecount];
	assert(_facevertcount);
	_face = new int*[_facecount];
	assert(_face);
	_fillcolor = new COLORREF[_facecount];
	assert(_fillcolor);
    _solidnormal = new Vector4[_facecount];
	assert(_solidnormal);
    _facecentroid = new Vector4[_facecount];
	assert(_facecentroid);
	_facevisible = new BOOL[_facecount];
	assert(_facevisible);
	_facedistance = new Real[_facecount];
	assert(_facedistance);
	for (int i=0; i<_facecount; i++)
	{
		_facevertcount[i] = newfacevertcount[i];
		int temp = _facevertcount[i];
		_face[i] = new int[temp];
		assert(_face[i]);
	}
	_edgecount=newedgecount;
	_vertedgecount= new int[_edgecount];
	assert(_vertedgecount);
	_vertedge=new Vector4*[_edgecount];
	assert(_vertedge);

	for(i=0;i<_edgecount;i++)
	{
		_vertedgecount[i]=newvertedgecount[i];
		int temp = _vertedgecount[i];
		_vertedge[i]=new Vector4[temp];
		assert(_vertedge[i]);
	}
}


void Polytope4::_setfacecentroid()
{
     for (int i=0; i<_facecount; i++)
	{
         _facecentroid[i] = Vector4(0.0,0.0, 0.0, 0.0);
	     for (int j=0; j<_facevertcount[i]; j++)
                _facecentroid[i] += _vector4vert[_face[i][j]];
	     _facecentroid[i] /= _facevertcount[i];
	}
}

void Polytope4::_setsolidnormal()
{
     for (int i=0; i<_facecount; i++)
 		_solidnormal[i] =
	    		TripleCross((_vector4vert[_face[i][1]] - _vector4vert[_face[i][0]]), 
                (_vector4vert[_face[i][2]] - _vector4vert[_face[i][1]]),
	            (_vector4vert[_face[i][3]] - _vector4vert[_face[i][2]])
			).Normalize();
}

//Simple Parallel/Projective
void StandardProject(Polytope4  &p, Polyhedron3 &q)
{
	for (int i=0; i<p.Count(); i++)
		{
			q._vector3vert[i].Set(
				p._vector4vert[i].x(), p._vector4vert[i].y(), p._vector4vert[i].z());
		}
	q._centroid.Set(p._centroid.x(),p._centroid.y(), p._centroid.z());
	for ( i=0; i<p._edgecount  ;i++)
	{	
		for( int j=0; j<p._vertedgecount[i] ;j++)
		{
			q._vertedge[i][j].Set(p._vertedge[i][j].x(), p._vertedge[i][j].y(),
				p._vertedge[i][j].z());
			//to be safe and to help debug, normalize again
			q._vertedge[i][j].Normalize();//Rucker added 10/30

		}
	}
	for ( i=0; i<p._facecount; i++)
	{
		q._facecentroid[i].Set(p._facecentroid[i].x(),p._facecentroid[i].y(), p._facecentroid[i].z());
		q._facenormal[i].Set(p._solidnormal[i].x(),p._solidnormal[i].y(), p._solidnormal[i].z());
	}

}

void Polytope4::HyperHouse(Real radius)
{
        int newfacevertcount[32] = {4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4};
		int newvertedgecount[17]={4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,8};
        Size(17,32,newfacevertcount,17,newvertedgecount);
        Real halfedge = radius / 2.0; //radius = sqrt(4 * edge^2).

        _vector4vert[0] = Vector4(-halfedge, halfedge, halfedge, -halfedge);
        _vector4vert[1] = Vector4(halfedge, halfedge, halfedge, -halfedge);
        _vector4vert[2] = Vector4(halfedge, -halfedge, halfedge, -halfedge);
        _vector4vert[3] = Vector4(-halfedge, -halfedge, halfedge, -halfedge);
        _vector4vert[4] = Vector4(-halfedge, -halfedge,- halfedge, -halfedge);
        _vector4vert[5] = Vector4(-halfedge, halfedge, -halfedge, -halfedge);
        _vector4vert[6] = Vector4(halfedge, halfedge, -halfedge, -halfedge);
        _vector4vert[7] = Vector4(halfedge, -halfedge, -halfedge, -halfedge);
        _vector4vert[8] = Vector4(-halfedge, halfedge, halfedge, halfedge);
        _vector4vert[9] = Vector4(halfedge, halfedge, halfedge, halfedge);
        _vector4vert[10] = Vector4(halfedge, -halfedge, halfedge, halfedge);
        _vector4vert[11] = Vector4(-halfedge, -halfedge, halfedge, halfedge);
        _vector4vert[12] = Vector4(-halfedge, -halfedge, -halfedge, halfedge);
        _vector4vert[13] = Vector4(-halfedge, halfedge, -halfedge, halfedge);
        _vector4vert[14] = Vector4(halfedge, halfedge, -halfedge, halfedge);
        _vector4vert[15] = Vector4(halfedge, -halfedge, -halfedge, halfedge);
		_vector4vert[16] = Vector4(0.0,0.0,0.0,2*halfedge);

        _face[0][0] = 0;
        _face[0][1] = 3;
		_face[0][2] = 2;
        _face[0][3] = 1;

		_face[1][0] = 0;
        _face[1][1] = 5;
        _face[1][2] = 4;
        _face[1][3] = 3;

        _face[2][0] = 4;
        _face[2][1] = 5;
        _face[2][2] = 6;
        _face[2][3] = 7;

        _face[3][0] = 1;
        _face[3][1] = 2;
        _face[3][2] = 7;
        _face[3][3] = 6;

        _face[4][0] = 2;
        _face[4][1] = 3;
        _face[4][2] = 4;
        _face[4][3] = 7;

        _face[5][0] = 0;
        _face[5][1] = 1;
        _face[5][2] = 6;
        _face[5][3] = 5;

        _face[6][0] = 8;
        _face[6][1] = 11;
        _face[6][2] = 10;
        _face[6][3] = 9;

        _face[7][0] = 13;
        _face[7][1] = 12;
        _face[7][2] = 11;
        _face[7][3] = 8;

        _face[8][0] = 12;
        _face[8][1] = 13;
        _face[8][2] = 14;
        _face[8][3] = 15;

        _face[9][0] = 9;
        _face[9][1] = 10;
        _face[9][2] = 15;
        _face[9][3] = 14;

        _face[10][0] = 8;
        _face[10][1] = 9;
        _face[10][2] = 14;
        _face[10][3] = 13;

        _face[11][0] = 11;
        _face[11][1] = 12;
        _face[11][2] = 15;
        _face[11][3] = 10;

        _face[12][0] = 6;
        _face[12][1] = 1;
        _face[12][2] = 9;
        _face[12][3] = 14;

        _face[13][0] = 0;
        _face[13][1] = 3;
        _face[13][2] = 11;
        _face[13][3] = 8;

        _face[14][0] = 0;
        _face[14][1] = 8;
        _face[14][2] = 9;
        _face[14][3] = 1;

        _face[15][0] = 6;
        _face[15][1] = 14;
        _face[15][2] = 15;
        _face[15][3] = 7;

        _face[16][0] = 3;
        _face[16][1] = 4;
        _face[16][2] = 12;
        _face[16][3] = 11;

        _face[17][0] = 4;
        _face[17][1] = 7;
        _face[17][2] = 15;
        _face[17][3] = 12;

        _face[18][0] = 1;
        _face[18][1] = 9;
        _face[18][2] = 10;
        _face[18][3] = 2;

        _face[19][0] = 2;
        _face[19][1] = 7;
        _face[19][2] = 15;
        _face[19][3] = 10;

        _face[20][0] = 5;
        _face[20][1] = 13;
        _face[20][2] = 14;
        _face[20][3] = 6;

        _face[21][0] = 5;
        _face[21][1] = 0;
        _face[21][2] = 8;
        _face[21][3] = 13;

        _face[22][0] = 5;
        _face[22][1] = 4;
        _face[22][2] = 12;
        _face[22][3] = 13;

        _face[23][0] = 2;
        _face[23][1] = 10;
        _face[23][2] = 11;
        _face[23][3] = 3;

        _face[24][0] = 16;
        _face[24][1] = 8;
        _face[24][2] = 9;
		_face[24][3] = 16;//this is redundant but it is useful for _setsolidnormal()

        _face[25][0] = 16;
        _face[25][1] = 13;
        _face[25][2] = 8;
		_face[25][3] = 16;

        _face[26][0] = 16;
        _face[26][1] = 14;
        _face[26][2] = 13;
		_face[26][3] = 16;

        _face[27][0] = 16;
        _face[27][1] = 9;
        _face[27][2] = 14;
		_face[27][3] = 16;

        _face[28][0] = 16;
        _face[28][1] = 12;
        _face[28][2] = 1;
		_face[28][3] = 16;

        _face[29][0] = 16;
        _face[29][1] = 11;
        _face[29][2] = 10;
		_face[29][3] = 16;
	
        _face[30][0] = 16;
        _face[30][1] = 10;
        _face[30][2] = 15;
		_face[30][3] = 16;
		
        _face[31][0] = 16;
        _face[31][1] = 15;
        _face[31][2] = 12;
		_face[31][3] = 16;

        for (int i=0; i< 32; i++)
	{
		_facevisible[i] = TRUE;
                //_fillcolor[i] = 2 + Random(14);
                _fillcolor[i] = TRANSPARENT;
		_facedistance[i] = 0.0;
	}
	_setfacecentroid();
	_setsolidnormal();
    _centroid = Vector4(0.0, 0.0, 0.0, 0.0);
	_radius = radius;

}

void Polytope4::HyperCube(Real radius)
{
        int newfacevertcount[24] = {4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4};
		int newvertedgecount[16]={4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4};
        Size(16,24,newfacevertcount,16,newvertedgecount);
        Real halfedge = radius / 2.0; //radius = sqrt(4 * edge^2).

        _vector4vert[0] = Vector4(-halfedge, halfedge, halfedge, -halfedge);
        _vector4vert[1] = Vector4(halfedge, halfedge, halfedge, -halfedge);
        _vector4vert[2] = Vector4(halfedge, -halfedge, halfedge, -halfedge);
        _vector4vert[3] = Vector4(-halfedge, -halfedge, halfedge, -halfedge);
        _vector4vert[4] = Vector4(-halfedge, -halfedge,- halfedge, -halfedge);
        _vector4vert[5] = Vector4(-halfedge, halfedge, -halfedge, -halfedge);
        _vector4vert[6] = Vector4(halfedge, halfedge, -halfedge, -halfedge);
        _vector4vert[7] = Vector4(halfedge, -halfedge, -halfedge, -halfedge);
        _vector4vert[8] = Vector4(-halfedge, halfedge, halfedge, halfedge);
        _vector4vert[9] = Vector4(halfedge, halfedge, halfedge, halfedge);
        _vector4vert[10] = Vector4(halfedge, -halfedge, halfedge, halfedge);
        _vector4vert[11] = Vector4(-halfedge, -halfedge, halfedge, halfedge);
        _vector4vert[12] = Vector4(-halfedge, -halfedge, -halfedge, halfedge);
        _vector4vert[13] = Vector4(-halfedge, halfedge, -halfedge, halfedge);
        _vector4vert[14] = Vector4(halfedge, halfedge, -halfedge, halfedge);
        _vector4vert[15] = Vector4(halfedge, -halfedge, -halfedge, halfedge);


        _face[0][0] = 0;
        _face[0][1] = 3;
		_face[0][2] = 2;
        _face[0][3] = 1;

		_face[1][0] = 0;
        _face[1][1] = 5;
        _face[1][2] = 4;
        _face[1][3] = 3;

        _face[2][0] = 4;
        _face[2][1] = 5;
        _face[2][2] = 6;
        _face[2][3] = 7;

        _face[3][0] = 1;
        _face[3][1] = 2;
        _face[3][2] = 7;
        _face[3][3] = 6;

        _face[4][0] = 2;
        _face[4][1] = 3;
        _face[4][2] = 4;
        _face[4][3] = 7;

        _face[5][0] = 0;
        _face[5][1] = 1;
        _face[5][2] = 6;
        _face[5][3] = 5;

        _face[6][0] = 8;
        _face[6][1] = 11;
        _face[6][2] = 10;
        _face[6][3] = 9;

        _face[7][0] = 13;
        _face[7][1] = 12;
        _face[7][2] = 11;
        _face[7][3] = 8;

        _face[8][0] = 12;
        _face[8][1] = 13;
        _face[8][2] = 14;
        _face[8][3] = 15;

        _face[9][0] = 9;
        _face[9][1] = 10;
        _face[9][2] = 15;
        _face[9][3] = 14;

        _face[10][0] = 8;
        _face[10][1] = 9;
        _face[10][2] = 14;
        _face[10][3] = 13;

        _face[11][0] = 11;
        _face[11][1] = 12;
        _face[11][2] = 15;
        _face[11][3] = 10;

        _face[12][0] = 6;
        _face[12][1] = 1;
        _face[12][2] = 9;
        _face[12][3] = 14;

        _face[13][0] = 0;
        _face[13][1] = 3;
        _face[13][2] = 11;
        _face[13][3] = 8;

        _face[14][0] = 0;
        _face[14][1] = 8;
        _face[14][2] = 9;
        _face[14][3] = 1;

        _face[15][0] = 6;
        _face[15][1] = 14;
        _face[15][2] = 15;
        _face[15][3] = 7;

        _face[16][0] = 3;
        _face[16][1] = 4;
        _face[16][2] = 12;
        _face[16][3] = 11;

        _face[17][0] = 4;
        _face[17][1] = 7;
        _face[17][2] = 15;
        _face[17][3] = 12;

        _face[18][0] = 1;
        _face[18][1] = 9;
        _face[18][2] = 10;
        _face[18][3] = 2;

        _face[19][0] = 2;
        _face[19][1] = 7;
        _face[19][2] = 15;
        _face[19][3] = 10;

        _face[20][0] = 5;
        _face[20][1] = 13;
        _face[20][2] = 14;
        _face[20][3] = 6;

        _face[21][0] = 5;
        _face[21][1] = 0;
        _face[21][2] = 8;
        _face[21][3] = 13;

        _face[22][0] = 5;
        _face[22][1] = 4;
        _face[22][2] = 12;
        _face[22][3] = 13;

        _face[23][0] = 2;
        _face[23][1] = 10;
        _face[23][2] = 11;
        _face[23][3] = 3;

          
        for (int i=0; i< 24; i++)
	{
		_facevisible[i] = TRUE;
                //_fillcolor[i] = 2 + Random(14);
                _fillcolor[i] = TRANSPARENT;
		_facedistance[i] = 0.0;
	}
	_setfacecentroid();
	_setsolidnormal();
    _centroid = Vector4(0.0, 0.0, 0.0, 0.0);
	_radius = radius;
	
	_vertedge[0][0]=(_vector4vert[1]-_vector4vert[0]).Normalize();
	_vertedge[0][1]=(_vector4vert[3]-_vector4vert[0]).Normalize();
	_vertedge[0][2]=(_vector4vert[8]-_vector4vert[0]).Normalize();
	_vertedge[0][3]=(_vector4vert[5]-_vector4vert[0]).Normalize();

	_vertedge[1][0]=(_vector4vert[2]-_vector4vert[1]).Normalize();
	_vertedge[1][1]=(_vector4vert[0]-_vector4vert[1]).Normalize();
	_vertedge[1][2]=(_vector4vert[6]-_vector4vert[1]).Normalize();
	_vertedge[1][3]=(_vector4vert[9]-_vector4vert[1]).Normalize();

	_vertedge[2][0]=(_vector4vert[1]-_vector4vert[2]).Normalize();
	_vertedge[2][1]=(_vector4vert[10]-_vector4vert[2]).Normalize();
	_vertedge[2][2]=(_vector4vert[3]-_vector4vert[2]).Normalize();
	_vertedge[2][3]=(_vector4vert[7]-_vector4vert[2]).Normalize();

	_vertedge[3][0]=(_vector4vert[0]-_vector4vert[3]).Normalize();
	_vertedge[3][1]=(_vector4vert[2]-_vector4vert[3]).Normalize();
	_vertedge[3][2]=(_vector4vert[11]-_vector4vert[3]).Normalize();
	_vertedge[3][3]=(_vector4vert[4]-_vector4vert[3]).Normalize();

	_vertedge[4][0]=(_vector4vert[7]-_vector4vert[4]).Normalize();
	_vertedge[4][1]=(_vector4vert[3]-_vector4vert[4]).Normalize();
	_vertedge[4][2]=(_vector4vert[12]-_vector4vert[4]).Normalize();
	_vertedge[4][3]=(_vector4vert[5]-_vector4vert[4]).Normalize();

	_vertedge[5][0]=(_vector4vert[6]-_vector4vert[5]).Normalize();
	_vertedge[5][1]=(_vector4vert[0]-_vector4vert[5]).Normalize();
	_vertedge[5][2]=(_vector4vert[4]-_vector4vert[5]).Normalize();
	_vertedge[5][3]=(_vector4vert[13]-_vector4vert[5]).Normalize();

	_vertedge[6][0]=(_vector4vert[1]-_vector4vert[6]).Normalize();
	_vertedge[6][1]=(_vector4vert[7]-_vector4vert[6]).Normalize();
	_vertedge[6][2]=(_vector4vert[5]-_vector4vert[6]).Normalize();
	_vertedge[6][3]=(_vector4vert[14]-_vector4vert[6]).Normalize();

	_vertedge[7][0]=(_vector4vert[15]-_vector4vert[7]).Normalize();
	_vertedge[7][1]=(_vector4vert[2]-_vector4vert[7]).Normalize();
	_vertedge[7][2]=(_vector4vert[4]-_vector4vert[7]).Normalize();
	_vertedge[7][3]=(_vector4vert[6]-_vector4vert[7]).Normalize();

	_vertedge[8][0]=(_vector4vert[0]-_vector4vert[8]).Normalize();
	_vertedge[8][1]=(_vector4vert[11]-_vector4vert[8]).Normalize();
	_vertedge[8][2]=(_vector4vert[13]-_vector4vert[8]).Normalize();
	_vertedge[8][3]=(_vector4vert[9]-_vector4vert[8]).Normalize();

	_vertedge[9][0]=(_vector4vert[10]-_vector4vert[9]).Normalize();
	_vertedge[9][1]=(_vector4vert[1]-_vector4vert[9]).Normalize();
	_vertedge[9][2]=(_vector4vert[8]-_vector4vert[9]).Normalize();
	_vertedge[9][3]=(_vector4vert[14]-_vector4vert[9]).Normalize();

	_vertedge[10][0]=(_vector4vert[11]-_vector4vert[10]).Normalize();
	_vertedge[10][1]=(_vector4vert[2]-_vector4vert[10]).Normalize();
	_vertedge[10][2]=(_vector4vert[15]-_vector4vert[10]).Normalize();
	_vertedge[10][3]=(_vector4vert[9]-_vector4vert[10]).Normalize();

	_vertedge[11][0]=(_vector4vert[12]-_vector4vert[11]).Normalize();
	_vertedge[11][1]=(_vector4vert[8]-_vector4vert[11]).Normalize();
	_vertedge[11][2]=(_vector4vert[3]-_vector4vert[11]).Normalize();
	_vertedge[11][3]=(_vector4vert[10]-_vector4vert[11]).Normalize();

	_vertedge[12][0]=(_vector4vert[11]-_vector4vert[12]).Normalize();
	_vertedge[12][1]=(_vector4vert[13]-_vector4vert[12]).Normalize();
	_vertedge[12][2]=(_vector4vert[4]-_vector4vert[12]).Normalize();
	_vertedge[12][3]=(_vector4vert[15]-_vector4vert[12]).Normalize();

	_vertedge[13][0]=(_vector4vert[14]-_vector4vert[13]).Normalize();
	_vertedge[13][1]=(_vector4vert[5]-_vector4vert[13]).Normalize();
	_vertedge[13][2]=(_vector4vert[8]-_vector4vert[13]).Normalize();
	_vertedge[13][3]=(_vector4vert[12]-_vector4vert[13]).Normalize();

	_vertedge[14][0]=(_vector4vert[9]-_vector4vert[14]).Normalize();
	_vertedge[14][1]=(_vector4vert[15]-_vector4vert[14]).Normalize();
	_vertedge[14][2]=(_vector4vert[6]-_vector4vert[14]).Normalize();
	_vertedge[14][3]=(_vector4vert[13]-_vector4vert[14]).Normalize();

	_vertedge[15][0]=(_vector4vert[10]-_vector4vert[15]).Normalize();
	_vertedge[15][1]=(_vector4vert[12]-_vector4vert[15]).Normalize();
	_vertedge[15][2]=(_vector4vert[7]-_vector4vert[15]).Normalize();
	_vertedge[15][3]=(_vector4vert[14]-_vector4vert[15]).Normalize();

}

void Polytope4::Room(const Box4 &worldbox)
{
// Use this as a room for things to be inside.  Arrange so the
// normals point inward; if you are outside the room, a wall is
// transparent.
        int newfacecount[24] = {4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4};
		int newvertedgecount[16]={4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4};
        Size(16,24,newfacecount,16,newvertedgecount);

        _vector4vert[0] = Vector4(worldbox.lox(), worldbox.hiy(), worldbox.hiz(), worldbox.low());
        _vector4vert[1] = Vector4(worldbox.hix(), worldbox.hiy(), worldbox.hiz(), worldbox.low());
        _vector4vert[2] = Vector4(worldbox.hix(), worldbox.loy(), worldbox.hiz(), worldbox.low());
        _vector4vert[3] = Vector4(worldbox.lox(), worldbox.loy(), worldbox.hiz(), worldbox.low());
        _vector4vert[4] = Vector4(worldbox.lox(), worldbox.loy(), worldbox.loz(), worldbox.low());
        _vector4vert[5] = Vector4(worldbox.lox(), worldbox.hiy(), worldbox.loz(), worldbox.low());
        _vector4vert[6] = Vector4(worldbox.hix(), worldbox.hiy(), worldbox.loz(), worldbox.low());
        _vector4vert[7] = Vector4(worldbox.hix(), worldbox.loy(), worldbox.loz(), worldbox.low());
        _vector4vert[8] = Vector4(worldbox.lox(), worldbox.hiy(), worldbox.hiz(), worldbox.hiw());
        _vector4vert[9] = Vector4(worldbox.hix(), worldbox.hiy(), worldbox.hiz(), worldbox.hiw());
        _vector4vert[10] = Vector4(worldbox.hix(), worldbox.loy(), worldbox.hiz(), worldbox.hiw());
        _vector4vert[11] = Vector4(worldbox.lox(), worldbox.loy(), worldbox.hiz(), worldbox.hiw());
        _vector4vert[12] = Vector4(worldbox.lox(), worldbox.loy(), worldbox.loz(), worldbox.hiw());
        _vector4vert[13] = Vector4(worldbox.lox(), worldbox.hiy(), worldbox.loz(), worldbox.hiw());
        _vector4vert[14] = Vector4(worldbox.hix(), worldbox.hiy(), worldbox.loz(), worldbox.hiw());
        _vector4vert[15] = Vector4(worldbox.hix(), worldbox.loy(), worldbox.loz(), worldbox.hiw());

        _face[0][0] = 0;
        _face[0][1] = 3;
		_face[0][2] = 2;
        _face[0][3] = 1;

		_face[1][0] = 0;
        _face[1][1] = 5;
        _face[1][2] = 4;
        _face[1][3] = 3;

        _face[2][0] = 4;
        _face[2][1] = 5;
        _face[2][2] = 6;
        _face[2][3] = 7;

        _face[3][0] = 1;
        _face[3][1] = 2;
        _face[3][2] = 7;
        _face[3][3] = 6;

        _face[4][0] = 2;
        _face[4][1] = 3;
        _face[4][2] = 4;
        _face[4][3] = 7;

        _face[5][0] = 0;
        _face[5][1] = 1;
        _face[5][2] = 6;
        _face[5][3] = 5;

        _face[6][0] = 8;
        _face[6][1] = 11;
        _face[6][2] = 10;
        _face[6][3] = 9;

        _face[7][0] = 13;
        _face[7][1] = 12;
        _face[7][2] = 11;
        _face[7][3] = 8;

        _face[8][0] = 12;
        _face[8][1] = 13;
        _face[8][2] = 14;
        _face[8][3] = 15;

        _face[9][0] = 9;
        _face[9][1] = 10;
        _face[9][2] = 15;
        _face[9][3] = 14;

        _face[10][0] = 8;
        _face[10][1] = 9;
        _face[10][2] = 14;
        _face[10][3] = 13;

        _face[11][0] = 11;
        _face[11][1] = 12;
        _face[11][2] = 15;
        _face[11][3] = 10;

        _face[12][0] = 6;
        _face[12][1] = 1;
        _face[12][2] = 9;
        _face[12][3] = 14;

        _face[13][0] = 0;
        _face[13][1] = 3;
        _face[13][2] = 11;
        _face[13][3] = 8;

        _face[14][0] = 0;
        _face[14][1] = 8;
        _face[14][2] = 9;
        _face[14][3] = 1;

        _face[15][0] = 6;
        _face[15][1] = 14;
        _face[15][2] = 15;
        _face[15][3] = 7;

        _face[16][0] = 3;
        _face[16][1] = 4;
        _face[16][2] = 12;
        _face[16][3] = 11;

        _face[17][0] = 4;
        _face[17][1] = 7;
        _face[17][2] = 15;
        _face[17][3] = 12;

        _face[18][0] = 1;
        _face[18][1] = 9;
        _face[18][2] = 10;
        _face[18][3] = 2;

        _face[19][0] = 2;
        _face[19][1] = 7;
        _face[19][2] = 15;
        _face[19][3] = 10;

        _face[20][0] = 5;
        _face[20][1] = 13;
        _face[20][2] = 14;
        _face[20][3] = 6;

        _face[21][0] = 5;
        _face[21][1] = 0;
        _face[21][2] = 8;
        _face[21][3] = 13;

        _face[22][0] = 5;
        _face[22][1] = 4;
        _face[22][2] = 12;
        _face[22][3] = 13;

        _face[23][0] = 2;
        _face[23][1] = 10;
        _face[23][2] = 11;
        _face[23][3] = 3;

                             
	for (int i=0; i< _facecount; i++)
	{
		_facevisible[i] = TRUE;
		_facedistance[i] = 0.0;
                _fillcolor[i] = TRANSPARENT;//for now then convert it into red, darkgray...
	}
	_setfacecentroid();
	_setsolidnormal();
	_centroid = worldbox.Center();
    _radius = (Vector4(worldbox.XRadius(), worldbox.YRadius(),
                worldbox.ZRadius(), worldbox.WRadius())).Magnitude();

	_vertedge[0][0]=(_vector4vert[1]-_vector4vert[0]).Normalize();
	_vertedge[0][1]=(_vector4vert[3]-_vector4vert[0]).Normalize();
	_vertedge[0][2]=(_vector4vert[8]-_vector4vert[0]).Normalize();
	_vertedge[0][3]=(_vector4vert[5]-_vector4vert[0]).Normalize();

	_vertedge[1][0]=(_vector4vert[2]-_vector4vert[1]).Normalize();
	_vertedge[1][1]=(_vector4vert[0]-_vector4vert[1]).Normalize();
	_vertedge[1][2]=(_vector4vert[6]-_vector4vert[1]).Normalize();
	_vertedge[1][3]=(_vector4vert[9]-_vector4vert[1]).Normalize();

	_vertedge[2][0]=(_vector4vert[1]-_vector4vert[2]).Normalize();
	_vertedge[2][1]=(_vector4vert[10]-_vector4vert[2]).Normalize();
	_vertedge[2][2]=(_vector4vert[3]-_vector4vert[2]).Normalize();
	_vertedge[2][3]=(_vector4vert[7]-_vector4vert[2]).Normalize();

	_vertedge[3][0]=(_vector4vert[0]-_vector4vert[3]).Normalize();
	_vertedge[3][1]=(_vector4vert[2]-_vector4vert[3]).Normalize();
	_vertedge[3][2]=(_vector4vert[11]-_vector4vert[3]).Normalize();
	_vertedge[3][3]=(_vector4vert[4]-_vector4vert[3]).Normalize();

	_vertedge[4][0]=(_vector4vert[7]-_vector4vert[4]).Normalize();
	_vertedge[4][1]=(_vector4vert[3]-_vector4vert[4]).Normalize();
	_vertedge[4][2]=(_vector4vert[12]-_vector4vert[4]).Normalize();
	_vertedge[4][3]=(_vector4vert[5]-_vector4vert[4]).Normalize();

	_vertedge[5][0]=(_vector4vert[6]-_vector4vert[5]).Normalize();
	_vertedge[5][1]=(_vector4vert[0]-_vector4vert[5]).Normalize();
	_vertedge[5][2]=(_vector4vert[4]-_vector4vert[5]).Normalize();
	_vertedge[5][3]=(_vector4vert[13]-_vector4vert[5]).Normalize();

	_vertedge[6][0]=(_vector4vert[1]-_vector4vert[6]).Normalize();
	_vertedge[6][1]=(_vector4vert[7]-_vector4vert[6]).Normalize();
	_vertedge[6][2]=(_vector4vert[5]-_vector4vert[6]).Normalize();
	_vertedge[6][3]=(_vector4vert[14]-_vector4vert[6]).Normalize();

	_vertedge[7][0]=(_vector4vert[15]-_vector4vert[7]).Normalize();
	_vertedge[7][1]=(_vector4vert[2]-_vector4vert[7]).Normalize();
	_vertedge[7][2]=(_vector4vert[4]-_vector4vert[7]).Normalize();
	_vertedge[7][3]=(_vector4vert[6]-_vector4vert[7]).Normalize();

	_vertedge[8][0]=(_vector4vert[0]-_vector4vert[8]).Normalize();
	_vertedge[8][1]=(_vector4vert[11]-_vector4vert[8]).Normalize();
	_vertedge[8][2]=(_vector4vert[13]-_vector4vert[8]).Normalize();
	_vertedge[8][3]=(_vector4vert[9]-_vector4vert[8]).Normalize();

	_vertedge[9][0]=(_vector4vert[10]-_vector4vert[9]).Normalize();
	_vertedge[9][1]=(_vector4vert[1]-_vector4vert[9]).Normalize();
	_vertedge[9][2]=(_vector4vert[8]-_vector4vert[9]).Normalize();
	_vertedge[9][3]=(_vector4vert[14]-_vector4vert[9]).Normalize();

	_vertedge[10][0]=(_vector4vert[11]-_vector4vert[10]).Normalize();
	_vertedge[10][1]=(_vector4vert[2]-_vector4vert[10]).Normalize();
	_vertedge[10][2]=(_vector4vert[15]-_vector4vert[10]).Normalize();
	_vertedge[10][3]=(_vector4vert[9]-_vector4vert[10]).Normalize();

	_vertedge[11][0]=(_vector4vert[12]-_vector4vert[11]).Normalize();
	_vertedge[11][1]=(_vector4vert[8]-_vector4vert[11]).Normalize();
	_vertedge[11][2]=(_vector4vert[3]-_vector4vert[11]).Normalize();
	_vertedge[11][3]=(_vector4vert[10]-_vector4vert[11]).Normalize();

	_vertedge[12][0]=(_vector4vert[11]-_vector4vert[12]).Normalize();
	_vertedge[12][1]=(_vector4vert[13]-_vector4vert[12]).Normalize();
	_vertedge[12][2]=(_vector4vert[4]-_vector4vert[12]).Normalize();
	_vertedge[12][3]=(_vector4vert[15]-_vector4vert[12]).Normalize();

	_vertedge[13][0]=(_vector4vert[14]-_vector4vert[13]).Normalize();
	_vertedge[13][1]=(_vector4vert[5]-_vector4vert[13]).Normalize();
	_vertedge[13][2]=(_vector4vert[8]-_vector4vert[13]).Normalize();
	_vertedge[13][3]=(_vector4vert[12]-_vector4vert[13]).Normalize();

	_vertedge[14][0]=(_vector4vert[9]-_vector4vert[14]).Normalize();
	_vertedge[14][1]=(_vector4vert[15]-_vector4vert[14]).Normalize();
	_vertedge[14][2]=(_vector4vert[6]-_vector4vert[14]).Normalize();
	_vertedge[14][3]=(_vector4vert[13]-_vector4vert[14]).Normalize();

	_vertedge[15][0]=(_vector4vert[10]-_vector4vert[15]).Normalize();
	_vertedge[15][1]=(_vector4vert[12]-_vector4vert[15]).Normalize();
	_vertedge[15][2]=(_vector4vert[7]-_vector4vert[15]).Normalize();
	_vertedge[15][3]=(_vector4vert[14]-_vector4vert[15]).Normalize();


}

