/***********************************************************
* Project:	CS298                                          *
* File:		vector5.cpp                                    *
* Purpose:	This file contains information about 2-tuple,  *
*			3-tuple, and 4-tuple vectors.                  *
************************************************************/
#include <math.h>
#include "vector5.hpp"

//----TWO DIMENSIONAL STUFF-----------------------------------------
//------------------Vector2 methods-------------------
Vector2 Vector2::operator/=(Real f)
{
	if (fabs(f) >= SMALL_REAL)
     {
     	_x/=f;
          _y/=f;
     }
     return *this;
}

Real Vector2::Magnitude()
{
	return sqrt(_x*_x + _y*_y);
}

Vector2 Vector2::Normalize()
{ //Make the Vector2 a unit Vector2, and return its old magnitude
	Real m = Magnitude();
	if (m < SMALL_REAL)
	{
		_x = 1.0;
		_y = 0;
		return *this;
	}
	else
	{
		_x /= m;
		_y /= m;
		return *this;
	}
}

//--------------Friend functions for Vector2-------------

Vector2 operator*(const Vector2 &u, Real f) //Scalar prod
{
	return Vector2(u._x * f,	u._y * f);
}

Vector2 operator*(Real f, const Vector2 &u) //Scalar prod
{
	return Vector2(u._x * f, u._y * f);
}

Vector2 operator+(const Vector2 &u, const Vector2 &v)
{
	return Vector2(u._x + v._x, u._y + v._y);
}

Vector2 operator-(const Vector2 &u, const Vector2 &v)
{
	return Vector2(u._x - v._x, u._y - v._y);
}

Real operator%(const Vector2 &u, const Vector2 &v)
{
	return (u._x*v._x + u._y*v._y);
}

//----------Matrix3 operations----------------
void Matrix3::Identity()
{
	for (int i=0; i<3; i++)
     	for (int j=0; j<3; j++)
          	_m[i][j] = (i==j)?1:0;
}

void Matrix3::SetTranslationRotationScale(Vector2 trans, Real ang, Real sc)
{
	_m[0][0] = sc*cos(ang);
	_m[0][1] = -sc*sin(ang);
	_m[0][2] = trans.x();
	_m[1][0] = sc*sin(ang);
	_m[1][1] = sc*cos(ang);
	_m[1][2] = trans.y();
}

Matrix3 ScaleMatrix(Real sx, Real sy)
{
	Matrix3 M;

     M._m[0][0] = sx;
     M._m[1][1] = sy;
     return M;
}

Matrix3 RotationMatrix(Real ang)
{ //Rotate counterclockwise by ang radians.
	Matrix3 M;

     M._m[0][0] = cos(ang);
     M._m[0][1] = -sin(ang);
     M._m[1][0] = -M._m[0][1]; //sin(ang)
     M._m[1][1] = M._m[0][0]; //cos(ang)
     return M;
}

Matrix3 TranslationMatrix(Vector2 trans)
{
	Matrix3 M;

     M._m[0][2] = trans.x();
     M._m[1][2] = trans.y();
     return M;
}

Matrix3 operator*(const Matrix3 &K, const Matrix3 &L)
{ //Assume for now that the bottom row of the matrix is always 0 0 1
	Matrix3 M;

	for (int i=0; i<2; i++)
		for (int j=0; j<3; j++)
     	{
          	M._m[i][j] = 0.0;
               for (int k=0; k<3; k++)
               	M._m[i][j] += K._m[i][k]*L._m[k][j];
          }
	return M;
}

//----------Matrix3 friend operations----------------
/* The following two friend operations are used to multiply
   in the matrix on the left, as in V = M * U for U and V vectors.*/

Vector2 operator*(const Matrix3 &M, const Vector2 &V)
{  //Assume for now that the bottom row of matrix is always 0 0 1
	Vector2 U;

     U._x = V._x * M._m[0][0] + V._y * M._m[0][1] + M._m[0][2];
     U._y = V._x * M._m[1][0] + V._y * M._m[1][1] + M._m[1][2];
     //Assume bottom row of the matrix is 0 0 1
	return U;
}

//----THREE DIMENSIONAL STUFF-----------------------------------------
//------------------Vector3 methods-------------------
Vector3 Vector3::operator/=(Real f)
{
	if (fabs(f) >= SMALL_REAL)
     {
     	_x/=f;
          _y/=f;
          _z/=f;
     }
     return *this;
}

Real Vector3::Magnitude()
{
	return sqrt(_x*_x + _y*_y + _z*_z);
}

Vector3 Vector3::Normalize()
{ //Make the Vector3 a unit Vector3, and return it
	Real m = Magnitude();
	if (m < SMALL_REAL)
	{
		_x = 1.0;
		_y = 0;
          _z = 0;
		return *this;
	}
	else
	{
		_x /= m;
		_y /= m;
		_z /= m;
		return *this;
	}
}

Real Vector3::ReturnMagnitudeAndNormalize()
{ //Make the Vector3 a unit Vector3, and return its old magnitude
	Real m = Magnitude();
	if (m < SMALL_REAL)
	{
		_x = 1.0;
		_y = 0;
          _z = 0;
		return m;
	}
	else
	{
		_x /= m;
		_y /= m;
		_z /= m;
		return m;
	}
}


//--------------Friend functions for Vector3-------------

Vector3 operator*(const Vector3 &u, Real f) //Scalar prod
{
	return Vector3(u._x * f,	u._y * f, u._z * f);
}

Vector3 operator*(Real f, const Vector3 &u) //Scalar prod
{
	return Vector3(u._x * f, u._y * f, u._z * f);
}

Vector3 operator+(const Vector3 &u, const Vector3 &v)
{
	return Vector3(u._x + v._x, u._y + v._y, u._z + v._z);
}

Vector3 operator-(const Vector3 &u, const Vector3 &v)
{
	return Vector3(u._x - v._x, u._y - v._y, u._z - v._z);
}

Real operator%(const Vector3 &u, const Vector3 &v)
{
	return (u._x*v._x + u._y*v._y + u._z*v._z);
}

Real Determinant(const Vector3 &A, const Vector3 &B, const Vector3 &C, const Vector3 &D)
{
	return ((A._z - C._z) * (D._x*B._y - B._x*D._y) +
		    (A._z - B._z) * (C._x*D._y - D._x*C._y) +
			(A._z - D._z) * (B._x*C._y - C._x*B._y) +
			(B._z - C._z) * (A._x*D._y - D._x*A._y) +
			(B._z - D._z) * (C._x*A._y - A._x*C._y) +
			(C._z - D._z) * (A._x*B._y - B._x*A._y) );
}

Vector3 operator*(const Vector3 &u, const Vector3 &v)
{
	return Vector3(u._y*v._z - u._z*v._y, -u._x*v._z + u._z*v._x,
		u._x*v._y - u._y*v._x);
}

//----------Matrix4 operations----------------

void Matrix4::Identity()
{
	for (int i=0; i<4; i++)
     	for (int j=0; j<4; j++)
          	_m[i][j] = (i==j)?1:0;
}

Matrix4 Matrix4::operator=(const Matrix4 &A)
{
	for (int i=0; i<4; i++)
     	for (int j=0; j<4; j++)
          	_m[i][j] = A._m[i][j];
	return *this;
}

void Matrix4::SetTranslationRotationScale(Vector3 trans,
	const Matrix4 &Rot, Real sc)
{
	for (int i=0; i<3; i++)
     	for (int j=0; j<3; j++)
          	_m[i][j] = sc * Rot._m[i][j];
	_m[0][3] = trans.x();
	_m[1][3] = trans.y();
	_m[2][3] = trans.z();
}

void Matrix4::SetTranslationRotation(Vector3 trans, const Matrix4 &rot)
{
	SetTranslationRotationScale(trans, rot, 1.0);
}

void Matrix4::SetColumns(const Vector3 &u, const Vector3 &v,
	const Vector3 &w)
{
		_m[0][0] = u._x;
		_m[0][1] = v._x;
		_m[0][2] = w._x;
		_m[1][0] = u._y;
		_m[1][1] = v._y;
		_m[1][2] = w._y;
		_m[2][0] = u._z;
		_m[2][1] = v._z;
		_m[2][2] = w._z;
}

Matrix4 Matrix4::Invert()
{ /*This only works if we assume the matrix is a rigid-body matrix, i.e.
that its Vector3 columns are orthonormal unit vectors as are its Vector3
rows (,ignoring the foruth column.)  You can have any translation you
like in the fourth column.  The inverse of such a matrix just takes the
transpose of the 3x3 rotational part, and then adjusts the fourth column
so that the matrix product will give zeroes there.  Thanks to Bob Holt
of Autodesk for showing me this trick.*/
	Matrix4 inv;

	inv.Identity();
	for (int i=0; i<3; i++)
		for (int j=0; j<3; j++)
		{
			inv._m[i][j] = _m[j][i];
			inv._m[i][3] -= inv._m[i][j]*_m[j][3];
		}
	*this = inv;
	return inv;
}

Matrix4 XRotationMatrix(Real ang)
{ //Rotate y-axis towards z-axis by ang radians.
	Matrix4 M;

     M._m[1][1] = cos(ang);
     M._m[1][2] = -sin(ang);
     M._m[2][1] = sin(ang);
     M._m[2][2] = cos(ang);
     return M;
}

Matrix4 YRotationMatrix(Real ang)
{ //Rotate z-axis towards x-axis by ang radians.
	Matrix4 M;

     M._m[0][0] = cos(ang);
     M._m[0][2] = sin(ang);
     M._m[2][0] = -sin(ang);
     M._m[2][2] = cos(ang);
     return M;
}

Matrix4 ZRotationMatrix(Real ang)
{ //Rotate x-axis towards y-axis by ang radians.
	Matrix4 M;

     M._m[0][0] = cos(ang);
     M._m[0][1] = -sin(ang);
     M._m[1][0] = -M._m[0][1]; //sin(ang)
     M._m[1][1] = M._m[0][0]; //cos(ang)
     return M;
}

Matrix4 TranslationMatrix(Vector3 trans)
{
	Matrix4 M;

     M._m[0][3] = trans.x();
     M._m[1][3] = trans.y();
     M._m[2][3] = trans.z();
     return M;
}

Matrix4 operator*(const Matrix4 &K, const Matrix4 &L)
{ //Assume for now that the bottom row of the matrix is always 0 0 0 1
	Matrix4 M;

	for (int i=0; i<3; i++)
		for (int j=0; j<4; j++)
     	{
          	M._m[i][j] = 0.0;
               for (int k=0; k<4; k++)
               	M._m[i][j] += K._m[i][k]*L._m[k][j];
          }
	return M;
}

//----------Matrix4 friend operations----------------
/* The following two friend operations are used to multiply
   in the matrix on the left, as in V = M * U for U and V vectors.*/

Vector3 operator*(const Matrix4 &M, const Vector3 &V)
{  //Assume for now that the bottom row of matrix is always 0 0 1
	Vector3 U;

     U._x = V._x * M._m[0][0] + V._y * M._m[0][1] + V._z * M._m[0][2] +
		M._m[0][3];
     U._y = V._x * M._m[1][0] + V._y * M._m[1][1] + V._z * M._m[1][2] +
		M._m[1][3];
     U._z = V._x * M._m[2][0] + V._y * M._m[2][1] + V._z * M._m[2][2] +
		M._m[2][3];
	return U;
}


//----FOUR DIMENSIONAL STUFF-----------------------------------------
//------------------Vector4 methods-------------------
Vector4 Vector4::operator/=(Real f)
{
	if (fabs(f) >= SMALL_REAL)
     {
     	_x/=f;
          _y/=f;
          _z/=f;
          _w/=f;
     }
     return *this;
}

Real Vector4::Magnitude()
{
	return sqrt(_x*_x + _y*_y + _z*_z + _w*_w);
}

Vector4 Vector4::Normalize()
{ //Make the Vector4 a unit Vector4, and return it
	Real m = Magnitude();
	if (m < SMALL_REAL)
	{
		_x = 1.0;
		_y = 0;
          _z = 0;     
          _w =0;
		return *this;
	}
	else
	{
		_x /= m;
		_y /= m;
		_z /= m;
		_w /= m;
		return *this;
	}
}

Real Vector4::ReturnMagnitudeAndNormalize()
{ //Make the Vector4 a unit Vector4, and return its old magnitude
	Real m = Magnitude();
	if (m < SMALL_REAL)
	{
		_x = 1.0;
		_y = 0;
          _z = 0;  
          _w = 0;
		return m;
	}
	else
	{
		_x /= m;
		_y /= m;
		_z /= m; 
		_w /= m;
		return m;
	}
}


//--------------Friend functions for Vector3-------------

Vector4 operator*(const Vector4 &u, Real f) //Scalar prod
{
	return Vector4(u._x * f,	u._y * f, u._z * f , u._w * f);
}

Vector4 operator*(Real f, const Vector4 &u) //Scalar prod
{
	return Vector4(u._x * f, u._y * f, u._z * f, u._w * f);
}

Vector4 operator+(const Vector4 &u, const Vector4 &v)
{
	return Vector4(u._x + v._x, u._y + v._y, u._z + v._z, u._w + v._w);
}

Vector4 operator-(const Vector4 &u, const Vector4 &v)
{
	return Vector4(u._x - v._x, u._y - v._y, u._z - v._z, u._w -v._w);
}

Real operator%(const Vector4 &u, const Vector4 &v)
{
	return (u._x*v._x + u._y*v._y + u._z*v._z + u._w*v._w);
}


Vector4 operator*(const Vector4 &u, const Vector4 &v)
{
	return Vector4(u._x*v._x, u._y*v._y,u._z*v._z, u._w*v._w);
}

Vector4 TripleCross(const Vector4 &u, const Vector4 &v, const Vector4 &s)
{
	return Vector4(u._y*(v._z*s._w - s._z*v._w) -u._z*(v._y*s._w - v._w*s._y)+u._w*(v._y*s._z-v._z*s._y),
		-u._x *(v._z*s._w - s._z*v._w) + u._z*(v._x*s._w-s._x*v._w)-u._w*(v._x*s._z-v._z*s._x),
		u._x*(v._y*s._w-v._w*s._y) - u._y*(v._x*s._w - v._w*s._x) + u._w*(v._x*s._y -v._y*s._x),
		-u._x*(v._y*s._z-v._z*s._y) +u._y*(v._x*s._z-v._z*s._x) - u._z*(v._x*s._y-v._y*s._x));               
}

//----------Matrix5 operations----------------

void Matrix5::Identity()
{
	for (int i=0; i<5; i++)
     	for (int j=0; j<5; j++)
          	_m[i][j] = (i==j)?1:0;
}

Matrix5 Matrix5::operator=(const Matrix5 &A)
{
	for (int i=0; i<5; i++)
     	for (int j=0; j<5; j++)
          	_m[i][j] = A._m[i][j];
	return *this;
}

void Matrix5::SetTranslationRotationScale(Vector4 trans,
	const Matrix5 &Rot, Real sc)
{
	for (int i=0; i<4; i++)
     	for (int j=0; j<4; j++)
          	_m[i][j] = sc * Rot._m[i][j];
	_m[0][4] = trans.x();
	_m[1][4] = trans.y();
	_m[2][4] = trans.z();
	_m[3][4] = trans.w();
}

void Matrix5::SetTranslationRotation(Vector4 trans, const Matrix5 &rot)
{
	SetTranslationRotationScale(trans, rot, 1.0);
}

void Matrix5::SetColumns(const Vector4 &u, const Vector4 &v,
	const Vector4 &w, const Vector4 &s)
{
		_m[0][0] = u._x;
		_m[0][1] = v._x;
		_m[0][2] = w._x;
		_m[0][3] = s._x;
		_m[1][0] = u._y;
		_m[1][1] = v._y;
		_m[1][2] = w._y;
		_m[1][3] = s._y;
		_m[2][0] = u._z;
		_m[2][1] = v._z;
		_m[2][2] = w._z;
		_m[2][3] = s._z;
		_m[3][0] = u._w;
		_m[3][1] = v._w;
		_m[3][2] = w._w;
		_m[3][3] = s._w;
}

Matrix5 Matrix5::Invert()
{ /*This only works if we assume the matrix is a rigid-body matrix, i.e.
that its Vector4 columns are orthonormal unit vectors as are its Vector4
rows (,ignoring the foruth column.)  You can have any translation you
like in the fourth column.  The inverse of such a matrix just takes the
transpose of the 4x4 rotational part, and then adjusts the fifth column
so that the matrix product will give zeroes there.  Thanks to Bob Holt
of Autodesk for showing me this trick.*/
	Matrix5 inv;

	inv.Identity();
	for (int i=0; i<4; i++)
		for (int j=0; j<4; j++)
		{
			inv._m[i][j] = _m[j][i];
			inv._m[i][4] -= inv._m[i][j]*_m[j][4];
		}
	*this = inv;
	return inv;
}

Matrix5 P12RotationMatrix5(Real ang)
{ //Rotate around plane 3-4
	Matrix5 M;

     M._m[0][0] = cos(ang);
     M._m[0][1] = sin(ang);
     M._m[1][0] = -sin(ang);
     M._m[1][1] = cos(ang);
     return M;
}

Matrix5 P13RotationMatrix5(Real ang)
{ //Rotate around plane 2-4
	Matrix5 M;

     M._m[0][0] = cos(ang);
     M._m[0][2] = sin(ang);
     M._m[2][0] = -sin(ang);
     M._m[2][2] = cos(ang);
     return M;
}

Matrix5 P14RotationMatrix5(Real ang)
{ //Rotate around plane 2-3
	Matrix5 M;

     M._m[0][0] = cos(ang);
     M._m[0][3] = sin(ang);
     M._m[3][0] = -sin(ang);
     M._m[3][3] = cos(ang);
     return M;
}

Matrix5 P23RotationMatrix5(Real ang)
{ //Rotate around plane 1-4
	Matrix5 M;

     M._m[1][1] = cos(ang);
     M._m[1][2] = sin(ang);
     M._m[2][1] = -sin(ang);
     M._m[2][2] = cos(ang);
     return M;
}

Matrix5 P24RotationMatrix5(Real ang)
{ //Rotate around plane 1-3
	Matrix5 M;

     M._m[1][1] = cos(ang);
     M._m[1][3] = sin(ang);
     M._m[3][1] = -sin(ang);
     M._m[3][3] = cos(ang);
     return M;
}

Matrix5 P34RotationMatrix5(Real ang)
{ //Rotate around plane 1-2
	Matrix5 M;

     M._m[2][2] = cos(ang);
     M._m[2][3] = sin(ang);
     M._m[3][2] = -sin(ang);
     M._m[3][3] = cos(ang);
     return M;
}



Matrix5 TranslationMatrix(Vector4 trans)
{
	Matrix5 M;

     M._m[0][4] = trans.x();
     M._m[1][4] = trans.y();
     M._m[2][4] = trans.z();
     M._m[3][4] = trans.w();
     return M;
}

Matrix5 operator*(const Matrix5 &K, const Matrix5 &L)
{ //Assume for now that the bottom row of the matrix is always 0 0 0 0 1
	Matrix5 M;

	for (int i=0; i<4; i++)
		for (int j=0; j<5; j++)
     	{
          	M._m[i][j] = 0.0;
               for (int k=0; k<5; k++)
               	M._m[i][j] += K._m[i][k]*L._m[k][j];
          }
	return M;
}

//----------Matrix5 friend operations----------------
/* The following two friend operations are used to multiply
   in the matrix on the left, as in V = M * U for U and V vectors.*/

Vector4 operator*(const Matrix5 &M, const Vector4 &V)
{  //Assume for now that the bottom row of matrix is always 0 0 1
	Vector4 U;

     U._x = V._x * M._m[0][0] + V._y * M._m[0][1] + V._z * M._m[0][2] +
		V._w * M._m[0][3] + M._m[0][4];
     U._y = V._x * M._m[1][0] + V._y * M._m[1][1] + V._z * M._m[1][2] +
		V._w * M._m[1][3] + M._m[1][4];
     U._z = V._x * M._m[2][0] + V._y * M._m[2][1] + V._z * M._m[2][2] +
		V._w * M._m[2][3] + M._m[2][4];
	 U._w = V._x * M._m[3][0] + V._y * M._m[3][1] + V._z * M._m[3][2] +
		V._w * M._m[3][3] + M._m[3][4];	
	return U;
}






