/***********************************************************
* Project:	CS298                                          *
* File:		Box2.cpp                                       *
* Purpose:	The primitive properties of the bounding box   *
*			are defined in this file.  For example, the    *
*			way wrapping is handled, velocity should be    *
*			added to bounce the object, or the way objects *
*			should be positioned inside the box are        *
*			defined in here.                               *
************************************************************/
#include "box2.hpp"

//----TWO DIMENSIONAL -----

void Box2::_Arrange()
{ //Helper function
	Real temp;
	if (_lox > _hix)
	{
		temp = _lox;
		_lox = _hix;
		_hix = temp;
	}
	if (_loy > _hiy)  //In Real land loy IS < hiy.
	{
		temp = _loy;
		_loy = _hiy;
		_hiy = temp;
	}
}

Box2::Box2(Real px, Real py, Real qx, Real qy):
_lox(px), _loy(py),_hix(qx),_hiy(qy)
{
	_Arrange();
}

void Box2::MatchAspect(int cx, int cy)
{
	Real oldaspect, newaspect, centery;// ,centerx;

	if (!(_hiy - _loy) || !cy)
		return;
	oldaspect = (_hix - _lox)/(_hiy - _loy);
	if (!oldaspect)
		return;
	newaspect = (Real)cx/(Real)cy;
	if (!newaspect)
		return;
//	centerx = (_hix - _lox)/2.0;
	centery = (_hiy - _loy)/2.0;
	//stretch or shrink the box vertically
    	_hiy = centery + (oldaspect/newaspect)*(_hiy - centery);
    	_loy = centery - (oldaspect/newaspect)*(centery - _loy);
/*
	//stretch or shrink the box horizontally
    	_hix = centerx + (newaspect/oldaspect)*(_hix - centerx);
    	_lox = centerx - (newaspect/oldaspect)*(centerx - _lox);
*/
}

BOOL Box2::Inside(const Vector2 &testpos)
{
	return (_lox <= testpos.x() && testpos.x() <= _hix &&
		_loy <= testpos.y() && testpos.y() <= _hiy);
}

int Box2::Outcode(const Vector2 &testpos)
{ /* This tells you which of the nine possible positions testpos has
relative to the Box2*/
	int outcode = INSIDE; //This is 0.

	if (_lox > testpos.x())
		outcode |= LEFT;
	if (testpos.x() > _hix)
		outcode |= RIGHT;
	if (_loy > testpos.y())
		outcode |= DOWN;
	if (testpos.y() > _hiy)
		outcode |= UP;
	return outcode;
}

Real Box2::Distance(const Vector2 &testpos)
{ /*This gives the distance from testpost to the closest point of the
Box2.  If you are in a "side" zone your nearest distance is a point
on the side.  If you are in a "corner" zone your nearest distance is a
corner.  If you are inside, we call the distance 0.*/
	int outcode = Outcode(testpos);

	switch(outcode)
	{
		case INSIDE:
			return 0.0;
		case LEFT:
			return _lox - testpos.x();
		case RIGHT:
			return testpos.x() - _hix;
		case DOWN:
			return _loy - testpos.y();
		case UP:
			return testpos.y() - _hiy;
		case DOWN_LEFT:
			return (testpos - Vector2(_lox,_loy)).Magnitude();
		case DOWN_RIGHT:
			return (testpos - Vector2(_hix,_loy)).Magnitude();
		case UP_LEFT:
			return (testpos - Vector2(_lox,_hiy)).Magnitude();
		case UP_RIGHT:
			return (testpos - Vector2(_hix,_hiy)).Magnitude();
	}
	return 0.0; //Unreachable default case.
}

BOOL Box2::AddBounce(Vector2 &position, Vector2 &velocity)
{
 	BOOL bounceflag = FALSE;
     Vector2 oldposition = position;

     position += velocity;
     //Now reverse appropriate velocity component if you've passed an edge
     if (position.x() < _lox || position.x() > _hix)
     {
     	velocity.Set(-velocity.x(), velocity.y());
          bounceflag = TRUE;
     }
     if (position.y() < _loy || position.y() > _hiy)
     {
     	velocity.Set(velocity.x(), -velocity.y());
		bounceflag = TRUE;
	}
	if (bounceflag) //If you did pass an edge, move back to where you were.
		position = oldposition;
     return bounceflag;
}

BOOL Box2::Wrap(Vector2 &position)
{
 	BOOL wrapflag = FALSE;
     if (position.x() < _lox)
     {
     	wrapflag = TRUE;
     	position.Set(_hix - _lox + position.x(), position.y());
     }
     if (position.x() > _hix)
     {
     	wrapflag = TRUE;
     	position.Set(_lox - _hix + position.x(), position.y());
     }
     if (position.y() < _loy)
     {
     	wrapflag = TRUE;
     	position.Set(position.x(), _hiy - _loy + position.y());
     }
     if (position.y() > _hiy)
     {
     	wrapflag = TRUE;
     	position.Set(position.x(), _loy - _hiy + position.y());
     }
	return wrapflag;
}

BOOL Box2::Clamp(Vector2 &position)
{
 	BOOL clampflag = FALSE;
     if (position.x() < _lox)
     {
     	clampflag = TRUE;
     	position.Set(_lox, position.y());
     }
     if (position.x() > _hix)
     {
     	clampflag = TRUE;
     	position.Set(_hix, position.y());
     }
     if (position.y() < _loy)
     {
     	clampflag = TRUE;
     	position.Set(position.x(), _loy);
     }
     if (position.y() > _hiy)
     {
     	clampflag = TRUE;
     	position.Set(position.x(), _hiy);
     }
	return clampflag;
}


BOOL Box2::Wrap(Vector3 &position)
{
 	BOOL wrapflag = FALSE;
     if (position.x() < _lox)
     {
     	wrapflag = TRUE;
     	position.Set(_hix - _lox + position.x(), position.y(), position.z());
     }
     if (position.x() > _hix)
     {
     	wrapflag = TRUE;
     	position.Set(_lox - _hix + position.x(), position.y(), position.z());
     }
     if (position.y() < _loy)
     {
     	wrapflag = TRUE;
     	position.Set(position.x(), _hiy - _loy + position.y(), position.z());
     }
     if (position.y() > _hiy)
     {
     	wrapflag = TRUE;
     	position.Set(position.x(), _loy - _hiy + position.y(), position.z());
     }
	return wrapflag;
}

BOOL Box2::Clamp(Vector3 &position)
{
 	BOOL clampflag = FALSE;
     if (position.x() < _lox)
     {
     	clampflag = TRUE;
     	position.Set(_lox, position.y(), position.z());
     }
     if (position.x() > _hix)
     {
     	clampflag = TRUE;
     	position.Set(_hix, position.y(), position.z());
     }
     if (position.y() < _loy)
     {
     	clampflag = TRUE;
     	position.Set(position.x(), _loy, position.z());
     }
     if (position.y() > _hiy)
     {
     	clampflag = TRUE;
     	position.Set(position.x(), _hiy, position.z());
     }
	return clampflag;
}

BOOL Box2::Wrap(Vector4 &position)
{
 	BOOL wrapflag = FALSE;
     if (position.x() < _lox)
     {
     	wrapflag = TRUE;
        position.Set(_hix - _lox + position.x(), position.y(), position.z(), position.w());
     }
     if (position.x() > _hix)
     {
     	wrapflag = TRUE;
        position.Set(_lox - _hix + position.x(), position.y(), position.z(), position.w());
     }
     if (position.y() < _loy)
     {
     	wrapflag = TRUE;
        position.Set(position.x(), _hiy - _loy + position.y(), position.z(), position.w());
     }
     if (position.y() > _hiy)
     {
     	wrapflag = TRUE;
        position.Set(position.x(), _loy - _hiy + position.y(), position.z(), position.w());
     }
	return wrapflag;
}

BOOL Box2::Clamp(Vector4 &position)
{
 	BOOL clampflag = FALSE;
     if (position.x() < _lox)
     {
     	clampflag = TRUE;
        position.Set(_lox, position.y(), position.z(), position.w());
     }
     if (position.x() > _hix)
     {
     	clampflag = TRUE;
        position.Set(_hix, position.y(), position.z(), position.w());
     }
     if (position.y() < _loy)
     {
     	clampflag = TRUE;
        position.Set(position.x(), _loy, position.z(), position.w());
     }
     if (position.y() > _hiy)
     {
     	clampflag = TRUE;
        position.Set(position.x(), _hiy, position.z(), position.w());
     }
	return clampflag;
}

//----THREE DIMENSIONAL -----

void Box3::_Arrange()
{ //Helper function
	Real temp;
	if (_lox > _hix)
	{
		temp = _lox;
		_lox = _hix;
		_hix = temp;
	}
	if (_loy > _hiy)
	{
		temp = _loy;
		_loy = _hiy;
		_hiy = temp;
	}
	if (_loz > _hiz)
	{
		temp = _loz;
		_loz = _hiz;
		_hiz = temp;
	}
}

Box3::Box3(Real px, Real py, Real pz, Real qx, Real qy, Real qz):
_lox(px),_loy(py), _loz(pz), _hix(qx),_hiy(qy), _hiz(qz)
{
	_Arrange();
}

Vector3 Box3::Center() const
{
	return Vector3(_lox + XRadius(), _loy + YRadius(), _loz + ZRadius());
}

BOOL Box3::AddBounce(Vector3 &position, Vector3 &velocity)
{
 	BOOL bounceflag = FALSE;
     Vector3 oldposition = position;

     position += velocity;
     //Now reverse appropriate velocity component if you've passed an edge
     if (position.x() < _lox || position.x() > _hix)
     {
     	velocity.Set(-velocity.x(), velocity.y(), velocity.z());
          bounceflag = TRUE;
     }
     if (position.y() < _loy || position.y() > _hiy)
     {
     	velocity.Set(velocity.x(), -velocity.y(), velocity.z());
		bounceflag = TRUE;
	}
     if (position.z() < _loz || position.z() > _hiz)
     {
     	velocity.Set(velocity.x(), velocity.y(), -velocity.z());
		bounceflag = TRUE;
	}
	if (bounceflag) //If you did pass an edge, move back to where you were.
		position = oldposition;
     return bounceflag;
}

BOOL Box3::Clamp(Vector3 &position)
{
 	BOOL clampflag = FALSE;
     if (position.x() < _lox)
     {
     	clampflag = TRUE;
     	position.Set(_lox, position.y(), position.z());
     }
     if (position.x() > _hix)
     {
     	clampflag = TRUE;
     	position.Set(_hix, position.y(), position.z());
     }
     if (position.y() < _loy)
     {
     	clampflag = TRUE;
     	position.Set(position.x(), _loy, position.z());
     }
     if (position.y() > _hiy)
     {
     	clampflag = TRUE;
     	position.Set(position.x(), _hiy, position.z());
     }
     if (position.z() < _loz)
     {
     	clampflag = TRUE;
     	position.Set(position.x(), position.y(), _loz);
     }
     if (position.z() > _hiz)
     {
     	clampflag = TRUE;
     	position.Set(position.x(), position.y(), _hiz);
     }
	return clampflag;
}

BOOL Box3::Wrap(Vector3 &position)
{
 	BOOL wrapflag = FALSE;
     if (position.x() < _lox)
     {
     	wrapflag = TRUE;
     	position.Set(_hix - _lox + position.x(), position.y(), position.z());
     }
     if (position.x() > _hix)
     {
     	wrapflag = TRUE;
     	position.Set(_lox - _hix + position.x(), position.y(), position.z());
     }
     if (position.y() < _loy)
     {
     	wrapflag = TRUE;
     	position.Set(position.x(), _hiy - _loy + position.y(), position.z());
     }
     if (position.y() > _hiy)
     {
     	wrapflag = TRUE;
     	position.Set(position.x(), _loy - _hiy + position.y(), position.z());
     }
     if (position.z() < _loz)
     {
     	wrapflag = TRUE;
     	position.Set(position.x(), position.y(), _hiz - _loz + position.z());
     }
     if (position.z() > _hiz)
     {
     	wrapflag = TRUE;
     	position.Set(position.x(), position.y(), _loz - _hiz + position.z());
     }
	return wrapflag;
}

BOOL Box3::Clamp(Vector4 &position)
{
 	BOOL clampflag = FALSE;
     if (position.x() < _lox)
     {
     	clampflag = TRUE;
        position.Set(_lox, position.y(), position.z(), position.w());
     }
     if (position.x() > _hix)
     {
     	clampflag = TRUE;
        position.Set(_hix, position.y(), position.z(), position.w());
     }
     if (position.y() < _loy)
     {
     	clampflag = TRUE;
        position.Set(position.x(), _loy, position.z(), position.w());
     }
     if (position.y() > _hiy)
     {
     	clampflag = TRUE;
        position.Set(position.x(), _hiy, position.z(), position.w());
     }
     if (position.z() < _loz)
     {
     	clampflag = TRUE;
        position.Set(position.x(), position.y(), _loz, position.w());
     }
     if (position.z() > _hiz)
     {
     	clampflag = TRUE;
        position.Set(position.x(), position.y(), _hiz, position.w());
     }
	return clampflag;
}

BOOL Box3::Wrap(Vector4 &position)
{
 	BOOL wrapflag = FALSE;
     if (position.x() < _lox)
     {
     	wrapflag = TRUE;
        position.Set(_hix - _lox + position.x(), position.y(), position.z(), position.w());
     }
     if (position.x() > _hix)
     {
     	wrapflag = TRUE;
        position.Set(_lox - _hix + position.x(), position.y(), position.z(), position.w());
     }
     if (position.y() < _loy)
     {
     	wrapflag = TRUE;
        position.Set(position.x(), _hiy - _loy + position.y(), position.z(), position.w());
     }
     if (position.y() > _hiy)
     {
     	wrapflag = TRUE;
        position.Set(position.x(), _loy - _hiy + position.y(), position.z(), position.w());
     }
     if (position.z() < _loz)
     {
     	wrapflag = TRUE;
        position.Set(position.x(), position.y(), _hiz - _loz + position.z(), position.w());
     }
     if (position.z() > _hiz)
     {
     	wrapflag = TRUE;
        position.Set(position.x(), position.y(), _loz - _hiz + position.z(), position.w());
     }
	return wrapflag;
}

//----FOUR DIMENSIONAL -----

void Box4::_Arrange()
{ //Helper function
	Real temp;
	if (_lox > _hix)
	{
		temp = _lox;
		_lox = _hix;
		_hix = temp;
	}
	if (_loy > _hiy)
	{
		temp = _loy;
		_loy = _hiy;
		_hiy = temp;
	}
	if (_loz > _hiz)
	{
		temp = _loz;
		_loz = _hiz;
		_hiz = temp;
	}
        if (_low > _hiw)
	{
                temp = _low;
                _low = _hiw;
                _hiw = temp;
	}
        
}

Box4::Box4(Real px, Real py, Real pz,Real pw, Real qx, Real qy, Real qz, Real qw):
_lox(px),_loy(py), _loz(pz), _low(pw), _hix(qx),_hiy(qy), _hiz(qz), _hiw(qw)
{
	_Arrange();
}

Vector4 Box4::Center() const
{
        return Vector4(_lox + XRadius(), _loy + YRadius(), _loz + ZRadius(), _low + WRadius());
}

BOOL Box4::AddBounce(Vector4 &position, Vector4 &velocity)
{
 	BOOL bounceflag = FALSE;
     Vector4 oldposition = position;

     position += velocity;
     //Now reverse appropriate velocity component if you've passed an edge
     if (position.x() < _lox || position.x() > _hix)
     {
        velocity.Set(-velocity.x(), velocity.y(), velocity.z(), velocity.w());
          bounceflag = TRUE;
     }
     if (position.y() < _loy || position.y() > _hiy)
     {
        velocity.Set(velocity.x(), -velocity.y(), velocity.z(), velocity.w());
		bounceflag = TRUE;
	}
     if (position.z() < _loz || position.z() > _hiz)
     {
        velocity.Set(velocity.x(), velocity.y(), -velocity.z(), velocity.w());
		bounceflag = TRUE;
	}
     if (position.w() < _low || position.w() > _hiw)
     {
        velocity.Set(velocity.x(), velocity.y(), velocity.z(), -velocity.w());
		bounceflag = TRUE;
	}

	if (bounceflag) //If you did pass an edge, move back to where you were.
		position = oldposition;
     return bounceflag;
}

BOOL Box4::Clamp(Vector4 &position)
{
 	BOOL clampflag = FALSE;
     if (position.x() < _lox)
     {
     	clampflag = TRUE;
        position.Set(_lox, position.y(), position.z(), position.w());
     }
     if (position.x() > _hix)
     {
     	clampflag = TRUE;
        position.Set(_hix, position.y(), position.z(), position.w());
     }
     if (position.y() < _loy)
     {
     	clampflag = TRUE;
        position.Set(position.x(), _loy, position.z(), position.w());
     }
     if (position.y() > _hiy)
     {
     	clampflag = TRUE;
        position.Set(position.x(), _hiy, position.z(), position.w());
     }
     if (position.z() < _loz)
     {
     	clampflag = TRUE;
        position.Set(position.x(), position.y(), _loz, position.w());
     }
     if (position.z() > _hiz)
     {
     	clampflag = TRUE;
        position.Set(position.x(), position.y(), _hiz, position.w());
     }
     if (position.w() < _low)
     {
     	clampflag = TRUE;
        position.Set(position.x(), position.y(), position.z(),  _low);
     }
     if (position.w() > _hiw)
     {
     	clampflag = TRUE;
        position.Set(position.x(), position.y(), position.z(), _hiw);
     }

	return clampflag;
}

BOOL Box4::Wrap(Vector4 &position)
{
 	BOOL wrapflag = FALSE;
     if (position.x() < _lox)
     {
     	wrapflag = TRUE;
        position.Set(_hix - _lox + position.x(), position.y(), position.z(), position.w());
     }
     if (position.x() > _hix)
     {
     	wrapflag = TRUE;
        position.Set(_lox - _hix + position.x(), position.y(), position.z(), position.w());
     }
     if (position.y() < _loy)
     {
     	wrapflag = TRUE;
        position.Set(position.x(), _hiy - _loy + position.y(), position.z(), position.w());
     }
     if (position.y() > _hiy)
     {
     	wrapflag = TRUE;
        position.Set(position.x(), _loy - _hiy + position.y(), position.z(), position.w());
     }
     if (position.z() < _loz)
     {
     	wrapflag = TRUE;
        position.Set(position.x(), position.y(), _hiz - _loz + position.z(), position.w());
     }
     if (position.z() > _hiz)
     {
     	wrapflag = TRUE;
        position.Set(position.x(), position.y(), _loz - _hiz + position.z(), position.w());
     }
     if (position.w() < _low)
     {
     	wrapflag = TRUE;
        position.Set(position.x(), position.y(), position.z(), _hiw - _low + position.w());
     }
     if (position.w() > _hiw)
     {
     	wrapflag = TRUE;
        position.Set(position.x(), position.y(), position.z(), _low - _hiw + position.w());
     }

	return wrapflag;
}







