/***********************************************************
* Project:	CS298                                          *
* File:		Box2.hpp                                       *
* Purpose:	The primitive properties of the bounding box   *
*			are defined in this file.  For example, the    *
*			way wrapping is handled, velocity should be    *
*			added to bounce the object, or the way objects *
*			should be positioned inside the box are        *
*			defined in here.                               *
************************************************************/

#ifndef BOX_HPP
#define BOX_HPP

#include "types.h"
#include "vector5.hpp"

//Outcodes for Box2::Outcode
#define INSIDE 0
#define LEFT 1
#define UP 2
#define RIGHT 4
#define DOWN 8
#define BELOW 16
#define ABOVE 32
#define UP_LEFT UP|LEFT
#define UP_RIGHT UP|RIGHT
#define DOWN_LEFT DOWN|LEFT
#define DOWN_RIGHT DOWN|RIGHT


class Box2
{
private:
	Real _lox, _hix, _loy, _hiy;
     void _Arrange(); //helper function
public:
	//constructors
	Box2():_lox(0),_hiy(0),_hix(0),_loy(0){}
	Box2(Real px, Real py, Real qx, Real qy);
	//accessors
	Real lox()const {return _lox;}
	Real hix()const {return _hix;}
	Real hiy()const {return _hiy;}
	Real loy()const {return _loy;}
	Real XRadius()const {return (_hix - _lox)/2.0;}
	Real YRadius()const {return (_hiy - _loy)/2.0;}
	//mutators
	void MatchAspect(int cx, int cy);
	//methods
	BOOL Inside(const Vector2 &testpos);
	int Outcode(const Vector2 &testpos);
	Real Distance(const Vector2 &testpos);
	BOOL AddBounce(Vector2 &position, Vector2 &velocity);
     /*add velocity to position, but if you pass the border, then
     	reflect velocity and keep the old position*/
	BOOL Wrap(Vector2 &position); /* If you move off one edge,
		then come back in the same amount from the other side */
	BOOL Clamp(Vector2 &position); /*Make sure position is inside*/
	BOOL Wrap(Vector3 &position); /* If you move off one edge,
		then come back in the same amount from the other side */
	BOOL Clamp(Vector3 &position); /*Make sure position is inside*/
    BOOL Wrap(Vector4 &position); /* If you move off one edge,
		then come back in the same amount from the other side */
    BOOL Clamp(Vector4 &position); /*Make sure position is inside*/

};

class Box3
{
private:
	Real _lox, _hix, _loy, _hiy, _loz, _hiz;
     void _Arrange(); //helper function
public:
	Box3():_lox(-0.5),_hix(0.5),_loy(-0.5),_hiy(0.5),_loz(-0.5),_hiz(0.5){}
	Box3(Real px, Real py, Real pz, Real qx, Real qy, Real qz);
	Real lox()const {return _lox;}
	Real hix()const {return _hix;}
	Real loy()const {return _loy;}
	Real hiy()const {return _hiy;}
	Real loz()const {return _loz;}
	Real hiz()const {return _hiz;}
	Real XRadius()const {return (_hix - _lox)/2.0;}
	Real YRadius()const {return (_hiy - _loy)/2.0;}
	Real ZRadius()const {return (_hiz - _loz)/2.0;}
	Vector3 Center() const;
	BOOL AddBounce(Vector3 &position, Vector3 &velocity);
     /*add velocity to position, but if you pass the border, then
     	reflect velocity and keep the old position*/
	BOOL Wrap(Vector3 &position); /* If you move off one edge,
		then come back in the same amount from the other side */
	BOOL Clamp(Vector3 &position); /*Make sure position is inside*/
    BOOL Wrap(Vector4 &position); /* If you move off one edge,
		then come back in the same amount from the other side */
    BOOL Clamp(Vector4 &position); /*Make sure position is inside*/

};

//-----four dimension stuffs --------

class Box4
{
private:
    Real _lox, _hix, _loy, _hiy, _loz, _hiz, _low, _hiw;
    void _Arrange(); //helper function
public:
    Box4():_lox(-1.0),_hix(1.0),_loy(-1.0),_hiy(1.0),_loz(-1.0),_hiz(1.0),_low(-1.0),_hiw(1.0){}
    Box4(Real px, Real py, Real pz, Real pw, Real qx, Real qy, Real qz, Real qw);
	Real lox()const {return _lox;}
	Real hix()const {return _hix;}
	Real loy()const {return _loy;}
	Real hiy()const {return _hiy;}
	Real loz()const {return _loz;}
	Real hiz()const {return _hiz;}
    Real low()const {return _low;}
    Real hiw()const {return _hiw;}
	Real XRadius()const {return (_hix - _lox)/2.0;}
	Real YRadius()const {return (_hiy - _loy)/2.0;}
	Real ZRadius()const {return (_hiz - _loz)/2.0;}
    Real WRadius()const {return (_hiw - _low)/2.0;}
    Vector4 Center() const;
    BOOL AddBounce(Vector4 &position, Vector4 &velocity);
     /*add velocity to position, but if you pass the border, then
     	reflect velocity and keep the old position*/
    BOOL Wrap(Vector4 &position); /* If you move off one edge,
		then come back in the same amount from the other side */
    BOOL Clamp(Vector4 &position); /*Make sure position is inside*/
};



#endif //BOX_HPP
