/***********************************************************
* Project:	CS298                                          *
* File:		hyperspace8.cpp                                *
* Purpose:	This is the main file that creates the         *
*			interactive windows program.                   *
************************************************************/

/*----------------------Log----------------------
Playpen.  Begun on January 26, 1996.
Purpose of the program is to provice a Windows graphics programming
environment for students in CS 116A Computer Graphics in SJSU
Dept. of Math & CS.  This course uses the Foley, Van Dam, et. al. book
INTRODUCTION TO COMPUTER GRAPHICS.  We implement definitions of
some of their "SRGP" graphics library functions defined in that book.
This program is Copyright (C) 1996 Rudy Rucker, but may be freely
used and distributed as shareware.
Updated 2/1/96
February 1, 1996. Version 1.1.
Changed to use point and rectangle "constructors"
Added polygon feature.  Made colors brighter: changed to LIGHT?????
February 6, 1996
Started using Frame.
February 7, Switched to C++, merged the polygon stuff with the Frame stuff.
Feb 26, made simple for testing line and circle algorithms.  Standard
patterns are the fan and the disk.  Note that the fan code isn't done,
i.e. the SRGP_lineCoord doesn't work yet except for GENTLE_POSITIVE case.
March 6, Put in animation and copyPixel.
March 14, started using matrix, did bouncing flatland.
April 3, Added 3D, started snowflakes.
April 11, Added VR camera control with view matrix.
April 12.  Debugged:  vector5.cpp, srgpwin6.cpp.
	Improved: polygon5.cpp, frame5.*, camera2.*, playpen9.cpp
April 17. Put in gooded snowflake shapes, added Segment class for
painter's algorithm.
April 24. Put in polyhedra (cubes), back face removal.
April 30. Fixed memory leak bug in Polygon7.cpp, added icosohedra.
	started room, but still not quite right.
9/17/97 Ported to VCC 5.0, Win 32. Renamed files to be consistent
with JOY OF HACKING
Fall 1997 added fourth dimension
------------------------------------------------------------*/
//----------------------Includes----------------------
#include "types.h"
#include <math.h>
#include "point2.hpp"
#include "vector5.hpp"
#include "frame1.hpp"   //For RealToPixel
#include "polyhedron4.hpp"
#include "GDIToolbox1.hpp"
#include "bitmap1.hpp"
#include "box2.hpp"   //Replaces RealRect
#include "camera3.hpp"  //For projecting
#include "random.h" //Rudy's randomize functions
#include "segment1.hpp"
#include "resource.h"
//---------
#define WORLDSIZE_BUMP 1.5
#define MAX_POLYGONS 5
#define MAX_POLYHEDRA 5
#define MAX_POLYTOPE 5
#define MAX_POLYTOPEFRAME 5
#define MAX_HOUSETOPEFRAME 5
#define MAX_POLYGONS_PER_POLYHEDRON 12 //Note that this could grow!
#define STAR_TYPES 15
#define MIN_REAL_X -4.0
#define MIN_REAL_Y -3.0
#define MIN_REAL_Z -2.0
#define MIN_REAL_W -1.0
#define MAX_REAL_X 4.0
#define MAX_REAL_Y 3.0
#define MAX_REAL_Z 2.0
#define MAX_REAL_W 1.0
#define MOUSE_SENSITIVITY 15 //pixels
//-------Initialization variables-------
Real minradius = 0.5;
Real defaultradius = 0.35;
Real defaultspeed = 0.05;
Real maxzspeed = 0.02; //0.02; //Set this 0 to keep the flakes in the right order.
Real roomsize = 1.0;
Box3 worldbox(roomsize*MIN_REAL_X, roomsize*MIN_REAL_Y, roomsize*MIN_REAL_Z,
	roomsize*MAX_REAL_X, roomsize*MAX_REAL_Y, roomsize*MAX_REAL_Z);
Box4 worldbox1(roomsize*MIN_REAL_X, roomsize*MIN_REAL_Y, roomsize*MIN_REAL_Z, roomsize*MIN_REAL_W,
        roomsize*MAX_REAL_X, roomsize*MAX_REAL_Y, roomsize*MAX_REAL_Z, roomsize*MAX_REAL_W);//4D

//--------------Variables used to communicate between WndProc and mydraw---
GDIToolbox gdibox; //WndProc, fixmenuchecks and mydraw.
Frame frame;
Camera camera;
WindowBitmap *wbm;
//the following line should be chosen carefully when new objects are added
//otherwise the program will not run!
Segment segment(6 + MAX_POLYGONS +  MAX_HOUSETOPEFRAME +MAX_POLYTOPE + MAX_POLYTOPEFRAME + MAX_POLYHEDRA 
	+ MAX_POLYGONS_PER_POLYHEDRON * (10*( MAX_HOUSETOPEFRAME+MAX_POLYTOPE + MAX_POLYTOPEFRAME) +MAX_POLYHEDRA)); //room + flakes + topes
Poly3 *polylist;
Vector3 *position;
Vector3 *velocity;
Matrix4 *tumble_step;
Matrix4 *tumble;
Matrix4 *aspect;
Polyhedron3 room;
Polyhedron3 *polyhedronlist;
Vector3 *polyhedronposition;
Vector3 *polyhedronvelocity;
Matrix4 *polyhedrontumble_step;
Matrix4 *polyhedrontumble;
Matrix4 *polyhedronaspect;
//fourth dimensional stuffs
//Polytope variables
Polytope4 *polytope;
Polytope4 *polytopeframe;
Polyhedron3 *visp;
Polyhedron3 *vispframe;
Matrix5 *polytopeaspect;
Matrix5 *polytopetumble;
Vector4 *polytopeposition;
Vector4 *polytopevelocity;
Matrix5 *polytopetumble_step;
Matrix5 *polytope_aspect;
Matrix5 *polytope_tumble;
Vector4 *polytope_position;
Vector4 *polytope_velocity;
Matrix5 *polytope_tumble_step;
//HOUSETOPE variables
Polytope4 *housetopeframe;
Polyhedron3 *house_frame;
Matrix5 *housetope_aspect;
Matrix5 *housetope_tumble;
Vector4 *housetope_position;
Vector4 *housetope_velocity;
Matrix5 *housetope_tumble_step;
int rotationtype=IDM_XYPLANE; //IDM_XYPLANE..IDM_ZANAPLANE
//---------------------
BOOL pauseflag = FALSE;
BOOL rotationflag = FALSE;
BOOL showroomflag = FALSE;
BOOL bounceflag = TRUE;
BOOL bounceobject = TRUE;
BOOL hypercubesolid = TRUE;
BOOL hypercubeframe = TRUE;
BOOL houseframe = FALSE;
BOOL snowtype = FALSE;
BOOL polyhedratype = FALSE;
int showtype;//IDM_SNOW, IDM_POLYHEDRA, IDM_ALL,IDM_HYPERCUBE, IDM_HYPER_FRAME,IDM_HOUSE_FRAME
//---------------WndProc control variables --------
BOOL changecameraposition = FALSE, changecamerarotation = FALSE;
Point oldpos, newpos;
//----------------------Function prototypes----------------------
void fixmenuchecks(HWND);
void resetcamera();
int GCD(int, int);
Point next_rel_prime_pair(Point p);
void mydraw(HWND hwnd);
LRESULT CALLBACK WndProc( HWND, UINT, WPARAM, LPARAM);
BOOL APIENTRY About( HWND hDlg, UINT message, UINT wParam, LONG lParam);
//----------------------WinMain----------------------
HINSTANCE ghinst;
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
	PSTR lpszCmdParam, int nCmdShow)
{
	ghinst = hInstance;
	static char szAppName[] = "Play11";
	HWND masterhwnd;
	MSG msg;
	WNDCLASS wndclass;

	if (!hPrevInstance)
	{
		wndclass.style = CS_HREDRAW | CS_VREDRAW;
		wndclass.lpfnWndProc = WndProc;
		wndclass.cbClsExtra = 0;
		wndclass.cbWndExtra = 0;
		wndclass.hInstance = hInstance;
		wndclass.hIcon = LoadIcon( NULL, IDI_APPLICATION );
		wndclass.hCursor = LoadCursor( NULL, IDC_ARROW );
		wndclass.hbrBackground = GetStockObject( WHITE_BRUSH );
		wndclass.lpszMenuName = "Hyperspace";
		wndclass.lpszClassName = szAppName;
		RegisterClass (&wndclass);
	}
	masterhwnd = CreateWindow(szAppName,		// window class name
			"Hyperspace",	// window caption
			WS_OVERLAPPEDWINDOW,		// window style
			CW_USEDEFAULT,				// initial x position
			CW_USEDEFAULT,				// initial y position
			CW_USEDEFAULT,				// initial x size
			CW_USEDEFAULT,				// initial y size
			NULL,					// parent window handle
			NULL,					// window menu handle
			hInstance,				// program instance handle
			NULL);					// creation parameters

	ShowWindow(masterhwnd, nCmdShow);
	UpdateWindow(masterhwnd);

	while (TRUE) //Keep it growing.  See Petzold's RANDRECT example.
	{
		if ( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) )
		{
			if (msg.message == WM_QUIT)
				break;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else
			mydraw(masterhwnd);
	}
	return msg.wParam;
}
//----------------------WndProc----------------------
LRESULT CALLBACK WndProc( HWND hwnd, UINT message, WPARAM wParam,
	LPARAM lParam)
{	
	HDC hdc;
	PAINTSTRUCT ps;
    Real randangle;
    int i;
	Point relprimepair;

	switch (message)
	{
		case WM_CREATE:
//          	Randomize();
				 /*this seeds the Rudy randomizer so that
               	different runs will use different random numbers.  You
                    only need to seed the randomizer ONCE in a given
                    program.*/
			gdibox.SetBlankColor(RGB(0,0,0));
			gdibox.SetLineColor(RGB(255,255,255)); //Polygon edges
			gdibox.SetLineWidth(1);
			wbm = new WindowBitmap(hwnd, RGB(0,0,0)); //Default background is white.
			gdibox.SetHDC(wbm->hdc());
			camera.Initialize(worldbox);
			camera.SetRealWindow(frame);
			frame.SetPixelWindow(GetSystemMetrics(SM_CXSCREEN),
				GetSystemMetrics(SM_CYSCREEN)); //Actual size of screen
			
			//Information about objects start from the next line
			polylist = new Poly3[MAX_POLYGONS];
			position = new Vector3[MAX_POLYGONS];
			velocity = new Vector3[MAX_POLYGONS];
			tumble_step = new Matrix4[MAX_POLYGONS];
			tumble = new Matrix4[MAX_POLYGONS];
			aspect = new Matrix4[MAX_POLYGONS];
               for (i=0; i<MAX_POLYGONS; i++)
               {
               	randangle = -(PI/2.0) + (PI/4.0)*Randomsignreal();
/* We are going to imagine the word of the snowflakes as having the snow
drop down the z axis.  The x axis points towards the camera.  The flakes
move primarily in the negative z direction.  We can give them a little x
motion as well */
                    velocity[i] = Vector3(cos(randangle), sin(randangle),
					Randomsignreal()*maxzspeed);
				velocity[i].Normalize();
				velocity[i] *= defaultspeed;
                 	position[i] = Vector3(
					worldbox.Center().x() +
						Randomsignreal() * worldbox.XRadius(),
					worldbox.Center().y() +
						Randomsignreal() * worldbox.YRadius(),
					worldbox.loz() +
						(Real(i)/MAX_POLYGONS)*2.0*worldbox.ZRadius());
                    tumble[i].Identity();
				tumble_step[i] =  /*This is the tumble matrix */
					XRotationMatrix((PI/50)*Randomsignreal()) *
					YRotationMatrix((PI/50)*Randomsignreal()) *
					ZRotationMatrix((PI/50)*Randomsignreal());
				if (!(i%STAR_TYPES))
					relprimepair = Point(5,2);
				else
					relprimepair = next_rel_prime_pair(relprimepair);
	               polylist[i].StarPolygon(relprimepair.x(), relprimepair.y(),
					Vector3(0.0,0.0,0.0),defaultradius);
				polylist[i].RandomizeFaceColor(hwnd);//Not BLACK or WHITE
               }
			polyhedronlist = new Polyhedron3[MAX_POLYHEDRA];
			polyhedronposition = new Vector3[MAX_POLYHEDRA];
			polyhedronvelocity = new Vector3[MAX_POLYHEDRA];
			polyhedrontumble_step = new Matrix4[MAX_POLYHEDRA];
			polyhedrontumble = new Matrix4[MAX_POLYHEDRA];
			polyhedronaspect = new Matrix4[MAX_POLYHEDRA];
               for (i=0; i<MAX_POLYHEDRA; i++)
               {
               	randangle = -(PI/2.0) + (PI/4.0)*Randomsignreal();
/* We are going to imagine the world of the snowflakes as having the snow
drop down the z axis.  The x axis points towards the camera.  The flakes
move primarily in the negative z direction.  We can give them a little x
motion as well */
                    polyhedronvelocity[i] = Vector3(cos(randangle), sin(randangle),
					Randomsignreal()*maxzspeed);
				polyhedronvelocity[i].Normalize();
				polyhedronvelocity[i] *= 1.5*defaultspeed;
                 	polyhedronposition[i] = Vector3(
					worldbox.Center().x() +
						Randomsignreal() * worldbox.XRadius(),
					worldbox.Center().y() +
						Randomsignreal() * worldbox.YRadius(),
					worldbox.loz() +
						(Real(i)/MAX_POLYHEDRA)*2.0*worldbox.ZRadius());
                    polyhedrontumble[i].Identity();
				polyhedrontumble_step[i] =  /*This is the tumble matrix */
					XRotationMatrix((PI/50)*Randomsignreal()) *
					YRotationMatrix((PI/50)*Randomsignreal()) *
					ZRotationMatrix((PI/50)*Randomsignreal());
				if (!(i%4))  //Every fourth i
 					polyhedronlist[i].Dodecahedron(defaultradius);
 				else
					polyhedronlist[i].Cube(defaultradius);
				polyhedronlist[i].RandomizeFaceColor(hwnd);
               }

//--------------fourth dimension-------
			polytope = new Polytope4[MAX_POLYTOPE];
			polytopetumble = new Matrix5[MAX_POLYTOPE];
			polytopetumble_step = new Matrix5[MAX_POLYTOPE];
			polytopevelocity = new Vector4[MAX_POLYTOPE];
			polytopeposition = new Vector4[MAX_POLYTOPE];
			polytopeaspect = new Matrix5[MAX_POLYTOPE];
			visp = new Polyhedron3[MAX_POLYTOPE];

			for(i=0;i<MAX_POLYTOPE;i++)
			{
               	randangle = -(PI/2.0) + (PI/4.0)*Randomsignreal();
                polytopevelocity[i] = Vector4(cos(randangle), sin(randangle),
					0.0, Randomsignreal()*maxzspeed);
				polytopevelocity[i].Normalize();
					polytopevelocity[i] *= 1.5*defaultspeed;
			    polytopeposition[i] = Vector4(
					worldbox1.Center().x() +
					Randomsignreal() * worldbox1.XRadius(),
					worldbox1.Center().y() +
					Randomsignreal() * worldbox1.YRadius(),
					worldbox1.Center().z() +
					Randomsignreal() * worldbox1.ZRadius(),
					worldbox1.Center().w() +
					Randomsignreal() * worldbox1.WRadius());
	  			polytopetumble[i].Identity();
				polytopetumble_step[i] =
					P12RotationMatrix5((PI/50)*Randomsignreal()) *
					P13RotationMatrix5((PI/50)*Randomsignreal()) *
					P14RotationMatrix5((PI/50)*Randomsignreal()) *
					P23RotationMatrix5((PI/50)*Randomsignreal()) *
					P24RotationMatrix5((PI/50)*Randomsignreal()) *
					P34RotationMatrix5((PI/50)*Randomsignreal()); 
				polytope[i].HyperCube(defaultradius);
				visp[i].ProjectHyperCube(defaultradius);

			}
			//the followings should be created because otherwise the release version will not work correctly.
			polytopeframe = new Polytope4[MAX_POLYTOPEFRAME];
			polytope_tumble = new Matrix5[MAX_POLYTOPEFRAME];
			polytope_tumble_step = new Matrix5[MAX_POLYTOPEFRAME];
			polytope_velocity = new Vector4[MAX_POLYTOPEFRAME];
			polytope_position = new Vector4[MAX_POLYTOPEFRAME];
			polytope_aspect = new Matrix5[MAX_POLYTOPEFRAME];
			vispframe = new Polyhedron3[MAX_POLYTOPEFRAME];

			for(i=0;i<MAX_POLYTOPEFRAME;i++)
			{
               	randangle = -(PI/2.0) + (PI/4.0)*Randomsignreal();
                polytope_velocity[i] = Vector4(cos(randangle), sin(randangle),
					0.0, Randomsignreal()*maxzspeed);
				polytope_velocity[i].Normalize();
					polytope_velocity[i] *= 1.5*defaultspeed;
			    polytope_position[i] = Vector4(
					worldbox1.Center().x() +
					Randomsignreal() * worldbox1.XRadius(),
					worldbox1.Center().y() +
					Randomsignreal() * worldbox1.YRadius(),
					worldbox1.Center().z() +
					Randomsignreal() * worldbox1.ZRadius(),
					worldbox1.Center().w() +
					Randomsignreal() * worldbox1.WRadius());
	  			polytope_tumble[i].Identity();
				polytope_tumble_step[i] =
					P12RotationMatrix5((PI/50)*Randomsignreal()) *
					P13RotationMatrix5((PI/50)*Randomsignreal()) *
					P14RotationMatrix5((PI/50)*Randomsignreal()) *
					P23RotationMatrix5((PI/50)*Randomsignreal()) *
					P24RotationMatrix5((PI/50)*Randomsignreal()) *
					P34RotationMatrix5((PI/50)*Randomsignreal()); 
				polytopeframe[i].HyperCube(defaultradius);
				vispframe[i].ProjectHyperCube1(defaultradius);

			}

//4D House-----
			housetopeframe = new Polytope4[MAX_HOUSETOPEFRAME];
			housetope_tumble = new Matrix5[MAX_HOUSETOPEFRAME];
			housetope_tumble_step = new Matrix5[MAX_HOUSETOPEFRAME];
			housetope_velocity = new Vector4[MAX_HOUSETOPEFRAME];
			housetope_position = new Vector4[MAX_HOUSETOPEFRAME];
			housetope_aspect = new Matrix5[MAX_HOUSETOPEFRAME];
			house_frame = new Polyhedron3[MAX_HOUSETOPEFRAME];

			for(i=0;i<MAX_HOUSETOPEFRAME;i++)
			{
               	randangle = -(PI/2.0) + (PI/4.0)*Randomsignreal();
                housetope_velocity[i] = Vector4(cos(randangle), sin(randangle),
					0.0, Randomsignreal()*maxzspeed);
				housetope_velocity[i].Normalize();
					housetope_velocity[i] *= 1.5*defaultspeed;
			    housetope_position[i] = Vector4(
					worldbox1.Center().x() +
					Randomsignreal() * worldbox1.XRadius(),
					worldbox1.Center().y() +
					Randomsignreal() * worldbox1.YRadius(),
					worldbox1.Center().z() +
					Randomsignreal() * worldbox1.ZRadius(),
					worldbox1.Center().w() +
					Randomsignreal() * worldbox1.WRadius());
	  			housetope_tumble[i].Identity();
				housetope_tumble_step[i] =
					P12RotationMatrix5((PI/50)*Randomsignreal()) *
					P13RotationMatrix5((PI/50)*Randomsignreal()) *
					P14RotationMatrix5((PI/50)*Randomsignreal()) *
					P23RotationMatrix5((PI/50)*Randomsignreal()) *
					P24RotationMatrix5((PI/50)*Randomsignreal()) *
					P34RotationMatrix5((PI/50)*Randomsignreal()); 
				housetopeframe[i].HyperHouse(defaultradius);
				house_frame[i].ProjectHouse1(defaultradius);

			}


#ifdef ONEHYPER
		camera.SetEyeFromUp(Vector3(2.0,0,0),Vector3(1,0,0),Vector3(0,1,0));
#endif
//--------------------------------------  
   			room.Room(worldbox);
			/*Go under the worldbox and look up, so snow falls on your
			face */
			resetcamera();
 			fixmenuchecks(hwnd);
			return 0;

		case WM_PAINT:
 			hdc = BeginPaint(hwnd, &ps); //Get the  HDC
			wbm->CopyTo(hdc, ps.rcPaint);
          	EndPaint(hwnd, &ps);  //Release the HDC
			return 0;

		case WM_SIZE:
			frame.SetPixelWindow(LOWORD(lParam), HIWORD(lParam));
			camera.MatchViewRectAspect(LOWORD(lParam), HIWORD(lParam));
	 		camera.SetRealWindow(frame);
			gdibox.SetClipRectangle(hwnd);
				/*The wbm->hdc() is as big as the whole screen, so you it
				 won't do be default do "hardware" clipping like a screen window
				 would.  You set the clip rectangle so that	no time is wasted
				 writing to parts of the memory canvas that	won't be copied to
				 the screen window.*/
		    wbm->Clear(); //so you don't see wrong crap in the first repaint.
			switch (wParam)
			{
				case SIZE_MINIMIZED:
					pauseflag = TRUE;
					break;
				case SIZE_RESTORED:
					pauseflag = FALSE;
			}
			fixmenuchecks(hwnd);
			return 0;

		case WM_KILLFOCUS:
			pauseflag = TRUE;
			fixmenuchecks(hwnd);
			return 0;

		case WM_SETFOCUS:
			pauseflag = FALSE;
			fixmenuchecks(hwnd);
			return 0;

		case WM_LBUTTONDOWN:
			if (wParam & MK_SHIFT)
				camera.ZBumpRotate(-1);
			else
				camera.ZBump(-1);
	  		camera.SetRealWindow(frame);
			return 0;

		case WM_RBUTTONDOWN:
			if (wParam & MK_SHIFT)
				camera.ZBumpRotate(1);
			else
				camera.ZBump(1);
			return 0;

		case WM_MOUSEMOVE:
			if (!(changecameraposition || changecamerarotation) &&
				((wParam & MK_CONTROL) || (wParam & MK_SHIFT)) )
               {
				if (wParam & MK_CONTROL)
                    	changecameraposition = TRUE;
				else //(wParam && MK_SHIFT)
					changecamerarotation = TRUE;
				oldpos = Point(lParam);
				return 0;
			}
			if (!(changecameraposition || changecamerarotation))
				return 0;
			if ((changecameraposition || changecamerarotation) &&
				!((wParam & MK_CONTROL) || (wParam & MK_SHIFT)) )
			{
				changecameraposition = FALSE;
				changecamerarotation = FALSE;
                    return 0;
			}
			if (changecameraposition)
			{
				newpos = Point(lParam);
				if (newpos.x() < oldpos.x() - MOUSE_SENSITIVITY)
				{
					camera.XBump(-1);
					oldpos= Point(newpos.x(),oldpos.y());
				}
				else if (newpos.x() > oldpos.x() + MOUSE_SENSITIVITY)
				{
					camera.XBump(1);
					oldpos= Point(newpos.x(),oldpos.y());
				}
				if (newpos.y() < oldpos.y() - MOUSE_SENSITIVITY)
				{
					camera.YBump(1);
					oldpos= Point(oldpos.x(),newpos.y());
				}
				else if (newpos.y() > oldpos.y() + MOUSE_SENSITIVITY)
				{
					camera.YBump(-1);
					oldpos= Point(oldpos.x(),newpos.y());
				}
		 		camera.SetRealWindow(frame);
			}
			if (changecamerarotation)
			{
				newpos = Point(lParam);
				if (newpos.x() < oldpos.x() - MOUSE_SENSITIVITY)
				{ //Rotate _from around _up
					camera.YBumpRotate(1);
					oldpos= Point(newpos.x(),oldpos.y());
				}
				else if (newpos.x() > oldpos.x() + MOUSE_SENSITIVITY)
				{
					camera.YBumpRotate(-1);
					oldpos= Point(newpos.x(),oldpos.y());
				}
				if (newpos.y() < oldpos.y() - MOUSE_SENSITIVITY)
				{  //Rotate _from around _up*_from (xaxis)
					camera.XBumpRotate(1);
					oldpos= Point(oldpos.x(),newpos.y());
				}
				else if (newpos.y() > oldpos.y() + MOUSE_SENSITIVITY)
				{
					camera.XBumpRotate(-1);
					oldpos= Point(oldpos.x(),newpos.y());
				}
		 		camera.SetRealWindow(frame);
			}
			return 0;


		case WM_COMMAND:
			switch (wParam)
			{
				case IDM_RESET:
					resetcamera();
					fixmenuchecks(hwnd);
					break;
				case IDM_PAUSE:
					pauseflag ^= TRUE;
					fixmenuchecks(hwnd);
					break;
				case IDM_BOUNCEOBJECT:
					bounceobject ^= TRUE;
					fixmenuchecks(hwnd);
					break;
				case IDM_WORLDBOX:
					showroomflag ^= TRUE;
					fixmenuchecks(hwnd);
					break;
				case IDM_BIGGERWORLDBOX:
					roomsize *= WORLDSIZE_BUMP;
					worldbox = Box3(roomsize*MIN_REAL_X,
						roomsize*MIN_REAL_Y, roomsize*MIN_REAL_Z,
						roomsize*MAX_REAL_X, roomsize*MAX_REAL_Y,
						roomsize*MAX_REAL_Z);
					room.Room(worldbox);
					for (i=0; i<MAX_POLYGONS; i++)
 						worldbox.Clamp(position[i]);
					for (i=0; i<MAX_POLYHEDRA; i++)
						worldbox.Clamp(polyhedronposition[i]);
					worldbox1 = Box4(
						roomsize*MIN_REAL_X,roomsize*MIN_REAL_Y,
						roomsize*MIN_REAL_Z,roomsize*MIN_REAL_W,
						roomsize*MAX_REAL_X,roomsize*MAX_REAL_Y,
						roomsize*MAX_REAL_Z,roomsize*MAX_REAL_W);
					for (i=0; i<MAX_POLYTOPE; i++)
						worldbox1.Clamp(polytopeposition[i]);
					for (i=0; i<MAX_POLYTOPEFRAME; i++)
						worldbox1.Clamp(polytope_position[i]);
					for (i=0; i<MAX_HOUSETOPEFRAME; i++)
						worldbox1.Clamp(housetope_position[i]);

					break;
				case IDM_SMALLERWORLDBOX:
					roomsize /= WORLDSIZE_BUMP;
					if (roomsize < 4.0*defaultradius/MAX_REAL_X)
						roomsize = 4.0*defaultradius/MAX_REAL_X;
					worldbox = Box3(roomsize*MIN_REAL_X,
						roomsize*MIN_REAL_Y, roomsize*MIN_REAL_Z,
						roomsize*MAX_REAL_X, roomsize*MAX_REAL_Y,
						roomsize*MAX_REAL_Z);
					room.Room(worldbox);
					for (i=0; i<MAX_POLYGONS; i++)
 						worldbox.Clamp(position[i]);
					for (i=0; i<MAX_POLYHEDRA; i++)
 						worldbox.Clamp(polyhedronposition[i]);
					worldbox1 = Box4(
						roomsize*MIN_REAL_X,roomsize*MIN_REAL_Y,
						roomsize*MIN_REAL_Z,roomsize*MIN_REAL_W,
						roomsize*MAX_REAL_X,roomsize*MAX_REAL_Y,
						roomsize*MAX_REAL_Z,roomsize*MAX_REAL_W);
					for (i=0; i<MAX_POLYTOPE; i++)
						worldbox1.Clamp(polytopeposition[i]);
					for (i=0; i<MAX_POLYTOPEFRAME; i++)
						worldbox1.Clamp(polytope_position[i]);
					for (i=0; i<MAX_HOUSETOPEFRAME; i++)
						worldbox1.Clamp(housetope_position[i]);

					break;
				case IDM_BOUNCE:
					bounceflag ^= TRUE;
					fixmenuchecks(hwnd);
					break;
				case IDM_PARALLEL:
				case IDM_PERSPECTIVE_TELEPHOTO:
				case IDM_PERSPECTIVE_NORMAL:
				case IDM_PERSPECTIVE_WIDEANGLE:
					camera.SetProjectionType(wParam - IDM_PARALLEL);
					fixmenuchecks(hwnd);
					break;
				case IDM_HYPERCUBE:
					hypercubesolid ^= TRUE;
					showtype = wParam;
					fixmenuchecks(hwnd);
					break;
				case IDM_HYPER_FRAME:
					hypercubeframe ^= TRUE;
					showtype = wParam;
					fixmenuchecks(hwnd);
					break;
				case IDM_SNOW:
					snowtype ^= TRUE;
					showtype = wParam;
					fixmenuchecks(hwnd);
					break;
				case IDM_POLYHEDRA:
					polyhedratype ^= TRUE;
					showtype = wParam;
					fixmenuchecks(hwnd);
					break;
				case IDM_HOUSE_FRAME:
					houseframe ^= TRUE;
					showtype = wParam;
					fixmenuchecks(hwnd);
					break;
				case IDM_ALL:
					showtype = wParam;
					fixmenuchecks(hwnd);
					break;
				case IDM_STANDARD_POS:
					for(i=0;i<MAX_POLYTOPE;i++)
					{
						polytopetumble[i].Identity();
					}
					for(i=0;i<MAX_POLYTOPEFRAME;i++)
					{
						polytope_tumble[i].Identity();
					}
					for(i=0;i<MAX_HOUSETOPEFRAME;i++)
					{
						housetope_tumble[i].Identity();
					}
					break;
				case IDM_XYPLANE:
					for(i=0;i<MAX_POLYTOPE;i++)
					{
						polytopetumble_step[i]=
	  					P12RotationMatrix5((PI/50));
					}
					for(i=0;i<MAX_POLYTOPEFRAME;i++)
					{
						polytope_tumble_step[i] =
						P12RotationMatrix5((PI/50));
					}
					for(i=0;i<MAX_HOUSETOPEFRAME;i++)
					{
						housetope_tumble_step[i] =
						P12RotationMatrix5((PI/50));
					}

					rotationflag = TRUE;
					rotationtype=wParam;
					fixmenuchecks(hwnd);
					break;
				case IDM_XZPLANE:
					for(i=0;i<MAX_POLYTOPE;i++)
					{
					   polytopetumble_step[i]=
					   P13RotationMatrix5((PI/50)); 			
					}
					for(i=0;i<MAX_POLYTOPEFRAME;i++)
					{
					   polytope_tumble_step[i]=
					   P13RotationMatrix5((PI/50)); 			
					}
					for(i=0;i<MAX_HOUSETOPEFRAME;i++)
					{
					   housetope_tumble_step[i]=
					   P13RotationMatrix5((PI/50)); 			
					}

					rotationflag = TRUE;
					rotationtype=wParam;
					fixmenuchecks(hwnd);
					break;
				case IDM_XANAPLANE:
					for(i=0;i<MAX_POLYTOPE;i++)
					{
						polytopetumble_step[i]=
						P14RotationMatrix5((PI/50));
					}
					for(i=0;i<MAX_POLYTOPEFRAME;i++)
					{
						polytope_tumble_step[i]=
						P14RotationMatrix5((PI/50));
					}
					for(i=0;i<MAX_HOUSETOPEFRAME;i++)
					{
						housetope_tumble_step[i]=
						P14RotationMatrix5((PI/50));
					}

					rotationflag = TRUE;
					rotationtype=wParam;
					fixmenuchecks(hwnd);
					break;
				case IDM_YZPLANE:
					for(i=0;i<MAX_POLYTOPE;i++)
					{
						polytopetumble_step[i]=
						P23RotationMatrix5((PI/50));
					}
					for(i=0;i<MAX_POLYTOPEFRAME;i++)
					{
						polytope_tumble_step[i]=
						P23RotationMatrix5((PI/50));
					}
					for(i=0;i<MAX_HOUSETOPEFRAME;i++)
					{
						housetope_tumble_step[i]=
						P23RotationMatrix5((PI/50));
					}

					rotationflag = TRUE;
					rotationtype=wParam;
					fixmenuchecks(hwnd);
					break;
				case IDM_YANAPLANE:
					for(i=0;i<MAX_POLYTOPE;i++)
					{
						polytopetumble_step[i]=
						P24RotationMatrix5((PI/50));
					}
					for(i=0;i<MAX_POLYTOPEFRAME;i++)
					{
						polytope_tumble_step[i]=
						P24RotationMatrix5((PI/50));
					}
					for(i=0;i<MAX_HOUSETOPEFRAME;i++)
					{
						housetope_tumble_step[i]=
						P24RotationMatrix5((PI/50));
					}

					rotationflag = TRUE;
					rotationtype=wParam;
					fixmenuchecks(hwnd);
					break;
				case IDM_ZANAPLANE:
					for(i=0;i<MAX_POLYTOPE;i++)
					{
						polytopetumble_step[i]=
						P34RotationMatrix5((PI/50));
					}
					for(i=0;i<MAX_POLYTOPEFRAME;i++)
					{
						polytope_tumble_step[i]=
						P34RotationMatrix5((PI/50));
					}
					for(i=0;i<MAX_HOUSETOPEFRAME;i++)
					{
						housetope_tumble_step[i]=
						P34RotationMatrix5((PI/50));
					}

					rotationflag = TRUE;
					rotationtype=wParam;
					fixmenuchecks(hwnd);
					break;
				case IDM_RANDOM:
					for(i=0;i<MAX_POLYTOPE;i++)
					{
						polytopetumble_step[i] =
						P12RotationMatrix5((PI/50)*Randomsignreal()) *
						P13RotationMatrix5((PI/50)*Randomsignreal()) *
						P14RotationMatrix5((PI/50)*Randomsignreal()) *
						P23RotationMatrix5((PI/50)*Randomsignreal()) *
						P24RotationMatrix5((PI/50)*Randomsignreal()) *
						P34RotationMatrix5((PI/50)*Randomsignreal()); 
					
					}
					for(i=0;i<MAX_POLYTOPEFRAME;i++)
					{
						polytope_tumble_step[i] =
						P12RotationMatrix5((PI/50)*Randomsignreal()) *
						P13RotationMatrix5((PI/50)*Randomsignreal()) *
						P14RotationMatrix5((PI/50)*Randomsignreal()) *
						P23RotationMatrix5((PI/50)*Randomsignreal()) *
						P24RotationMatrix5((PI/50)*Randomsignreal()) *
						P34RotationMatrix5((PI/50)*Randomsignreal()); 
					}
					for(i=0;i<MAX_HOUSETOPEFRAME;i++)
					{
						housetope_tumble_step[i] =
						P12RotationMatrix5((PI/50)*Randomsignreal()) *
						P13RotationMatrix5((PI/50)*Randomsignreal()) *
						P14RotationMatrix5((PI/50)*Randomsignreal()) *
						P23RotationMatrix5((PI/50)*Randomsignreal()) *
						P24RotationMatrix5((PI/50)*Randomsignreal()) *
						P34RotationMatrix5((PI/50)*Randomsignreal()); 
						
					}

					rotationflag = TRUE;
					rotationtype=wParam;
					fixmenuchecks(hwnd);
					break;
				case IDM_HELP_CONTENTS:
					WinHelp(hwnd, "myhelp.hlp", HELP_CONTENTS, 0L);
					return 0;
					break;
				case IDM_ABOUT:
					DialogBox(ghinst, MAKEINTRESOURCE(IDD_ABOUT), hwnd, (DLGPROC)About);
					break;
				case IDM_EXIT:
					SendMessage( hwnd, WM_DESTROY, 0, 0L ); //Goes to WndProc.
					break;

			}
			return 0;

		case WM_CLOSE:
		case WM_DESTROY:
			delete wbm;
 			delete[] polylist;
			delete[] position;
			delete[] velocity;
			delete[] tumble_step;
			delete[] tumble;
			delete[] aspect;
			delete[] polyhedronlist;
			delete[] polyhedronposition;
			delete[] polyhedronvelocity;
			delete[] polyhedrontumble_step;
			delete[] polyhedrontumble;
			delete[] polyhedronaspect;
			delete[] polytope;
			delete[] polytopetumble;
			delete[] polytopetumble_step;
			delete[] polytopevelocity;
			delete[] polytopeposition;
			delete[] polytopeaspect;
			delete[] visp;
			delete[] polytopeframe;
			delete[] polytope_tumble;
			delete[] polytope_tumble_step;
			delete[] polytope_velocity;
			delete[] polytope_position;
			delete[] polytope_aspect;
			delete[] vispframe;
			delete[] housetopeframe;
			delete[] housetope_tumble;
			delete[] housetope_tumble_step;
			delete[] housetope_velocity;
			delete[] housetope_position;
			delete[] housetope_aspect;
			delete[] house_frame;
			
			
			PostQuitMessage(0);
			return 0;
	}

	return DefWindowProc(hwnd, message, wParam, lParam);
}



//The dialog box
BOOL APIENTRY About( HWND hDlg, UINT message, UINT wParam, LONG lParam)
{
    switch (message)
    {
        case WM_INITDIALOG:                
            return (TRUE);

        case WM_COMMAND:                      
            if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
            {        
                EndDialog(hDlg, TRUE);        
                return TRUE;
            }
            break;
    }
    return FALSE;                           

}


void mydraw(HWND hwnd)
{
	
 	Rect screenrect;
    static Point upperleft(0,0);
    Poly3 currpoly;
	Polyhedron3 currpolyhedron;
	Vector3 currcentroid;
	Matrix4 view_times_aspect;

	int i;
	//fourth dimension
	Polyhedron3 temp3=visp[0];
	Polyhedron3 temp2=vispframe[0];
	Polytope4 temp4;
	
	if (pauseflag)
		return;

    wbm->Clear();
	segment.Empty();
	if (snowtype || showtype == IDM_ALL)
		for (i=0; i<MAX_POLYGONS; i++)
     	{
			if (!pauseflag)
			{
				if(bounceobject)
				{
					if (bounceflag)
						worldbox.AddBounce(position[i], velocity[i]);
					else //wrap
					{
						position[i] += velocity[i];
						worldbox.Wrap(position[i]);
					}
				}
				tumble[i] = tumble_step[i] * tumble[i]; //Accumulate
			}
			aspect[i].SetTranslationRotation(position[i],tumble[i]);
			view_times_aspect = camera.View() * aspect[i];
			 /*This matrix is where the action is.  aspect[i] moves the
     	      poly into it's world position; camera.View() converts its
          	 world coordinates into camera coordinates.  We save time by
	           combining matrices.*/
			currcentroid = view_times_aspect * polylist[i].Centroid();
			/*Before processing all the poly's vert's just do a clip test
			on it's centroid.  If the centroid is too close to the eye or
			too far out of the clipping pyramid, then ClipTest is FALSE*/
			if (camera.SphereClipTest(currcentroid, polylist[i].Radius()))
			{
				currpoly = view_times_aspect * polylist[i];
				camera.Project(currpoly);
				currpoly.RealToPixel(frame);
				segment.Post(currpoly);
			}
	     } //end of for polygon for-loop.
	if (polyhedratype || showtype == IDM_ALL)
		for (i=0; i<MAX_POLYHEDRA; i++)
     	{
			if (!pauseflag)
			{
				if(bounceobject)
				{
					if (bounceflag)
						worldbox.AddBounce(polyhedronposition[i],
						polyhedronvelocity[i]);
					else //wrap
					{
						polyhedronposition[i] += polyhedronvelocity[i];
						worldbox.Wrap(polyhedronposition[i]);
					}
				}
				polyhedrontumble[i] = polyhedrontumble_step[i] *
					polyhedrontumble[i]; //Accumulate
			}
			polyhedronaspect[i].SetTranslationRotation(
				polyhedronposition[i],polyhedrontumble[i]);
			view_times_aspect = camera.View() * polyhedronaspect[i];
			 /*This matrix is where the action is.  aspect[i] moves the
     	      poly into it's world position; camera.View() converts its
          	 world coordinates into camera coordinates.  We save time by
	           combining matrices.*/
			currcentroid = view_times_aspect * polyhedronlist[i].Centroid();
			/*Before processing all the poly's vert's just do a clip test
			on it's centroid.  If the centroid is too close to the eye or
			too far out of the clipping pyramid, then ClipTest is FALSE*/
			if (camera.SphereClipTest(currcentroid,
				polyhedronlist[i].Radius()))
			{
				currpolyhedron = view_times_aspect * polyhedronlist[i];
				camera.Project(currpolyhedron);
				currpolyhedron.RealToPixel(frame);
				camera.BackFaceCull(currpolyhedron);
				segment.Post(currpolyhedron);
			}
	     } //end of polyhedron for-loop.

if( showtype == IDM_ALL||hypercubesolid)
{
	for(i=0;i<MAX_POLYTOPE ;i++)
	{
		if (!pauseflag)
		{
			if(bounceobject)
			{
				if (bounceflag)
					worldbox1.AddBounce(polytopeposition[i],
					polytopevelocity[i]);
				else //wrap
				{
					polytopeposition[i] += polytopevelocity[i];
					worldbox1.Wrap(polytopeposition[i]);
				}
			}
			polytopetumble[i] = polytopetumble_step[i] *
				polytopetumble[i]; //Accumulate
		}
		polytopeaspect[i].SetTranslationRotation(
			polytopeposition[i],polytopetumble[i]);

		temp4 = polytopeaspect[i] * polytope[i];
		StandardProject(temp4,visp[i]);
		visp[i]._setcentroid();
		visp[i]._setfacecentroid();
		visp[i]._setfacenormal();
		temp3 = camera.View() * visp[i];
		camera.Project(temp3);
		temp3.RealToPixel(frame);
		camera.InternalFaceCull(temp3);
		segment.Post(temp3);
	}//for loop

}

if(hypercubeframe||showtype == IDM_ALL)
{
	for(i=0;i<MAX_POLYTOPEFRAME ;i++)
	{
		if (!pauseflag)
		{
			if(bounceobject)
			{
				if (bounceflag)
					worldbox1.AddBounce(polytope_position[i],
					polytope_velocity[i]);
				else //wrap
				{
					polytope_position[i] += polytope_velocity[i];
					worldbox1.Wrap(polytope_position[i]);
				}
			}
			polytope_tumble[i] = polytope_tumble_step[i] *
				polytope_tumble[i]; //Accumulate
			
		}
		polytope_aspect[i].SetTranslationRotation(
			polytope_position[i],polytope_tumble[i]);
		temp4 = polytope_aspect[i] * polytopeframe[i];
		StandardProject(temp4,vispframe[i]);
		vispframe[i]._setcentroid();
		vispframe[i]._setfacecentroid();
		vispframe[i]._setfacenormal();
		temp2 = camera.View() * vispframe[i];
		camera.Project(temp2);
		temp2.RealToPixel(frame);
		segment.Post(temp2);
	}//for loop
}
//4D house

if(houseframe||showtype == IDM_ALL)
{
	for(i=0;i<MAX_HOUSETOPEFRAME ;i++)
	{
		if (!pauseflag)
		{
			if(bounceobject)
			{
				if (bounceflag)
					worldbox1.AddBounce(housetope_position[i],
					housetope_velocity[i]);
				else //wrap
				{
					housetope_position[i] += housetope_velocity[i];
					worldbox1.Wrap(housetope_position[i]);
				}
			}
			housetope_tumble[i] = housetope_tumble_step[i] *
				housetope_tumble[i]; //Accumulate
		}
		housetope_aspect[i].SetTranslationRotation(
			housetope_position[i],housetope_tumble[i]);
		temp4 = housetope_aspect[i] * housetopeframe[i];
		StandardProject(temp4,house_frame[i]);
		house_frame[i]._setcentroid();
		house_frame[i]._setfacecentroid();
		house_frame[i]._setfacenormal();
		temp2 = camera.View() * house_frame[i];
		camera.Project(temp2);
		temp2.RealToPixel(frame);
		segment.Post(temp2);
	}//for loop
}
	

	if (showroomflag)
	{
		currpolyhedron = camera.View() * room;
			camera.ZClipTest(currpolyhedron);
		camera.Project(currpolyhedron);
		currpolyhedron.RealToPixel(frame);
		camera.BackFaceCull(currpolyhedron);
		segment.RoomPost(currpolyhedron); //Show NO_COLOR culled faces
	}
	segment.Sort();
	segment.Draw(gdibox);
	InvalidateRect(hwnd, NULL, FALSE); //CAll WM_PAINT for a bitblt
}

int GCD(int a, int b)
{
	if (a < b)
	{
		int temp = a;
		a = b;
		b = temp;
	}
	if (!b)
		return a;
	int q = a/b;
	int r = a - b*q;
	return GCD(q,r);
}

Point next_rel_prime_pair(Point p)
{
/*Start this out with p = Point(5,2), the first relatively prime pair,
and move on from there.  Rule is to bump y looking for GCD(y,x) = 1 up
until y >= x/2, and to then bump x and start y back at 2.  Actually if
count is large like 17 and stepis 2 or 3, then the star looks like a
hollow round doily, which is pretty nice, but if the step is like 7, then
the star looks all white and messy.*/
	int newcount = p.x();
	int newstep = p.y();

	while (TRUE) //Stay in here until a return bails you out.
	{
		newstep++;
//#define AVOID_ASTERISK
#ifndef AVOID_ASTERISK
		if (newstep >= newcount/2)
#else  //AVOID_ASTERISK
		if (newstep >= newcount/3)
#endif //AVOID_ASTERISK
		{
			newcount++;
//#define AVOID_DOILY
#ifndef AVOID_DOILY
			newstep = 2;
#else //AVOID_DOILY
			newstep = (newcount/4) + 1;  // was 2;
			if (newstep == 1)
				newstep++;
#endif //AVOID_DOILY
		}
		if (GCD(newcount, newstep) == 1)
			return Point(newcount, newstep);
	}
}


void fixmenuchecks(HWND hwnd)
{
	HMENU hmenu = GetMenu(hwnd);
	for (int i=IDM_PARALLEL; i<=IDM_PERSPECTIVE_WIDEANGLE; i++)
		CheckMenuItem(hmenu, i, MF_UNCHECKED);
	CheckMenuItem(hmenu, IDM_PARALLEL + camera.ProjectionType(), MF_CHECKED);
	
	for ( i=IDM_XYPLANE; i<=IDM_RANDOM; i++)
		CheckMenuItem(hmenu, i, MF_UNCHECKED);
	if(rotationflag)
		CheckMenuItem(hmenu,rotationtype , MF_CHECKED);

	for(i=IDM_HYPERCUBE;i<=IDM_ALL;i++)
		CheckMenuItem(hmenu, i, MF_UNCHECKED);
	CheckMenuItem(hmenu, showtype, MF_CHECKED);

	if (hypercubesolid)
		CheckMenuItem(hmenu, IDM_HYPERCUBE, MF_CHECKED);
	else
		CheckMenuItem(hmenu, IDM_HYPERCUBE, MF_UNCHECKED);
	if (hypercubeframe)
		CheckMenuItem(hmenu, IDM_HYPER_FRAME, MF_CHECKED);
	else
		CheckMenuItem(hmenu, IDM_HYPER_FRAME, MF_UNCHECKED);
	if (houseframe)
		CheckMenuItem(hmenu, IDM_HOUSE_FRAME, MF_CHECKED);
	else
		CheckMenuItem(hmenu, IDM_HOUSE_FRAME, MF_UNCHECKED);

	if (snowtype)
		CheckMenuItem(hmenu, IDM_SNOW, MF_CHECKED);
	else
		CheckMenuItem(hmenu, IDM_SNOW, MF_UNCHECKED);
	if (polyhedratype)
		CheckMenuItem(hmenu, IDM_POLYHEDRA, MF_CHECKED);
	else
		CheckMenuItem(hmenu, IDM_POLYHEDRA, MF_UNCHECKED);

	if (showtype == IDM_ALL)
	{
		CheckMenuItem(hmenu, IDM_HYPERCUBE, MF_UNCHECKED);
		CheckMenuItem(hmenu, IDM_HYPER_FRAME, MF_UNCHECKED);
		CheckMenuItem(hmenu, IDM_HOUSE_FRAME, MF_UNCHECKED);
		CheckMenuItem(hmenu, IDM_SNOW, MF_UNCHECKED);
		CheckMenuItem(hmenu, IDM_POLYHEDRA, MF_UNCHECKED);
		hypercubeframe = FALSE;
		hypercubesolid = FALSE;
		houseframe = FALSE;
		snowtype = FALSE;
		polyhedratype = FALSE;
	}
	
	if (pauseflag)
		CheckMenuItem(hmenu, IDM_PAUSE, MF_CHECKED);
	else
		CheckMenuItem(hmenu, IDM_PAUSE, MF_UNCHECKED);
	if(bounceobject)
		CheckMenuItem(hmenu, IDM_BOUNCEOBJECT, MF_CHECKED);
	else
		CheckMenuItem(hmenu, IDM_BOUNCEOBJECT, MF_UNCHECKED);
	if (showroomflag)
		CheckMenuItem(hmenu, IDM_WORLDBOX, MF_CHECKED);
	else
		CheckMenuItem(hmenu, IDM_WORLDBOX, MF_UNCHECKED);
	if (bounceflag)
		CheckMenuItem(hmenu, IDM_BOUNCE, MF_UNCHECKED);
	else
		CheckMenuItem(hmenu, IDM_BOUNCE, MF_CHECKED);
}


void resetcamera()
{
	roomsize=1.0;//needed to reset the camera
	Vector3 starteye, startfrom, startup;
	//In the following line 2.0 is changed to 1.0 to get a closer view
	starteye = 1.0*Vector3(roomsize*MAX_REAL_X,
		 2.0*roomsize*MIN_REAL_Y, roomsize*MAX_REAL_Z);
	startfrom = starteye;
	startfrom.Normalize();
	startup = (startfrom *     //like z*(y*z), like z*x.
		(Vector3(0.0, 1.0, 0.0) * startfrom)).Normalize();
	camera.SetEyeFromUp(starteye, startfrom, startup);
	camera.SetProjectionType(PERSPECTIVE_TELEPHOTO);

	for(int j=0;j<2;j++)//will make the worldbox twice smaller
	{
		roomsize /= WORLDSIZE_BUMP;
		if (roomsize < 4.0*defaultradius/MAX_REAL_X)
			roomsize = 4.0*defaultradius/MAX_REAL_X;
		worldbox = Box3(roomsize*MIN_REAL_X,
				roomsize*MIN_REAL_Y, roomsize*MIN_REAL_Z,
				roomsize*MAX_REAL_X, roomsize*MAX_REAL_Y,
				roomsize*MAX_REAL_Z);
		room.Room(worldbox);
		for (int i=0; i<MAX_POLYGONS; i++)
 			worldbox.Clamp(position[i]);
		for (i=0; i<MAX_POLYHEDRA; i++)
 			worldbox.Clamp(polyhedronposition[i]);
		worldbox1 = Box4(
				roomsize*MIN_REAL_X,roomsize*MIN_REAL_Y,
				roomsize*MIN_REAL_Z,roomsize*MIN_REAL_W,
				roomsize*MAX_REAL_X,roomsize*MAX_REAL_Y,
				roomsize*MAX_REAL_Z,roomsize*MAX_REAL_W);
		for (i=0; i<MAX_POLYTOPE; i++)
			worldbox1.Clamp(polytopeposition[i]);
		for (i=0; i<MAX_POLYTOPEFRAME; i++)
			worldbox1.Clamp(polytope_position[i]);
		for (i=0; i<MAX_HOUSETOPEFRAME; i++)
			worldbox1.Clamp(housetope_position[i]);
	}
}
