/***********************************************************
* Project:	CS298                                          *
* File:		polyhedron4.cpp                                *
* Purpose:	This file holds information about 2D, 3D, 4D   *
*			objects.The bounding box is also coded in here.*
************************************************************/

#include "polyhedron4.hpp"
#include <math.h>

//----------------------TWO DIMENSIONAL---------------------
Poly2::Poly2(const Poly2 &poly)
{
	_pointvert = new Point[poly._count];
	assert(_pointvert);
	_vector2vert = new Vector2[poly._count];
	assert(_vector2vert);
	_count = poly._count;
	for (int i=0; i<_count; i++)
	{
		_pointvert[i] = poly._pointvert[i];
		_vector2vert[i] = poly._vector2vert[i];
	}
	_centroid = poly._centroid;
	_radius = poly._radius;
	_closed = poly._closed;
	_fillcolor = poly._fillcolor;
	_transparent = poly._transparent;
}

Poly2::~Poly2()
{
	if (_vector2vert)
		delete [] _vector2vert;
	if (_pointvert)
		delete [] _pointvert;
}

void Poly2::_setcentroid()
{
	Real distance;

	_centroid = Vector2(0.0, 0.0);
	_radius = 0.0;
     for (int i=0; i<_count; i++)
     	_centroid += _vector2vert[i];
     _centroid /= _count;
	for (i=0; i<_count; i++)
	{
		distance = (_vector2vert[i] - _centroid).Magnitude();
		if (distance > _radius)
			_radius = distance;
	}
}

void Poly2::Resize(int newcount)
{ //If you call this with newcount 0, it empties the polygon out.
	Point *newpointvert = NULL;
	Vector2 *newvector2vert = NULL;
	if (newcount)
	{
		newpointvert = new Point[newcount];
		assert(newpointvert);
		newvector2vert = new Vector2[newcount];
		assert(newvector2vert);
	}
	int copynumber = (_count>newcount)?newcount:_count;  //min
	for (int i=0; i<copynumber; i++)
	{
		newpointvert[i] =_pointvert[i];
		newvector2vert[i] =_vector2vert[i];
	}
	if (_vector2vert)
		delete [] _vector2vert;
	if (_pointvert)
		delete [] _pointvert;
	_vector2vert = newvector2vert;
	_pointvert = newpointvert;
	_closed = FALSE;
	_count = newcount;
}

void Poly2::RegularPolygon(int vertexcount, Vector2 center, Real radius,
	Real initangle)
{
	Real angle = initangle;

     Resize(vertexcount);
     for (int i=0; i<vertexcount; i++)
     {
     	_vector2vert[i] = center + radius*Vector2(cos(angle), sin(angle));
          angle += (2*PI)/vertexcount;
     }
     _closed = TRUE;
	_centroid = center;
	_radius = radius;
}

void Poly2::StarPolygon(int vertexcount, int step, Vector2 center,
	Real radius, Real initangle)
{
	Real angle;
	int i = 0, stepi = 0;

	if (step < 1)
		step = 1; //Avoid endless loop you'd get if step == 0.
     Resize(vertexcount);
	while (!i || stepi)
	/* Do the while loop for the first step (when !i) and for any
     succeding step where stepi is not yet divisible by vertexcount*/
     {
		angle = initangle + (stepi * 2.0 * PI)/vertexcount;
     	_vector2vert[i] = center + radius*Vector2(cos(angle), sin(angle));
		i++;
          stepi = (i*step)%vertexcount;
     }
	Resize(i); //There are i actual vertexes that got used.
     _closed = TRUE;
	_centroid = center;
	_radius = radius;
}

Poly2 Poly2::operator=(const Poly2 &poly)
{
	if (this == &poly)
		return *this;
	Resize(poly._count);
	for (int i=0; i<_count; i++)
	{
		_pointvert[i] = poly._pointvert[i];
		_vector2vert[i] = poly._vector2vert[i];
	}
	_closed = poly._closed;
	_fillcolor = poly._fillcolor;
	_transparent = poly._transparent;
	_centroid = poly._centroid;
	_radius = poly._radius;
	return *this;
}

Poly2 Poly2::operator+(const Point &pix)
{
	Poly2 newpoly;

	newpoly = *this;
	newpoly.Resize(newpoly._count + 1);
	newpoly._pointvert[newpoly._count - 1] = pix;
	newpoly._closed = FALSE;
	newpoly._setcentroid();
	return newpoly;
}

Poly2 Poly2::operator+=(const Point &pix)
{
	Resize(_count + 1);
	_pointvert[_count-1] = pix;
	_closed = FALSE;
	return *this;
}

Poly2 Poly2::operator+(const Vector2 &vect)
{
	Poly2 newpoly;

	newpoly = *this;
	newpoly.Resize(newpoly._count + 1);
	newpoly._vector2vert[newpoly._count - 1] = vect;
	newpoly._closed = FALSE;
     newpoly._setcentroid();
	return newpoly;
}

Poly2 Poly2::operator+=(const Vector2 &vect)
{
	Resize(_count + 1);
	_vector2vert[_count-1] = vect;
	_closed = FALSE;
     _setcentroid();
	return *this;
}

void Poly2::Draw(GDIToolbox &gdibox)
{
	if (!_count)
		return;
	if (_count == 1)
	{
		gdibox.Marker(_pointvert[0]);
		return;
	}
	if (_closed)
  {
		gdibox.SetFillColor(_fillcolor);
		if (_transparent || _fillcolor == TRANSPARENT)
			gdibox.SetFillStyle(BS_HOLLOW);
		else
			gdibox.SetFillStyle(BS_SOLID);
		gdibox.Polygon(_count, _pointvert);
  }
	else
		gdibox.Polyline(_count, _pointvert);
}


Point Poly2::PointVert(int n)
{
	if (n>=_count)
  		return Point(0,0);
	else
  		return _pointvert[n];
}
//----------------------THREE DIMENSIONAL---------------------
Poly3::Poly3(const Poly3 &poly):_projectpoly(poly._projectpoly)
{
	_vector3vert = new Vector3[Count()];
	assert(_vector3vert);
	for (int i=0; i<Count(); i++)  //Don't copy _vector2vert or _pointvert
		_vector3vert[i] = poly._vector3vert[i];
	_centroid = poly._centroid;
	_radius = poly._radius;
}

void Poly3::SegmentCopy(const Poly3 &poly)
{/*Only copy the informatoin the Segment needs to Sort and to Show
the Poly3.  That is, just copy _count, _pointvert, _centroid, _closed,
and _filcolor.*/
	_projectpoly._count = poly.Count();
	_projectpoly._pointvert = new Point[Count()];
	assert(_projectpoly._pointvert);
	for (int i=0; i<Count(); i++)
		_projectpoly._pointvert[i] = poly._projectpoly._pointvert[i];
	_centroid = poly._centroid;
	SetClosed(poly.Closed());
	SetFillColor(poly.FillColor());
}

void Poly3::SegmentCopy(const Polyhedron3 &poly, int faceindex)
{/*Like SegmentCopy(Poly3 poly) above, except here we pick out the Poly3
info from the rats-nest of the Polyhedron3.*/
	_projectpoly._count = poly._facevertcount[faceindex];
	_projectpoly._pointvert = new Point[Count()];
	assert(_projectpoly._pointvert);
	for (int i=0; i<Count(); i++)
		_projectpoly._pointvert[i] =
			poly._pointvert[poly._face[faceindex][i]];
/*Instead of doing the logical thing, we could do a sneaky thing ,
assuming that _facedistance has been set to the desired
distance from the camera's eye to the facecentroid in Camera::BackFaceCull
	_centroid = Vector3(0.0, 0.0, poly._facedistance[faceindex]);*/
	_centroid = poly._facecentroid[faceindex];
	SetClosed(TRUE);
	SetFillColor(poly.FillColor(faceindex));
}

Poly3::~Poly3()
{
	if (_vector3vert)
		delete [] _vector3vert;
}

void Poly3::_setcentroid()
{
	Real distance;

	_centroid = Vector3(0.0, 0.0, 0.0);
	_radius = 0.0;
     for (int i=0; i<Count(); i++)
     	_centroid += _vector3vert[i];
     _centroid /= Count();
	for (i=0; i<Count(); i++)
	{
		distance = (_vector3vert[i] - _centroid).Magnitude();
		if (distance > _radius)
			_radius = distance;
	}
}

void Poly3::Resize(int newcount)
{ //If you call this with newcount 0, it empties the polygon out.
	Vector3 *newvector3vert = NULL;
	int oldcount = Count();

	_projectpoly.Resize(newcount);
	if (newcount)
	{
		newvector3vert = new Vector3[newcount];
		assert(newvector3vert);
	}
	int copynumber = (oldcount>newcount)?newcount:oldcount;  //min
	for (int i=0; i<copynumber; i++)
		newvector3vert[i] =_vector3vert[i];
	if (_vector3vert)
		delete [] _vector3vert;
	_vector3vert = newvector3vert;
}

void Poly3::RegularPolygon(int vertexcount, Vector3 center, Real radius,
	Real initangle)
{
	Real angle = initangle;

     Resize(vertexcount);
     for (int i=0; i<vertexcount; i++)
     {
     	_vector3vert[i] = center +
			radius*Vector3(cos(angle), sin(angle), 0.0);
          angle += (2*PI)/vertexcount;
     }
     SetClosed(TRUE);
	_centroid = center;
	_radius = radius;
}

void Poly3::StarPolygon(int vertexcount, int step, Vector3 center,
	Real radius, Real initangle)
{
	Real angle;
	int i = 0, stepi = 0;

	if (step < 1)
		step = 1; //Avoid endless loop you'd get if step == 0.
     Resize(vertexcount);
	while (!i || stepi)
	/* Do the while loop for the first step (when !i) and for any
     succeding step where stepi is not yet divisible by vertexcount*/
     {
		angle = initangle + (stepi * 2.0 * PI)/vertexcount;
     	_vector3vert[i] = center +
			radius*Vector3(cos(angle), sin(angle), 0.0);
		i++;
          stepi = (i*step)%vertexcount;
     }
	Resize(i); //There are i actual vertexes that got used.
     SetClosed(TRUE);
	_centroid = center;
	_radius = radius;
}

Poly3 Poly3::operator=(const Poly3 &poly)
{
 	if (this == &poly)
		return *this;
	Resize(poly.Count());
	for (int i=0; i<Count(); i++)
	// don't bother copying _pointvert and _vector2vert.
		_vector3vert[i] = poly._vector3vert[i];
	SetClosed(poly.Closed());
	SetFillColor(poly.FillColor());
	_centroid = poly._centroid;
	_radius = poly._radius;
	return *this;
}

Poly3 Poly3::operator+=(const Vector3 &vect)
{
	Resize(Count() + 1);
	_vector3vert[Count()-1] = vect;
	SetClosed(FALSE);
     _setcentroid();
	return *this;
}

void Poly3::Draw(GDIToolbox &gdibox)
{
	_projectpoly.Draw(gdibox);
}

//====================FRAME Methods===============================
BOOL Poly2::RealToPixel(const Frame &frame)
{
	BOOL rval = TRUE;
	int px, py;

	for (int i = 0; i<_count; i++)
	{
		if (!frame.RealToPixel(_vector2vert[i].x(), _vector2vert[i].y(), &px, &py))
			rval = FALSE;
		else
			_pointvert[i].Set(px, py);
	}
	return rval;
}

BOOL Poly3::RealToPixel(const Frame &frame)
{
	return _projectpoly.RealToPixel(frame);
}

BOOL Polyhedron3::RealToPixel(const Frame &frame)
{
	BOOL rval = TRUE;
	int px, py;

	for (int i = 0; i<_count; i++)
	{
		if (!frame.RealToPixel(_vector2vert[i].x(), _vector2vert[i].y(), &px, &py))
			rval = FALSE;
		else
			_pointvert[i].Set(px, py);
	}
	return rval;
}

//----------Matrix4 friend operations----------------
/* The following  friend operations are used to multiply
   in the matrix on the left, as in V = M * U for U and V vectors.*/


Poly2 operator*(const Matrix3 &M, const Poly2 &p)
{
	Poly2 Mp(p); //copy constructor

     Mp._centroid = M * Mp._centroid;
     for (int i=0; i<Mp._count; i++)
     	Mp._vector2vert[i] = M * Mp._vector2vert[i];
     return Mp;
}

Poly3 operator*(const Matrix4 &M, const Poly3 &p)
{
	Poly3 Mp(p); //copy constructor

     Mp._centroid = M * Mp._centroid;
     for (int i=0; i<Mp.Count(); i++)
     	Mp._vector3vert[i] = M * Mp._vector3vert[i];
     return Mp;
}

Polyhedron3 operator*(const Matrix4 &M, const Polyhedron3 &p)
{
	Polyhedron3 Mp(p); //copy constructor

     Mp._centroid = M * Mp._centroid;
     for (int i=0; i<Mp._count; i++)
     	Mp._vector3vert[i] = M * Mp._vector3vert[i];
     for (i=0; i<Mp._facecount; i++)
     	Mp._facecentroid[i] = M * Mp._facecentroid[i];
    	Mp._setfacenormal();
     return Mp;
}

//=====================Fill colors====================

void Poly3::RandomizeFaceColor(HWND hwnd)
{
	COLORREF colorref;
	HDC hdc = GetDC(hwnd);
	
	int rgb = Random(3);
	switch(rgb)
	{
		case 0:
			colorref = RGB(128+Random(128),
				Random(2)?256:Random(256),
				Random(2)?256:Random(256));
			break;
		case 1:
			colorref = RGB(Random(2)?256:Random(256),
				128+Random(128),
				Random(2)?256:Random(256));
			break;
		case 2:
			colorref = RGB(Random(2)?256:Random(256),
				Random(2)?256:Random(256),
				128+Random(128));
			break;
	}
	colorref = GetNearestColor(hdc, colorref);
	_projectpoly._fillcolor = colorref;
	ReleaseDC(hwnd, hdc);
}

void Polyhedron3::RandomizeFaceColor(HWND hwnd)
{
	COLORREF colorref;
	HDC hdc = GetDC(hwnd);
	
	for (int i=0; i< _facecount; i++)
	{
		int rgb = Random(3);
		switch(rgb)
		{
			case 0:
				colorref = RGB(128+Random(128),
					Random(2)?256:Random(256),
					Random(2)?256:Random(256));
				break;
			case 1:
				colorref = RGB(Random(2)?256:Random(256),
					128+Random(128),
					Random(2)?256:Random(256));
				break;
			case 2:
				colorref = RGB(Random(2)?256:Random(256),
					Random(2)?256:Random(256),
					128+Random(128));
				break;
		}
		colorref = GetNearestColor(hdc, colorref);
		_fillcolor[i] = colorref;
	}
	ReleaseDC(hwnd, hdc);
}



//================FOUR DIMENSIONAL STUFFS ============================================

void Polytope4::_setcentroid()
{
	Real distance;

	_centroid = Vector4(0.0,0.0, 0.0, 0.0);
	_radius = 0.0;
     for (int i=0; i<Count(); i++)
     	_centroid += _vector4vert[i];
     _centroid /= Count();
	for (i=0; i<Count(); i++)
	{
		distance = (_vector4vert[i] - _centroid).Magnitude();
		if (distance > _radius)
			_radius = distance;
	}
}




Polytope4 operator*(const Matrix5 &M, const Polytope4 &p)
{
	Polytope4 Mp(p); //copy constructor

     Mp._centroid = M * Mp._centroid;
     for (int i=0; i<Mp._count; i++)
     	Mp._vector4vert[i] = M * Mp._vector4vert[i];
     for (i=0; i<Mp._facecount; i++)
     	Mp._facecentroid[i] = M * Mp._facecentroid[i];
	 Mp._setsolidnormal();
	 //to rotate the vertedge vectors of the polytope with the polytope.
	 for(i=0; i<Mp._edgecount; i++)
	 {
		 for(int j=0; j<Mp._vertedgecount[i]; j++)
		 	 Mp._vertedge[i][j]=M*Mp._vertedge[i][j];
	 }
     return Mp;
}


