/***********************************************************
* Project:	CS298                                          *
* File:		polyhedron4.hpp                                *
* Purpose:	This file holds information about 2D, 3D, 4D   *
*			objects.The bounding box is also coded in here.*
************************************************************/

#ifndef POLYGON_HPP
#define POLYGON_HPP
#include "types.h" //For Real, and for <windows.h> for RGB
#include "point2.hpp"
#include "vector5.hpp"
#include "GDIToolbox1.hpp"
#include "box2.hpp"
#include "frame1.hpp"

//--------------TWO DIMENSIONAL--------------
class Poly2
{
private:
	int _count;
	Point *_pointvert;
	Vector2 *_vector2vert;
    Vector2 _centroid;
	Real _radius;
	BOOL _closed;
	COLORREF _fillcolor;
	BOOL _transparent;
    void _setcentroid(); //helper function.
//So the Frame RealToPixel and PixelToReal functions can use the private fields.
	friend class Frame;
//So Poly3 can maintain its _projectpoly
	friend class Poly3;
//So Camera's Project functions can project Poly3 objects onto _projectpolys
	friend class Camera;
public:
//Constructor and Destructor
	Poly2():_count(0),_pointvert(NULL),_vector2vert(NULL),_closed(FALSE),
	_fillcolor(RGB(255, 0, 0)), _transparent(FALSE), _centroid(0.0,0.0), _radius(0.0){}
	Poly2(const Poly2 &poly);
	~Poly2();
//Mutators
	void SetClosed(BOOL yesno){_closed = yesno;}
	void Resize(int maxcount);//Allocates arrays, preserves old info.
	void SetFillColor(COLORREF col){_fillcolor = col;}
	void SetTransparent(BOOL yesno){_transparent = yesno;}
		/*If you don't specify the center, radius, initangle arguments for
		RegularPolygon & StarPolygon, use default arguments */
     void RegularPolygon(int vertexcount,
		Vector2 center = Vector2(0.0, 0.0), Real radius = 1.0,
     	Real initangle = 0.0);
     void StarPolygon(int vertexcount, int step,
		Vector2 center = Vector2(0.0, 0.0), Real radius = 1.0,
     	Real initangle = 0.0);
//Accessors
	BOOL Closed() const {return _closed;}
	BOOL Transparent() const {return _transparent;}
	int Count() const {return _count;}  /*the const means the
		funtion doesn't change the class. I need this because some
		functions that	call (const Poly3 &p) use the Poly2::Count function.*/
	COLORREF FillColor() const {return _fillcolor;}
	Point PointVert(int n);
	Vector2 Centroid()const {return _centroid;}
	Real Radius(){return _radius;}
 //Overloaded operators for making free polygons by adding points.
	Poly2 operator=(const Poly2 &poly);
	Poly2 operator+(const Point &p);
	Poly2 operator+(const Vector2 &v);
	Poly2 operator+=(const Point &p);
	Poly2 operator+=(const Vector2 &v);
//Linear algebra
     friend Poly2 operator*(const Matrix3 &M, const Poly2 &p);
//Methods
	void Draw(GDIToolbox &gdibox);
	BOOL RealToPixel(Frame const &frame);
};

//--------------THREE DIMENSIONAL--------------
class Segment;
class Polyhedron3;

class Poly3
{
private:
	Poly2 _projectpoly; //Use the _count, _closed, _fillcolor in here.
	Vector3 *_vector3vert;
	Vector3 _centroid;
	Real _radius;
	BOOL _visible;
	void _setcentroid(); //helper function.
	friend class Frame;
	friend class Camera;
	friend class Segment;
public:
//Constructor and Destructor
	Poly3():_vector3vert(NULL),_centroid(0.0,0.0,0.0),
		_radius(0.0), _visible(TRUE){} //_projectpoly() gets called automatically.
	Poly3(const Poly3 &poly);
	~Poly3();
	void SegmentCopy(const Poly3 &poly);
	//Just copy _centroid and drawing info; make a "lightweight" copy.
	void SegmentCopy(const Polyhedron3 &poly, int faceindex);
	 /*Make a lightweight copy of the faceindex_th Poly3 that is
	implicityly coded up in the rats-nest of poly.*/
//Mutators
	void SetClosed(BOOL yesno){_projectpoly.SetClosed(yesno);}
	void Resize(int maxcount);//Allocates arrays, preserves old info.
	void SetFillColor(int col){_projectpoly.SetFillColor(col);}
	void SetTransparent(BOOL yesno){_projectpoly.SetTransparent(yesno);}
		/*If you don't specify the center, radius, initangle arguments for
		RegularPolygon & StarPolygon, use default arguments */
     void RegularPolygon(int vertexcount,
		Vector3 center = Vector3(0.0, 0.0, 0.0), Real radius = 1.0,
     	Real initangle = 0.0);
     void StarPolygon(int vertexcount, int step,
		Vector3 center = Vector3(0.0, 0.0, 0.0), Real radius = 1.0,
     	Real initangle = 0.0);
	void RandomizeFaceColor(HWND hwnd);
//Accessors
	BOOL Closed()const {return _projectpoly.Closed();}
	int Count() const {return _projectpoly.Count();} /*the const means the
		funtion doesn't change the class. I need this because some
		functions that	call (const Poly3 &p) use the Count function.*/
	int FillColor() const {return _projectpoly.FillColor();}
	Vector3 Centroid() const {return _centroid;}
	Real Radius(){return _radius;}
	Poly3 operator=(const Poly3 &poly);
	BOOL Visible(){return _visible;}
//Overloaded operator for adding points to a Poly3.
	Poly3 operator+=(const Vector3 &v);
//Linear algebra
     friend Poly3 operator*(const Matrix4 &M, const Poly3 &p);
//Methods
	void Draw(GDIToolbox &gdibox);
	BOOL RealToPixel(Frame const &frame);
};

class Polyhedron3
{
private://The rats-nest of vertices.
	int _count;
	Vector3 *_vector3vert;
	Vector2 *_vector2vert;
	Point *_pointvert;
	Vector3 _centroid;
	Real _radius;

/*The face info for _facecount faces, each with _facevertcount[i] vertices,
which are stored as index numbers into the rats-nest info.  So _face is
an array of arrays of index ints.*/
	int _facecount;
	int *_facevertcount;
	int **_face;
	COLORREF *_fillcolor;
	Vector3 *_facecentroid;
	Vector3 *_facenormal;
	BOOL *_facevisible; //Set these with Camera.BackFaceCull(Polyhedron 3).
	Real *_facedistance;
//Edge information
	int _edgecount;
	int *_vertedgecount;
	Vector3 **_vertedge;
	Real **BarycentricCoord;
	BOOL *_internalvert;
	BOOL *_internalface;

//Extra stuff.
	void _freememory(); //Used by destructor and Size operator.
	friend class Frame;
	friend class Camera;
	friend class Segment;
	friend class Poly3;
	
public:
//Constructor and Destructor
	Polyhedron3();
	Polyhedron3(const Polyhedron3 &poly);
	~Polyhedron3();
//Mutators
	void Size(int vertcount, int facecount, int *facevertcount, int edgecount, int *vertedgecount);//Allocates arrays, doesn't preserve old info.
	void SetFillColor(COLORREF col, int faceindex){_fillcolor[faceindex] = col;}
		/*If you don't specify the center, radius, initangle arguments for
		Cube use default arguments */
    void Cube(Real radius = 1.0);
    void Dodecahedron(Real radius = 1.0);
	void ProjectHyperCube(Real radius);
	void ProjectHyperCube1(Real radius);
	void ProjectHouse(Real radius);
	void ProjectHouse1(Real radius);
	void Room(const Box3 &worldbox);
	void RandomizeFaceColor(HWND hwnd);
//Accessors
	int Count() const {return _count;} /*the const means the
		funtion doesn't change the class. I need this because some
		functions that	call (const Polyhedron3 &p) use the Count function.*/
	COLORREF FillColor(int faceindex) const {return _fillcolor[faceindex];}
	Vector3 Centroid() const {return _centroid;}
	Real Radius()const {return _radius;}
	Polyhedron3 operator=(const Polyhedron3 &poly);
//Linear algebra
     friend Polyhedron3 operator*(const Matrix4 &M, const Polyhedron3 &p);
//Methods
 	void _setcentroid(); //helper function.
	void _setfacecentroid(); //helper function.
	void _setfacenormal(); //helper function.

	void SetInternalFace();
	void SetBarycentricCoords();//sets barycentric coordinates of all 16 vertices
	void SetVertInHull();//checks whether V is in Hull of points ABCD
	
	BOOL RealToPixel(Frame const &frame);
//--------------FOUR DIMENSIONAL--------------
	//Simple Parallel/Projective
	friend	void StandardProject(Polytope4  &p, Polyhedron3 &q);


};

class Polytope4
{
private:
//The rats-nest of vertices.
	int _count;
	Vector4 *_vector4vert;
	Vector3 *_vector3vert;
	Vector2 *_vector2vert;
	Point *_pointvert;
	Vector4 _centroid;
	Real _radius;

/*The face info for _facecount faces, each with _facevertcount[i] vertices,
which are stored as index numbers into the rats-nest info.  So _face is
an array of arrays of index ints.*/
	int _facecount;
	int *_facevertcount;
	int **_face;
	COLORREF *_fillcolor;
	Vector4 *_facecentroid;
	Vector4 *_solidnormal;
	BOOL *_facevisible; //Set these with Camera.BackFaceCull(Polyhedron 3).
	Real *_facedistance;
//information about edges:
	int _edgecount;
	int *_vertedgecount;//_vertedgecount[i] is the
						//number of edges going into 
						//_vector4vert[i].  for hypercube all are 4
	Vector4 **_vertedge;//_vertedge[i][j] is the jth edge coming out 
						//of _vector4vert[i].
		
//Extra stuff.
	void _freememory(); //Used by destructor and Size operator.
	friend class Frame;
	friend class Camera;
	friend class Segment;
	friend class Poly3;
public:
//Constructor and Destructor
	Polytope4();
	Polytope4(const Polytope4 &poly);
	~Polytope4();
//Mutators
	void Size(int vertcount, int facecount, int *facevertcount, int edgecount, int *vertedgecount);//Allocates arrays, doesn't preserve old info.
	void SetFillColor(COLORREF col, int faceindex){_fillcolor[faceindex] = col;}
		/*If you don't specify the center, radius, initangle arguments for
		Cube use default arguments */
	void HyperCube(Real radius);
	void HyperHouse(Real radius);
	void Room(const Box4 &worldbox);
	void RandomizeFaceColor(HWND hwnd);
//Accessors
	int Count() const {return _count;} /*the const means the
		funtion doesn't change the class. I need this because some
		functions that	call (const Polyhedron3 &p) use the Count function.*/
	COLORREF FillColor(int faceindex) const {return _fillcolor[faceindex];}
	Vector4 Centroid() const {return _centroid;}
	Real Radius()const {return _radius;}
	Polytope4 operator=(const Polytope4 &poly);
//Linear algebra
     friend Polytope4 operator*(const Matrix5 &M, const Polytope4 &p);
//Methods
 	void _setcentroid(); //helper function.
	void _setfacecentroid(); //helper function.
	void _setsolidnormal(); //helper function.

	BOOL RealToPixel(Frame const &frame);

	 //Simple Parallel/Projective
	 //friend	void StandardProject1(Polytope4  &p, HyperPolyhedron3 &q);
	 friend	void StandardProject(Polytope4  &p, Polyhedron3 &q);


};


#endif //POLYGON_HPP
